;; -*- Mode:Lisp; Syntax:Common-Lisp; Package: FUG5 -*-
;;; -----------------------------------------------------------------------
;;; File:         hnp.l
;;; Description:  system for noun-phrases in Hebrew.
;;; Author:       Yael Dahan Netzer
;;; Created:      30 Mar 1997
;;; Modified:     
;;; Package:      FUG5
;;; -----------------------------------------------------------------------

(in-package "FUG5") 


;;; TODO: words like 'mayim'... should get agreement as plural.
;;; TODO: shloshtam, arb'batam... 
;;; TODO: countries and cities are treated as female.
;;; TODO: take care of superlative of adjectives, some variations are:
;;; haOmanim htovim beyoter, heomanim haxi tovim (done), hatovim shebaOmanim, to
;;; haomanim, tovey haOmanim.
;;; also with pronoun: indefinite pronoun such as - miSehu maSehu.
;;; TODO: define possibilities for gapped nouns, allow quantifiers of (cat
;;; amount), with marking of definiteness, adjectives, numbers... if
;;; quantifier has the feature (nismax yes) 

;; BRANCH FOR SIMPLE-NP AND ITS SPECIALIZATIONS


;; Glinert p. 117 
;; likely order of NP:
;; partitive - det - amount- head - adjective - determiner - adverb/pp -
;; relative clause.
;; shisha me otam tisha parparim levanim me hodu she hesmanta.
(def-conj simple-np
  ;; modifiers  = (describer classifier)
  ;; We expect in the input at the top-level constituents:
  ;; - definite yes/no (default is yes).
  ;; - a lex that will be interpreted as the lex of the head.
  ;; - describer: an embedded list of describers.
  ;; - classifier: an embedded list of classifiers.
  ;; - qualifiers: an embedded list of qualifiers.

  (cat #(under np))
  (complex none)
  (generic-cat np)
  (:! functional-modifiers) ;; file functional.l
;;; common features for all NP types.
  (:& np-propagate) ;;
  
  ;; Distance is not always propagated to determiner
  ;; If lex is already specified, don't do anything.
  (alt np-determiner
       (((determiner ((lex given)))
	 (cset ((- determiner))))
	((determiner ((definite  {^2 definite})
		      (mark-definite {^2 mark-definite})
		      (countable {^2 syntax countable})
		      (number    {^2 syntax number})
		      (gender {^2 syntax gender})
		      (cardinal {^2 cardinal})
		      (fraction {^2 fraction})
		      (multiplier {^2 multiplier})
		      (ordinal {^2 ordinal})
		      (quantitative {^2 reference quantitative})
		      (status {^2 reference status})
		      (degree {^2 reference degree})
		      (exact {^2 reference exact})
		      (comparative {^2 reference comparative})
		      (superlative {^2 reference superlative})
		      (evaluative {^2 reference evaluative})
		      (evaluation {^2 reference evaluation})
		      (total     {^2 reference total})
		      (selective {^2 reference selective})
		      (possessive {^2 reference possessive})
		      (interrogative {^2 reference interrogative})
		      (orientation {^2 reference orientation})
		      (head-denotation {^2 semantics index denotation})
		      (reference-number {^2 semantics index reference-number})
		      (det-type {^2 det-type})
		      (partitive {^2 partitive}))))
	((determiner none))))
  (:! np-type)
  (:! np-number)
  
  ;; DETERMINER
  ;; If possible propagate distance to determiner
  (opt det-distance ((determiner ((distance {^2 syntax distance})))))
  (:! possessor) ;; under semantics? determiner?
  
  (semantics ((:! describer)))
  (semantics ((:! classifier)))
  (semantics ((:! qualifier)))
  (:! np-case)
  
  (determiner ((:& determiners-quantifiers)  )) ;; in file: determiners.l
  
  (:& syntactic-pattern))

;; Propagate all top-level features in an NP under their appropriate group
;; syntax, semantics, reference and head.
(def-conj np-propagate
  (syntax ((fset (animate gender case definite person partitive
			  number distance countable))
	   (animate {^2 animate})
	   (number {^2 number})
	   (gender {^2 gender})
	   (case   {^2 case})
	   (person {^2 person})
	   (definite {^2 definite})
	   (partitive {^2 partitive})
	   (distance {^2 distance})
	   (countable {^2 countable})))
  ;; JR-11-15-92: allowed index at the top-level and then copied it under semantics
  ;; This saves 24 characters per co-reference pointer, making them to fit on a
  ;; single line, e.g. 
  ;; ((circum ((partic ((processor ((index {^5 partic processor index}))))))))
  ;; instead of 
  ;; ((circum ((partic ((processor ((semantics ((index {^5 partic processor
  ;; semantics index})))) 
  (index ((fset (concept definite animate gender person number reference-number 
		 denotation countable))
	  (concept {^2 concept})
	  (animate {^2 animate})
	  (definite {^2 definite})
	  (number {^2 number})
	  (gender {^2 gender})
	  (person {^2 person})
	  (denotation {^2 denotation})
	  (reference-number {^2 reference-number})
	  (countable {^2 countable})))
  (semantics (
	      ;(fset (index describer qualifier classifier))
	      (index ((fset (concept definite animate gender person number
				     reference-number denotation
				     countable))
		      (concept {^3 concept})
		      (definite {^3 definite})
		      (animate {^3 animate})
		      (number {^3 number})
		      (gender {^3 gender})
		      (person {^3 person})
		      (denotation {^3 denotation})
		      (reference-number {^3 reference-number})
		      (countable {^3 countable})))
	      ;(possessor {^2 possessor})
	      ;(possessive-style {^2 possessive-style})
	      (describers-list {^2 describers-list})
	      (describers {^2 describers})
	      (describer {^2 describer})
	      (qualifier {^2 qualifier})
	      (qualifier-list {^2 qualifier-list})
	      (qualifiers {^2 qualifiers})
	      (classifier {^2 classifier})))
  (reference ((fset (type total selective possessive interrogative distance
			  quantitative exact orientation evaluative status
			  degree comparative superlative evaluation))
              (status {^2 status})
	      (degree {^2 degree})
	      (total {^2 total})
	      (selective {^2 selective})
	      (possessive {^2 possessive})
	      (interrogative {^2 interrogative})
	      (distance {^2 distance})
	      (quantitative {^2 quantitative})
	      (exact {^2 exact})
	      (comparative {^2 comparative})
	      (superlative {^2 superlative})
	      (evaluation {^2 evaluation})
	      (orientation {^2 orientation})
	      (evaluative {^2 evaluative})))
  (head ((lex {^2 lex}))))



(def-alt np-type (:index cat)
  (:demo "Is this a common noun, a pronoun or a proper noun?")
  (((cat common)
    (np-type common)
    ;; surge
    (:! relational-nouns)
    (alt common-determiner
	 (((determiner ((head-cat common))))
	  ((determiner none))))
    (head ((cat np-head)
	   (synt-funct head)
	   (alt common-head (:index cat)
		(((lex given)
		  (cat noun)
		  ;; all these features affect morphology
		  (definite {^2 definite})
		  (animate {^2 syntax animate})
		  (gender {^2 syntax gender})
		  (case {^2 syntax case})
		  (number {^2 syntax number}))
		 ((cat noun-compound)
                  (nismax no)
		  (number {^2 number})
		  (definite {^2 definite})
		  (gap none))
		 ((gap given))
		 ((complex given)
		  (cat np-head)
		  (number {^2 syntax number})
		  (definite {^2 syntax definite})
		  (common ((cat noun)
			   ;; conjunction gender: if any masculine, then
			   ;; gender is masculine, otherwise, feminine.
			   (nismax {^2 nismax}))))))))
   
    (definite  {^ syntax definite})
    (person third)
    ;; Hebrew definite - make default (definite no)
    (countable ((alt countable (yes #(under no))))))
	  
   ;; Pronouns----------------
   ((cat pronoun)
    (semantics ((classifier none)))
    (somex none)
    ;; pronouns can have qualifiers.
    (alt pronoun-determiner
	 (((determiner ((head-cat pronoun))))
	  ;; what is the determiner for pronouns?
	  ((determiner none))))
    (cset ((- head))) ;; do not recurse on head
    (head ((cat pronoun)
	   (gender {^2 syntax gender})
	   (number {^2 syntax number})
	   (animate {^2 syntax animate})
	   (case {^2 syntax case})
	   (pronoun-type {^2 pronoun-type})
	   (distance {^2 syntax distance})
	   (person {^2 syntax person})
	   (restrictive {^2 restrictive})))
    
    (:! pronoun-types)
    (syntax
     ((alt pronoun-case (:wait case)
	   (:demo "Is the pronoun subject, object or reflexive?")
	   (((case subjective))
	    ((case given)
	     (case ((alt (objective reflexive))))))))))
   ;; partitive -- kuli... 
    
    ;; Proper Nouns-----------------------
   ((cat proper)
    (np-type proper)
    (syntax ((person third)
	     (definite yes)))
    (somex none)
    (determiner ((head-cat proper)))
    ;; what about: moshe haHamud..
    (alt proper-type
	 (((cat basic-proper)
	   (qualifier none)
	   (classifier none)
	   ;; what determiners can proper get?
	   (head ((cat noun)
		  (lex given)
		  (nismax no)
		  (number {^2 syntax number})
		  (gender {^2 gender})
		  (alt proper-definite-mark
		       ((({^ number} singular)
			 (mark-definite no))
			(({^2 number} plural)
			 (mark-definite yes)))))))
	  ((cat compound-proper)
	   (classifier none)
	   (head ((cat #(under person-name))
		  (gender {^2 gender})
		  (number {^2 number})))))))))
	;; qualifiers for proper?

;;;;;;; pronoun type

(def-alt pronoun-types 
  (:index cat)
  (:demo "Is this a personal, demonstrative, question or quantified pronoun?")
  (((cat personal-pronoun)
    (np-type personal-pronoun)
    (definite yes)
    (head ((definite yes)
	   (pronoun-type personal)
	   (mark-definite no)))
    ;; default is third masculine.
    (syntax 
     ((alt person-animate
	   (:index person) (:wait person)
	   (((person third) (animate ((alt (yes no)))))
	    ((person #(under first)) (animate yes))
	    ((person #(under second)) (animate yes))))
      (alt gender (:index gender)
	   ;; neuter in Hebrew??
	   (((gender masculine))
	    ((gender #(under feminine)))
	    ((gender neuter)))))) ;; ze yafe
    (semantics ((describer none)
		(alt (((qualifier given)
		       (qualifier ((restrictive no))))
		      ((qualifier none)))))))
   ((cat demonstrative-pronoun)
    (np-type demonstrative-pronoun)
    (head ((pronoun-type demonstrative)))
    (determiner ((distance none)))
    (syntax ((definite yes)
	     (person third)
	     (distance ((alt (far #(under near)))))))
    (alt (((syntax ((distance far)))
           (head ((mark-definite yes))))
          ((head ((mark-definite no))))))
    (semantics ((describer none))))
   ((cat relative-pronoun)
    (np-type relative-pronoun)
    (determiner ((distance none)))
    (head ((pronoun-type relative)))
    (syntax ((person third))))
;   ((cat question-pronoun)
;    (np-type question-pronoun)
;    (determiner ((distance none)))
;    (head ((pronoun-type question)))
;    (syntax ((person third)))
;    (semantics ((describer none))))
 ;; - describers come after head: "something green"
   ;; Hebrew: mishehu.. 
  ((cat quantified-pronoun)
   (np-type quantified-pronoun)
   (head ((pronoun-type quantified)))
   (syntax ((person third) (definite no)))
   (alt (((semantics ((describer none))))
	 ((semantics ((describer given)))
	  (pattern (head describer dots))))))
   ))


;;;;;;;;; np-number
(def-alt np-number
  (((cardinal ((value given)))
    (control (and (numberp #@{^ cardinal value})
		  (= #@{^ cardinal value} 1)))
    (syntax ((number singular))))
   ((fraction ((num given)))
    (control (and (numberp #@{^ fraction num})
		  (= #@{^ fraction num} 1)))
    (syntax ((number singular))))	       
   ((cardinal ((value given)))
    (control (numberp #@{^ cardinal value}))
    (syntax ((number plural))))
   ((fraction ((num  given)))
    (control (numberp  #@{^ fraction num}))
    (syntax ((number plural)
	     (countable yes))))
   ((syntax ((number singular)
	     (countable ((alt (yes no)))))))
   ((syntax ((number #(under dual))
	     ;; how do I handle: ofana'im, mixnasa'im...??
	     (reference-number ((alt (single dual)))))))
   ((syntax ((number #(under plural)))))))
;;; TODO : yeled eHad - ha yeled ha eHad...?

;; possessor will be as following:
(def-alt possessor
  (:index possessive-style)
  (:demo "Is there a possessor?")
  (((possessor none))
   ((possessor given)
    
    (possessor ((cat np)
		(synt-funct possessor)
                (possessive-style {^2 possessive-style})))
    ;; gur Hatulim - (possessive-style construct/classifier) - same definite 
    ;; gur shel Hatulim (possessive-style pp/qualifier)
    ;; guram shel ha Hatulim (possessive-style double-genitive) - definite
    ;; head, no constraint on possessor.
    (alt possessive-style 
	 (((possessive-style ((alt (construct classifier)))) ;; use
							     ;; functional styles.
	   ;({^ definite} {^ possessor definite})
	   (head ((nismax yes)))
	   (distance none)    ;; according to Glinert
	   (possessor ((alt (((cat common))
			     ((cat #(under pronoun))
                              (definite yes)
                              )))))
	   (possessor ((synt-funct possessor)))
	   (somex {^ possessor}))
	  ((possessive-style ((alt (pp qualifier))))
	   (shel-phrase
	    ((cat pp)
		  (prep ((lex "Sel")))
		  (np {^2 possessor}))))
	  ((possessive-style double-genitive)
           (definite yes)
	   (somex ((cat personal-pronoun)
                   (syntax ((case objective)))
                   (gender {^2  possessor gender})
                   (number {^2  possessor number})
                   (person {^2  possessor person})))
	   
	   (shel-phrase
	    ((cat pp)
	     (prep ((lex "Sel")))
	     (np {^2 possessor})))))))))
   
#+ignore(def-alt describer
  (:demo "Are there describers?")
  ;; UNDER NP (SEMANTIC X)
  (((describer none))
   ((describer given)
    (describer 
     ((number {^3 syntax number})
      (gender {^3 syntax gender})
      (definite {^3 syntax definite})
      (mark-definite {^3 mark-definite})
      (alt describer-cat
	   (((cat ap))
	     ;;(qualifier none))- surge. WHY?
	    ((cat adj)
	     (definite no)
	     (modifier none)))))))))
     ;;; when post-determiner exists such as: ha-ze, eHad
     ;;; describer can come before or after it:
     ;;; ha-yeled ha-ze ha-Hadash - ha yeled ha-Hadash ha-ze.
     ;;; there should also be a possibility for the
     ;;; construction:
     ;; yofi shel yeled (definite no)?


(def-alt classifier (:demo "Is there a classifier?")
  (((classifier none))
   ((classifier   ;; np
     ((alt classifier-cat (:index cat) ;; classifier cannot be a pronoun
           (((cat common))
            ((cat proper))
            ((cat np-head))))
      (synt-funct classifier)))
    ({^ head} ((nismax yes)))
    ({^ classifier} {^ somex}))
	     ;; siHa telefonit, beHira parlamentarit, hafala yadanit
	     ;; all can be paraphrased to: siHat telefon, hafalat yad...
	     ;; adjectives are usualy noun-based.
   ((classifier ((alt (((cat ap))
                       ((cat adj)
                        (definite no))))
		 (synt-funct classifier)
		 (gender {^3 syntax gender})
		 (number {^3 syntax number})
		 (definite {^3 syntax definite})
		 (mark-definite {^3 mark-definite})))
    ({^ classifier} {^ nca-describer}))))

(def-alt qualifier (:demo "Is there a qualifier? Is it a PP or a clause?")
  (((qualifier none))
   ((qualifier given)
    (qualifier ((synt-function qualifier)
		(alt qualifier-cat (:index cat) 
		 (((cat pp)
		   (restrictive ((alt (yes no))))
		   (np ((syntax {^2 syntax})
			(semantics {^2 semantics})
			(lex {^2 lex}))))
		  ((cat list))
		  ((cat clause))))
		
		  (alt qualifier-restrictive (:index restrictive)
		       (((restrictive yes)
			 (punctuation ((before none))))
			((restrictive no)
			 (punctuation ((before ",") (after ",")))))))))))

(def-alt np-case (:index (syntax case))
  (:wait case) ;;possessor might be a pp and therefore case of noun
               ;; can be determined only after the realization of the 
               ;; possessor is determined.
  (((syntax ((case subjective))))
   ((syntax ((case given)))
    (syntax ((case ((alt (objective reflexive)))))))
   ((syntax ((case #(under possessive)))))))
				

;;;;;;;;
;; Pattern of NP
;; ==============================================================
;; Glinert p. 117 
;; likely order of NP:
;; partitive - det - amount- head - adjective - determiner - adverb/pp -
;; relative clause.
;; shisha me otam tisha parparim levanim me hodu she hesmanta.

(def-conj syntactic-pattern
;;  (tentative-pattern (partitive det amount thing adjective amount det
;;				possessor-phrase qualifier))
;; partitive = qunat (prep)
;; thing = (det) head
  
  (pattern (partitive-phrase
            indef-art
	    pre-determiner 
	    pre-quantifier
	    thing
	    somex
            nca-describer
  ;; yd describer 18/12
	    describers
	    post-quantifier
	    post-determiner
	    shel-phrases
	    qualifiers))


		
 (alt (((mark-definite given))
       ((mark-definite {^ definite}))))

  (alt somex 
       (((somex none))
        ((head ((cat #(under noun-compound))))
         (somex none))
        ((somex given)
         (head ((cat noun)
		(nismax yes)
                (mark-definite no)
		(alt (((complex none))
		      ((conmplex conjunction))))))
         (definite {^ somex definite})
	 (np-type common) ;; is somex is given, np-type cannot be proper or
	 (somex ((case objective)
	         (alt somex-cat
                      (((cat np)
			(mark-definite {^2 mark-definite}))
                       ((cat noun-compound))
                       ((cat noun)))))))))


  

    (alt definite (:index definite) (:wait {^ somex definite})
	 (:demo "Determine definite-mark-definite of NP")
	 (((definite no)
	   (mark-definite no)
	   (head ((mark-definite no))))
          ((definite #(under indef))
           (head ((mark-definite no)))
           (mark-definite ((alt (indef no)))))
	  ((definite yes)
	   (alt mark-definite 
		(((mark-definite #(under no))   ;; for instances like: yeled ze..
		  (head ((mark-definite no))))
		 ((denotation #(under zero-article-thing))
		  (head ((mark-definite no))))
		 ((mark-definite yes)
		  (head ((alt is-head-nismax (:wait nismax)
			      (:wait mark-definite)
			      (((nismax no)
				(definite yes)
				(mark-definite {^2 mark-definite}))
			       ((nismax yes)
				(mark-definite no))
			       ((mark-definite no))))))))))))
			
  (alt thing (:wait definite)
       (((head ((cat noun)
		(nismax no)
		(gap none)
		(mark-definite {^2 mark-definite})
		(mark-definite #(under yes))))
	 (thing ((cat phrase)
                 (pattern (det {^2 head}))
                 (det ((cat article)
                       (definite yes))))))
        ((thing {^ head}))))

  (alt shel-phrase
       (((shel-phrase none)
	 (shel-phrases none))
	(
	 (shel-phrase ((cat pp) (prep ((lex "Sel")))))
	 (shel-phrases {^ shel-phrase}))
	((shel-phrases ((cat phrase)
			(pattern ({^2 shel-phrase} shel-phrase1 shel-phrase2
				      shel-phrase3)))))))
  
  ;(alt qualifiers
  ;     (((qualifiers none)
;	 (qualifier none))
;	((qualifiers none)
;	 (qualifiers {^ qualifier}))
;	((qualifiers given)
;	 (qualifiers ((cat phrase)
;			(pattern ({^2 qualifier} qualifier1 qualifier2 qualifier3 
;				      qualifier4)))))))
  
   (alt pre-post-quantifiers
        (((determiner ((quantitative no)))
          (ordinal none))
         ((alt amount-quantifier-type
               (((reference ((quantitative no)))
                 (amount-quantifier none))
                ((cardinal given)
                 (alt (((partitive no))
                       ((total +))
                       ((fraction given) (definite yes))))
                 (alt (((control (/= #@{^ cardinal value} 1)))
                       ((cardinal ((digit yes))))))
                 (pre-quantifier {^ determiner cardinal}))
                ((fraction given)
                 (partitive no)
                 (pre-quantifier {^ determiner fraction}))
                ((multiplier given)
                 (partitive no)
                 (pre-quantifier {^ determiner multiplier}))
                ((cardinal none)
                 (determiner ((partitive no)
                              (pre-quantifier given)))
                 (pre-quantifier {^ determiner pre-quantifier}))
                ((pre-quantifier none))))
          
  	       
          (alt post-quantifier
               (((ordinal given)
                 ;(determiner ((realize-ordinal given)))
                 (post-quantifier {^ ordinal}))
                ((reference ((quantitative no))))
                ((cardinal given)
                 (alt (((partitive no))
                       ((fraction given))))
                 (cardinal ((value 1)
                            (digit no)))
                 (post-quantifier {^ cardinal}))
                ((determiner ((post-quantifier given)))
                 ;(partitive no)
                 (post-quantifier {^ determiner post-quantifier}))
		
		((post-quantifier none)))))))
   
   (alt pre-post-determiners
	(((determiner ((determiners none))))
	 ((alt pre-det-in-pattern
	       (((pre-det given)
		 (partitive-phrase none)
		 (amount-quantifier none)
		 (pre-determiner {^  pre-det}))
		((determiner ((pre-det given)))
		 (pre-determiner {^ determiner pre-det}))
		((pre-determiner none))))  ;; make position of determiner be under det etc..

	  (alt post-determiner
	       (((determiner ((post-det none)))
		 (determiner ((deictic none)))
		 (post-determiner none))
		((alt (((determiner ((post-det given))))
		       ((determiner ((deictic given))))))
		 (post-determiner ((cat phrase)
				   (pattern (deictic post-det))
				   (post-det {^2 determiner post-det})
				   (deictic {^2 determiner deictic})))))))))
   
 
  (alt partitive-phrase
       (((reference ((quantitative no)))
	 (partitive no)
	 (partitive-phrase none))
	((partitive no)
	 (partitive-phrase none))
	((determiner ((pre-quantifier given)
		      (pre-quantifier ((cat quant-partitive)))))
	 (partitive-phrase {^ determiner pre-quantifier}))

	(
	 ({^ partitive-phrase} {^ determiner partitive-phrase})

	 (head ((case objective)))
	 (partitive-phrase
	  (
	   (definite {^2 syntax definite})
	   (definite yes)
	   (partitive {^2 partitive})
	   (partitive yes)
	   (opt (({^2 cardinal} given)
		 (type cardinal)))
	   (alt ((({^2 fraction} given)
		  (type fraction))
		 (({^2 multiplier} given)
		  (type multiplier))
		 (({^2 multiplier} none)
		  ({^2 fraction} none))))
	   (alt (((type given)
		  (alt total-type (:index type)
		       (((type total)
			 (total +)
			 (part ((cat phrase)
				(lex "cl")))
			 (pattern-type part-only))
			((type total)
			 (total -)
			 (alt Sum-or-AF
			      ((({^2 countable} yes)
				(gender masculine)
				(part ((cat phrase)
				       (lex "Af EHad"))))
			       (({^2 countable} yes)
				(gender feminine)
				(part ((cat phrase)
				       (lex "Af AHat"))))
			       
			       (({^2 countable} no)
				(part ((cat phrase)
				       (lex "Sum davar"))))))
			 (pattern-type partitive-phrase))
			((type cardinal)
			 (pattern-type partitive-phrase)
			 (part {^2 cardinal})
			 (part ((nismax no))))
			((type fraction)
			 (pattern-type partitive-phrase)
			 (part ((cat fraction)))
			 (part {^2 fraction}))
			((type multiplier)
			 (pattern-type partitive-phrase)
			 (part {^2 multiplier}))
			
			((type amount)
			 (pattern-type partitive-phrase))
			((type partitive)
			 (pattern-type part-only)))))
		 ((type amount)
                  (pattern-type partitive-phrase))))
	   
	   (part ((alt part-cat
		       (((cat phrase)
			 (lex {^2 lex})
			 (alt part-default-lex
			      (((lex given))
			       ((lex "Helek")))))
 ;;***CAT ABOVE: you can also say "HALAKIM" I think its noun!!
                        ;; but if its plural, thats the head of the NP!!!
                        ((cat article))
			((cat cardinal))
			((cat fraction))))))
					  
	   (alt pattern-type
		(((pattern-type partitive-phrase)
		  (prep ((cat prep)
			 (lex ((alt ("m" "mn"))))))
		  (pattern (part prep)))
		 ((pattern-type part-only)
		  (pattern (part)))))))))))

;; cat quantifier: can stand alone.
;; cat det: cannot.

;; word    alone     inflection      smixut    definite   partitive
;;         (gapped)  (with pronouns) (with noun) (alone)   (when definite)
;; cardinal +           +              +         +           +
;; yoter    +           -              ?         -           - (yoter
;;                                                              m.. is comp..)
;; yoter                                                    
;; fractions +       only 1/2         some       +           +
;; me'at    +           -              +         +           +
;; cama     +           -              -         -           +
;; Helek    +           +              -         -           +
;; bakbuk.. +      not as measure      +         +           +
;; dunam..  +           -              +         +           +
;; marbit.. -           +              +         -           -
;; rov      +           +              +         +           -
;; ikar     +           -              +         +           -
;; she'ar   +           -              +         +           -
;; meitav   +           +              +         +           -
;; col      +           +              +         +           -
;; oto      -           +              +         -           -
;;

;; ============================================================
(provide "hnp")
;; ============================================================


  

