(require 'pretty-print)
(require 'common-list-functions)
(require 'i/o-extensions)

(define (get-env-var var-name)
  (let ((var-value (getenv var-name)))
    (if (not var-value)
        (error (string-append "The " var-value " environment variable must be set!")))
    var-value))

(define *cfuf-toplevel-path* (get-env-var "CFUF_TOPLEVEL_PATH"))
(define *surge-linearizer-path* (get-env-var "SURGE_LINEARIZER_PATH"))

(define (fail-load)
  (error (string-append "Usage: runfuf -l runfuf.scm [fd-file gr-file [trackbugs] [quick | pretty | flat] [wait] [ulimit n] [trace n]]")))

(define nargs (length *argv*))
(define *load-and-run* #t)
(if *interactive*
    (set! *load-and-run* #f)
    (begin
     (set! nargs (- nargs 3))
     (if (< nargs 0)
         (fail-load)
         (set! *argv* (cdddr *argv*)))))

(define fd-file #f)
(define gr-file #f)

;;; Can be one of '*LINEARIZE*,'*PRINT-LINEARIZE*,'*PPRINT-FD*,'*FD-TO-LIST-PRINT*,'*FD-TO-LIST*,'*NONE*
(define *print-action* '*NONE*)

(define *is-bugtracking* #f)
(define *is-waiting* #f)
(define *trace-level* #f)
(define *ulimit* 2500)

(if *load-and-run*
    (let ((get-numeric (lambda (args)
                         (if (null? (cdr args)) (fail-load))
                         (string->number (cadr args)))))
      (if (< nargs 2) (fail-load))
      (set! fd-file (car *argv*))
      (set! gr-file (cadr *argv*))
      (if (not (access fd-file "r"))
          (error (string-append "fd-file " fd-file " must be readable")))
      (if (not (access gr-file "r"))
          (error (string-append "gr-file " gr-file " must be readable")))
      (do ((args (cddr *argv*) (cdr args)))
          ((null? args) 'done)
        (let ((sy-arg (string->symbol (car args))))
          (case sy-arg
            ('trackbugs (set! *is-bugtracking* #t))
            ('wait (set! *is-waiting* #t))
            ('quick (set! *print-action* '*FD-TO-LIST-PRINT*))
            ('pretty (set! *print-action* '*PPRINT-FD*))
            ('flat (set! *print-action* '*PRINT-LINEARIZE*))
            ('ulimit (set! *ulimit* (get-numeric args))
                     (set! args (cdr args)))
            ('trace (set! *ulimit* (get-numeric args))
                    (set! *trace-level* (cdr args)))
            (else (fail-load)))))))

(define (get-abs-dir dirname)
  (let ((cwd (getcwd))
        (abs-dir #f))
    (if (not (chdir dirname))
        (error (string-append "Failed to chdir " dirname)))
    (set! abs-dir (getcwd))
    (chdir cwd)
    abs-dir))

(define (dirname file)
  (let* ((tmp (member #\/ (reverse (string->list file))))
         (res (if tmp
                  (get-abs-dir (list->string (reverse tmp)))
                  (getcwd))))
    (string-append res "/")))

;; (define control-demo print)
(define control-demo (lambda args #t))

(define cfd #f)
(define cgr #f)

(define linearize
  (let ((loaded #f))
    (lambda ()
      (if (cfuf:fd? cfd)
        (begin
          (if (not loaded)
            (let ((cwd (getcwd)))
              (chdir *cfuf-toplevel-path*)
              (load "base.scm")
              (verbose 0)
              (chdir *surge-linearizer-path*)
              (load "root.scm")
              (chdir cwd)
              (set! loaded #t)))
          (print-sentence (call-linearizer cfd)))))))

(define (write-to-file)
  (case *print-action*
    ('*NONE* #t)
    ('*LINEARIZE* (print (linearize)))
    ('*PRINT-LINEARIZE* (call-with-output-file "=u.txt" (lambda (port) (print port (linearize)))))
    ('*PPRINT-FD* (cfuf:file-pprint-fd cfd 'fd "_u.scm"))
    ('*FD-TO-LIST-PRINT* (call-with-output-file "_u.scm"
                                                (lambda (port)
                                                  (print port (list 'define 'fd (string->symbol "'") (cfuf:fd->list cfd))))))
    ('*FD-TO-LIST* (cfuf:fd->list cfd))))

(define (xy-unify)
  (let ((cont #t))
    (set! cfd (cfuf:compile-fd fd))
    (set! cgr (cfuf:compile-grammar gr))
    (set! cont (and cfd cgr))
    (or cont (print "Failed to compile!"))
    (set! cont (and cont (cfuf:unify-fd! cfd cgr)))
    (or cont (print "Failed to unify!"))
    (and cont (write-to-file))))

(if *load-and-run*
  (begin
   (cfuf:doing-wait *is-waiting*)
   (cfuf:ulimit *ulimit*)
   (if *trace-level*
       (begin
        (cfuf:silent #f)
        (cfuf:trace-level *trace-level*)))
   (if *is-bugtracking*
       (begin
        (load (string-append (dirname fd-file) "x.scm"))
        (load (string-append (dirname fd-file) "y.scm")))
       (begin
        (load fd-file)
        (load gr-file)))
   (xy-unify)))
