;; yael 2/2/99
;; ========================================================
;; make a macro that gets a list of morph-functions and arguments+defualts, and builds the following procedures.
;; then make a procedure like - init linearizer.
;;==========================================================
;; language-dependent-morpohlogy

;;==========================================================
(define (MORPH-NOUN word number a-an feature) 
  "If feature is possessive, then return the apostrephised form of the noun
   appropriate to the number.
   If a-an is 'an mark noun with mark-noun-as-an to agree with determiner.
   If noun is plural and ends with 's', mark noun as plural to let it agree
   with possessive mark that may follow it (single apostrophe or 's).
   Return word with number suffix." 
  (set! word (morph-number word number)) 
  (if (and (subsume 'plural number)
	   (equal? (string-ref word (1- (string-length word))) #\s))
      (set! word (mark-noun-as-plural word)))
  (if (equal? a-an 'an) (mark-noun-as-an word) word))

(define (MORPH-NUMBER word number) 
  "Adds the plural suffix to word if number is plural
   note that the default is singular." 
  (cond ((or (null? number) (not number)) word)
	((not word) word)
        ((subsume 'plural number)
         (let ((lex (lexfetch word 'plural)))
	       (if (and (not (null? lex)) lex)
		   lex
		   (pluralize word))))
        (else word)))


(define (MORPH-PRONOUN lex pronoun-type case gender number
                          distance animate person restrictive)
  "Returns the correct pronoun given the features."
  (cond ((and 
              (string? lex)
              (not (string=? lex "#f"))
              (not (string=? lex "nil"))
              (not (string=? lex "none"))) lex)
        ((eq? pronoun-type 'personal)
         ;; start with "he" then augment by person, then by
         ;; number, gender and finally by case.
         ;; yael: hack
         (when (or (not animate) (eq? gender 'feminine)) 
             (set! animate 'yes))
         (when (eq? animate 'no) (set! gender 'neuter))
         (set! lex "he")
         (unless (eq? person 'third) 
		 (set! lex (fetch-or-not lex person)))
         (unless (eq? number 'singular) 
		 (set! lex (fetch-or-not lex number)))
         (unless (eq? gender 'masculine) 
		 (set! lex (fetch-or-not lex gender)))
         (unless (eq? case 'subjective) 
		 (set! lex (fetch-or-not lex case)))
         lex)
        ((eq? pronoun-type 'demonstrative)
         (cond ((subsume 'plural number)
                (if (eq? distance 'far) "those" "these"))
               ((eq? distance 'far) "that")
               (else "this")))
        ((eq? pronoun-type 'relative)
         (when (not animate) (set! animate 'no))
	 (when (eq? animate 'no) (set! gender 'neuter))
         (cond ((eq? restrictive 'yes) "that")
               ((eq? case 'possessive) "whose")
               ((eq? case 'objective) 
                (if (eq? gender 'neuter) "which" "whom"))  
               ((eq? gender 'neuter) "which")
               ((eq? animate 'no) "which")
               (else "who")))
        ((eq? pronoun-type 'question)
         ;;hack
         (when (not animate) (set! animate 'no))
         (when (eq? animate 'no) (set! gender 'neuter))
         (cond ((eq? restrictive 'yes) "which")
               ((eq? case 'possessive) "whose")
               ((eq? case 'objective) 
                (if (eq? gender 'neuter) "what" "whom"))
               ((eq? gender 'neuter) "what")
               ((eq? animate 'no) "what")
               (else "who")))
        ((eq? pronoun-type 'quantified)
         lex)))

(define (MORPH-ADJ word superlative comparative inflection)
  (let ((ending  (if (eq? superlative 'no)
                      (if (eq? comparative 'no) '() 'comparative)
                      'superlative)))
    (if (or (eq? inflection 'no) (null? ending)) word
	(let ((lex (lexfetch word ending)))
	  (cond ((and (not (null? lex)) lex) lex)
		((eq? ending 'superlative) (form-adj word "est"))
		(else (form-adj word "er")))))))


(define (FORM-ADJ word ending)
  "changes in spelling:
   1. final base consonants are doubled when preceding vowel is stressed and
      spelled with a single letter: big-bigger-biggest; sad-sadder-saddest.
   2. bases ending in a consonant+y, final y changed to i
      angry-angrier-angriest.
   3. base ends in a mute -e, dropped before inflectional suffix:
      pure-purer-purest
      free-freer-freest"
   (let ((final (last-char word)))
     (case final
             ((#\e) (string-append (butlast-char word) ending))
             ((#\b #\d #\g #\m #\n #\p #\t)
              (if (and (vowel? (string-ref word (- (length word) 2)))
		       (not (vowel? (string-ref word (- (length word) 3)))))
	          (string-append word (make-string 1 final) ending)
	        (string-append word ending)))
             ((#\y) (if (vowel? (elt word (- (length word) 2)))
                      (string-append word ending)
                    (string-append (butlast-char word) "i" ending)))
             (else (string-append word ending)))))

(define (MORPH-VERB word ending number person tense) 
;  "Adds the proper suffix to the verb root taking into account
;   ending, number, person, and tense." 
  (cond ((eq? ending 'root) word)
        ((eq? ending 'infinitive) (list "to" word)) 
        ((equal? word "be") 
	 (morph-be number person 
		   (or (if (equal? ending '*internal*) #f ending)
		       tense))) 
        ((eq? ending 'present-participle)
         (let ((lex (lexfetch word 'present-participle)))
               (if (and lex (not (null? lex))) lex (form-ing word))))
        ((eq? ending 'past-participle) 
	 (let  ((lex (lexfetch word 'past-participle)))
	   (if (and lex (not (null? lex))) lex (form-past word))))
        ((let ((lex (lexfetch word (if 
				(and (eq? tense 'present)
                            	     (eq? person 'third)
                            	     (eq? number 'singular))
                       'present-third-person-singular
                       tense)))) (if (and lex (not (null? lex))) lex #f)))
        ((eq? tense 'present) (form-present-verb word number person))
        ((eq? tense 'past) (form-past word)) 
        (else nil)))

(define (MORPH-BE number person tense)
  (case tense
    ((present) (case number 
	       ((singular) (case person
			   ((first) "am")
			   ((second) "are")
			   ((third) "is"))) 
	       (else  "are"))) 
    ((past) (case number
	    ((singular) (case person
			((first) "was")
			((second) "were")
			((third) "was")))
	    (else "were")))
    ((present-participle) "being")
    ((past-participle) "been")))

(define (FORM-PRESENT-VERB word number person) 
  "Forms the suffix for the present tense of the verb WORD." 
  (when word
    (case person
      ((first) word)
      ((second) word)
      ((third) (case number
	       ((singular) (pluralize word))
	       ((plural) word)
	       ((dual) word)
	       ((not-one) word)
	       (else nil)))
      (else nil))))


(define (FORM-PAST word) 
  "Form past tense by adding 'ed to word,
  handles duplication of final consonant and special cases. " 
  (let ((char (string-ref word (1- (string-length word)))))
    (case char
      ((#\e) (string-append word "d")) 
      ((#\y) (if (vowel? (string-ref word (- (string-length word) 2)))
	       (string-append word "ed")
	     (string-append (substring word 0 (1- (string-length word))) 
			  "ied")))
      ((#\r) (if (char=? (string-ref word (- (string-length word) 2)) #\a)
	       (string-append word (make-string 1 char) "ed")
	     (string-append word "ed")))
      ((#\b #\d #\g #\m #\n #\p #\t)
       (if (and (vowel? (string-ref word (- (string-length word) 2)))
		(not (vowel? (string-ref word (- (string-length word) 3)))))
	   (string-append word (make-string 1 char) "ed")
	 (string-append word "ed")))
      (else (string-append word "ed")))))


(define (FORM-ING word)
;  "Add 'ing to word dropping final 'e if any,
;  handles duplication of final consonant and special cases."
  (let ((char (string-ref word (1- (string-length word)))))
    (case char
      ((#\e) (string-append (substring word 0 (1- (string-length word))) "ing")) 
      ((#\r) (if (char=? (string-ref word (- (string-length word) 2)) #\a)
	       (string-append word (make-string 1 char) "ing")
	     (string-append word "ing")))
      ((#\b #\d #\g #\m #\n #\p #\t)
       (if (and (vowel? (string-ref word (- (string-length word) 2)))
		(not (vowel? (string-ref word (- (string-length word) 3)))))
	   (string-append word (make-string 1 char) "ing")
	 (string-append word "ing")))
      (else (string-append word "ing")))))

(define (MORPH-DETERMINER word number)
  "Returns the correct article for the given type.
       DOES NOT CHECK for next word after indefinite - this is
       done in PUNCTUATE, but here just put special mark
       *indefinite-article* to indicate something is to be done."
  (cond ((equal? word "a")
         (cond ((subsume 'plural number) nil)
	       (else *indefinite-article*)))
        (else word)))

(define (format-ordinal value)
  ;; Integer to string of the form "nth" 
  ;; 1 -> st except for 11
  ;; 2 -> nd except for 12
  ;; 3 -> rd except for 13
  ;; otw th
  (let* ((lasttwo (modulo value 100))
	 (lastone (modulo value 10))
	 (ext (cond ((member lasttwo '(11 12 13)) "th")
		    ((= lastone 1) "st")
		    ((= lastone 2) "nd")
		    ((= lastone 3) "rd")
		    (else "th"))))
    (format nil "~s~a" value ext)))
	 

(define (MORPH-NUMERIC lex ord-or-card value digit)
  "Return a string given a number - either ordinal or cardinal"
  ;; Lucky common-lisp does all for us!!!
  (cond ((and (string? lex)
	      (not (number? value))     ;; if value is given use it
	      (not (string=? lex "nil"))
	      (not (string=? lex "none"))) lex)
	((integer? value)
	 (case digit
	   ((no) (case ord-or-card
		 ((ordinal) (format nil "~:r" value))
		 ((cardinal) (format nil "~r" value))
		 (else (format nil "~r" value))))
	   ((roman) (format nil "~@R" value))
	   (else  (case ord-or-card
		 ((ordinal) (format-ordinal value))
		 (else (format nil "~s" value))))))
	((number? value)
	 (format nil "~s" value))
	((string? value) value)
	(else (format nil "<unknown-number: ~s>" value))))
	 

(define (MORPH-FRACTION lex num den digit)
  "Return a string for a fraction"
  (cond ((and (integer? num) (integer? den))
	 (cond ((= den 1) 
		(case digit 
		  ((no) (format nil "~r" num))
		  ((roman) (format nil "~@R" num))
		  (else (format nil "~s" num))))
	       ((and (= num 1) (= den 2))
		(case digit
		  ((no) "half")
		  (else "1/2")))
	       ((= den 2)
		(case digit
		  ((no) "~r halves" num)
		  (else "~s/2" num)))
	       (else
		(case digit
		  ((no) (format nil "~r-~:r~P" num den num))
		  (else (format nil "~s/~s" num den))))))
	((and (string? lex)
	      (not (string=? lex "nil"))
	      (not (string=? lex "none"))) lex)
	(else (format nil "~s/~s" num den))))


;; ------------------------------------------------------------
;; Pluralize (written by Jay Meyer)
;; ------------------------------------------------------------

(define (PLURALIZE word-string)
  "Handles word-string ending in ch, sh, o, s, x, y, and z 
  as well as regular forms."
  (let* ((n (string-length word-string))
         (next-to-final (string-ref word-string (- n 2)))
         (final (string-ref word-string (1- n))))
    (case final
          ((#\s #\z #\x) (string-append word-string "es"))
          ((#\h) (if (member next-to-final '(#\c #\s))
                   (string-append word-string "es")
                   (string-append word-string "s")))
          ((#\o) (if (not (vowel? next-to-final))
                   (string-append word-string "es")
                   (string-append word-string "s")))
          ((#\y) (if (not (vowel? next-to-final))
                   (string-append (substring word-string 0 (1- n)) "ies")
                   (string-append word-string "s"))) 
          (else (string-append word-string "s")))))

;;==========================================================

;; ------------------------------------------------------------
;; Checks: verify validity of features influencing morphology 
;; and chooses default values.
;; ------------------------------------------------------------

(define (check-or-get-default feature value)
  (let ((registered (get-registered-values feature)))
    (cond ((equal? feature 'cat) 
	   (normalize-cat value))
	   ((not registered) value)
	   ((member value registered) value)
	   (else (get-default-from-registered registered)))))

(define (get-registered-values feature)
  (let ((features (safe-assoc feature *registered-feature-list*)))
	      (unless (null? features) (cdr features))))

(define (get-default-from-registered registered-values) (car
							 registered-values))

(define (init-registered-value-list)
 (map (lambda (val) 
	(store-registered-value (car val) (cdr val))) *flist*))

(define *registered-feature-list* '())
(define (store-registered-value feature list-of-values)
  (set! *registered-feature-list* (cons (cons feature list-of-values) *registered-feature-list*)))


(define *flist*
  (list '(pronoun-type personal demonstrative relative question quantified)
	'(person third second first)
	'(tense present past)
	'(number singular plural dual not-one)
	'(case subjective objective possessive reflexive)
	'(gender masculine feminine neutral)
	'(distance near far)
	'(animate no yes)
	'(a-an consonant an)
	'(restrictive no yes)
	'(superlative no yes)
	'(comparative no yes)
	'(inflected no yes)))

(init-registered-value-list)







