#include "cfuf.hxx"
#include "cfuf_fd.hxx"
#include "timer.hxx"
#include "b_lexer.hxx"
#include "common.hxx"

#include <stdlib.h>
#include <iomanip>

#ifdef WIN32
using std::cout;
using std::cerr;
using std::setw;
using std::flush;
using std::ios;
#endif

void PrintLastElapsed(CTimer& timer)
{
	cout << "(" << timer.GetLastElapsedReal() << ',' << timer.GetLastElapsedRun() << ")";
}

struct CmdLine
{
	static bool bPaste;
	static bool bCopy;
	static bool bTest;
	static bool bSurgeTypes;
	static bool bSingle;
	static char *szDstDir;
	static int nCount;
	static int nFrom;
};
bool CmdLine::bPaste;
bool CmdLine::bCopy;
bool CmdLine::bTest;
bool CmdLine::bSurgeTypes;
bool CmdLine::bSingle = true;
char *CmdLine::szDstDir = "RES";
int CmdLine::nCount = 0;
int CmdLine::nFrom = 1;

struct Operate
{
	virtual void Exec() const {}
	virtual void Cleanup() const {}
	virtual void Dump(const string& strFileName) const {}
};
struct Compile:Operate{};
struct Copy:Operate
{
	void Exec() const;
	void Cleanup() const;
	void Dump(const string& strFileName) const;
};
struct Paste:Operate
{
	void Exec() const;
	void Cleanup() const;
	void Dump(const string& strFileName) const;
};
struct CopyAndPaste:Operate
{
	void Exec() const;
	void Cleanup() const;
	void Dump(const string& strFileName) const;
};

fd_t g_fd = 0,g_fdCopy = 0,g_fdPaste = 0;
CTimer g_timer;

Compile g_opCompile;
Copy g_opCopy;
Paste g_opPaste;
CopyAndPaste g_opCopyAndPaste;
Operate *g_pOperate = &g_opCompile;

const char *GetSzDstDir();
const char *GetSzCopyDstDir();
const char *GetSzPasteDstDir();

bool Run(int i = 0);
void Dump(int i = 0);
void Parse();

Helper::CmdLineParam param1("-copy",Helper::CmdLineParam::BOOL,0,&CmdLine::bCopy);
Helper::CmdLineParam param2("-paste",Helper::CmdLineParam::BOOL,0,&CmdLine::bPaste);
Helper::CmdLineParam param3("-test",Helper::CmdLineParam::BOOL,0,&CmdLine::bTest);
Helper::CmdLineParam param4("-surgetypes",Helper::CmdLineParam::BOOL,0,&CmdLine::bSurgeTypes);
Helper::CmdLineParam param5("-single",Helper::CmdLineParam::BOOL,0,&CmdLine::bSingle);
Helper::CmdLineParam param6("-dstdir",Helper::CmdLineParam::STRING,0,&CmdLine::szDstDir);
Helper::CmdLineParam param7("-n",Helper::CmdLineParam::INT | Helper::CmdLineParam::ADJACENT,2,&CmdLine::nCount);
Helper::CmdLineParam param8("-from",Helper::CmdLineParam::INT | Helper::CmdLineParam::ADJACENT,5,&CmdLine::nFrom);

main(int argc,char *argv[])
{
	Helper::CmdLineParam params[8] = {param1,param2,param3,param4,param5,param6,param7,param8};

	int nArgsDone = Helper::ParseCmdLine(argc,argv,params,sizeof(params)/sizeof(Helper::CmdLineParam),1,1);
	if (nArgsDone < 0)
	{
		cerr << "Usage: " << *argv << " [-dstdir dir] [-copy] [-paste] [-test] [-surgetypes] [-single] [-n#] [-from#] filename\n";
		return 1;
	}

	{
		int len = strlen(CmdLine::szDstDir);
		if (len && CmdLine::szDstDir[len-1] == '/')
		{
			CmdLine::szDstDir[len-1] = '\0';
		}
	}

	if (CmdLine::nFrom > 0 && CmdLine::nCount > 0)
	{
		CmdLine::bSingle = false;
	}

	if (CmdLine::bCopy && CmdLine::bPaste) 
	{
		g_pOperate = &g_opCopyAndPaste;
	}
	else if (CmdLine::bCopy) 
	{
		g_pOperate = &g_opCopy;
	}
	else if (CmdLine::bPaste) 
	{
		g_pOperate = &g_opPaste;
	}

	CFUF::Initialize();
	g_pTheInput = new CInputFileStream(argv[++nArgsDone]);
	if (!g_pTheInput->IsOK())
	{
		cerr << "File not found: " << argv[nArgsDone] << endl;
		return 1;
	}

	int exitCode = 0;
	try
	{
		if (CmdLine::bSurgeTypes) 
		{
			Helper::SetSurgeTypes();
		}
		else
		{
			static char *types[] = {"1","3",0,"2","4",0,0};
			Helper::SetTypes(types);
		}

		cout.setf(ios::fixed | ios::left);
		cout.precision(2);

		if (CmdLine::bSingle)
		{
			Run();
		}
		else
		{
			Helper::SkipDefine(CmdLine::nFrom-1,Helper::SILENT);

			int i;
			for (i=CmdLine::nFrom;i<CmdLine::nCount+CmdLine::nFrom && Run(i);++i);
			int doneCount = i - CmdLine::nFrom;
			if (doneCount)
			{
				cout << "Time spent doing " << doneCount << " fds:\n";
				cout << " real time " << g_timer.GetTotalElapsedReal() << " seconds, ("
					<< g_timer.GetTotalElapsedReal()/doneCount << " in average)\n";
				cout << "  run time " << g_timer.GetTotalElapsedRun() << " seconds, ("
					<< g_timer.GetTotalElapsedRun()/doneCount << " in average)\n";
			}
			else 
			{
				cout << "No FDs left\n";
			}
		}
	}
	catch(Fail *e)
	{
		cout << e << endl;
		delete e;
		if (g_fd) 
		{
			g_fd->Destroy();
		}
		if (g_fdCopy) 
		{
			g_fdCopy->Destroy();
		}
		if (g_fdPaste) 
		{
			g_fdPaste->Destroy();
		}
		exitCode = 1;
	}

	delete g_pTheInput;
	CFUF::Shutdown();

	return exitCode;
}
bool Run(int i)
{
	token_t tokFDName = Helper::SkipDefine(1,Helper::POSITION);
	if (!tokFDName)
	{
		return false;
	}

	if (i)
	{
		cout << "--[" << i << "]\t" << setw(10) << tokFDName << flush;
	}

	Parse();

	g_pOperate->Exec();

	cout << endl;

	if (!CmdLine::bTest) 
	{
		Dump(i);
	}

	g_pOperate->Cleanup();

	g_fd->Destroy();
	g_fd = 0;

	return true;
}
void Dump(int i)
{
	string strFileName("_fd");
	if (i) 
	{
		strFileName << i;
	}
	{
		string strFinal(GetSzDstDir());
		strFinal << strFileName;
		Helper::DumpScm(strFinal,g_fd->GetFD()->root());
	}
	g_pOperate->Dump(strFileName);
}
void Parse()
{
	g_timer.Go();

	CLexer::s_bFlushTokenPutbackQueue = false;
	ufd_t fd = CFUFFD::Parse(*g_pTheInput);
	CLexer::s_bFlushTokenPutbackQueue = true;
	g_fd = fd.val;

	if (fd.fail) 
	{
		throw fd.fail;
	}

	g_timer.Stop();
	PrintLastElapsed(g_timer);
}

////////////////////////////////////////////////////////////////////////////////
class CDirBuffer
{
	string m_str;
public:
	CDirBuffer(char *suffix = 0):m_str(CmdLine::szDstDir)
	{
		if (suffix) 
		{
			m_str << suffix;
		}
#ifdef WIN32
		m_str << '\\';
#else
		m_str << '/';
#endif

		string strCmd;
#ifdef WIN32
		strCmd << "if not exist " << GetCStr() << " mkdir " << GetCStr();
#else
		strCmd << "mkdir -p " << GetCStr();
#endif
		if (system(strCmd.c_str()))
		{
			cerr << "Failed to create/access " << GetCStr() << "\nAborting...\n";
			exit(1);
		}
	}
	const char *GetCStr() const { return m_str.c_str(); }
};

const char *GetSzDstDir()
{
	static CDirBuffer dir;
	return dir.GetCStr();
}
const char *GetSzCopyDstDir()
{
	static CDirBuffer dir(".C");
	return dir.GetCStr();
}
const char *GetSzPasteDstDir()
{
	static CDirBuffer dir(".P");
	return dir.GetCStr();
}

////////////////////////////////////////////////////////////////////////////////
// Copy
//
void Copy::Exec() const
{
	g_timer.Go();
	g_fdCopy = g_fd->Copy();
	g_timer.Stop();
	PrintLastElapsed(g_timer);
}
void Copy::Cleanup() const
{
	g_fdCopy->Destroy();
	g_fdCopy = 0;
}
void Copy::Dump(const string& strFileName) const
{
	string strFinal(GetSzCopyDstDir());
	strFinal << strFileName;
	Helper::DumpScm(strFinal,g_fdCopy->GetFD()->root());
}

////////////////////////////////////////////////////////////////////////////////
// Paste
//
void Paste::Exec() const
{
	g_timer.Go();
	g_fdPaste = CFUFFD::MakeEmpty();
	Fail *f = g_fdPaste->Paste(g_fd);
	g_timer.Stop();
	if (f) 
	{
		throw f;
	}
	PrintLastElapsed(g_timer);
}
void Paste::Cleanup() const
{
	g_fdPaste->Destroy();
	g_fdPaste = 0;
}
void Paste::Dump(const string& strFileName) const
{
	string strFinal(GetSzPasteDstDir());
	strFinal << strFileName;
	Helper::DumpScm(strFinal,g_fdPaste->GetFD()->root());
}

////////////////////////////////////////////////////////////////////////////////
// CopyAndPaste
//
void CopyAndPaste::Exec() const
{
	g_opCopy.Exec();
	g_opPaste.Exec();
}
void CopyAndPaste::Cleanup() const
{
	g_opCopy.Cleanup();
	g_opPaste.Cleanup();
}
void CopyAndPaste::Dump(const string& strFileName) const
{
	g_opCopy.Dump(strFileName);
	g_opPaste.Dump(strFileName);
}
