#include "cfuf.hxx"
#include "common.hxx"

#include "p_parser.hxx"
#include "b_bkclass.hxx"
#include "b_altparams.hxx"
#include "p_altparams.hxx"
#include "p_alt.hxx"
#include "p_special.hxx"
#include "p_pattern.hxx"
#include "p_cset.hxx"
#include "p_fset.hxx"
#include "p_control.hxx"
#include <iomanip>

#ifdef WIN32
using std::cout;
using std::setw;
#endif

class CDummyGraph:public CGrammarBuilder{
	int m_indent;
public:
	CDummyGraph():m_indent(0){}
	Fail *Initialize();
	Fail *Realize();
	Fail *EnterSubgraph(token_t edge);
	void EnterSubgraph(int upcount);
	void LeaveSubgraph();
	Fail *AddLeaf(token_t edge,const hCType& type)        { return AddFeature("AddLeaf",edge,type); }
	Fail *AddPath(token_t edge,const hCPath& path)        { return AddFeature("AddPath",edge,path); }
	Fail *AddLeaf(const hCType& type)                     { return AddFeature("AddLeaf","*NULL*",type); }
	Fail *AddPath(const hCPath& path)                     { return AddFeature("AddPath","*NULL*",path); }
	Fail *AddSpecial(token_t attr,const hCObject& obj)    { return AddFeature("AddSpecial",attr,obj); }

	Fail *EnterAlt(CAltParams *params)    { return EnterOrNode("EnterAlt",params); }
	Fail *EnterRalt(CAltParams *params)   { return EnterOrNode("EnterRalt",params); }
	Fail *EnterOpt(CAltParams *params)    { return EnterOrNode("EnterOpt",params); }

	Fail *LeaveAlt()                      { return LeaveOrNode("LeaveAlt"); }
	Fail *LeaveRalt()                     { return LeaveOrNode("LeaveRalt"); }
	Fail *LeaveOpt()                      { return LeaveOrNode("LeaveOpt"); }

	Fail *EnterBranch();
	Fail *LeaveBranch();
	const EdgeList *cur_loc() const;
private:
	void indent();
	Fail *AddFeature(char *szCmdName,token_t attr,const hCObject& feat);
	Fail *EnterOrNode(char *szCmdName,CAltParams *params);
	Fail *LeaveOrNode(char *szCmdName);
};

void register_features();
void register_default_params();

main(int argc,char *argv[])
{
	int nArgsDone = Helper::ParseCmdLine(argc,argv,0,0,1,1);
	if (nArgsDone < 0)
	{
		cerr << "Usage: " << *argv << " filename\n";
		return 1;
	}

	CFUF::Initialize();
	g_pTheInput = new CInputFileStream(argv[++nArgsDone]);
	if (!g_pTheInput->IsOK())
	{
		cerr << "File not found: " << argv[nArgsDone] << endl;
		return 1;
	}

	CDummyGraph gr;
	ParseFD::set_grammar(&gr);

	register_features();
	int exitCode = 0;
	try
	{
		Helper::SkipDefine(1,Helper::POSITION);
		if (Fail *f = ParseFD::parse()) 
		{
			throw f;
		}
	}
	catch(Fail *e)
	{
		cerr << e << endl;
		delete e;
		exitCode = 1;
	}

	delete g_pTheInput;
	CFUF::Shutdown();

	return exitCode;
}
void register_features()
{
	// make the parser ignore tpattern feature
	static FeatureSkipper skip;
	ParseFD::register_feature(Predefined::tpattern,&skip);

	static SpecialFeatureParser<FsetParser,CFset,CDummyGraph> fset_parser;
	static SpecialFeatureParser<CsetParser,CCset,CDummyGraph> cset_parser;
	static SpecialFeatureParser<PatternParser,CPattern,CDummyGraph> pattern_parser;
	static SpecialFeatureParser<ControlParser,CControl,CDummyGraph> control_parser;
	ParseFD::register_feature(Predefined::fset,&fset_parser);
	ParseFD::register_feature(Predefined::cset,&cset_parser);
	ParseFD::register_feature(Predefined::pattern,&pattern_parser);
	ParseFD::register_feature(Predefined::control,&control_parser);

	register_default_params();              // register the default alt parameters

	static AltParser alt_parser;
	static RaltParser ralt_parser;
	static OptParser opt_parser;
	ParseFD::register_feature(Predefined::alt,&alt_parser);
	ParseFD::register_feature(Predefined::ralt,&ralt_parser);
	ParseFD::register_feature(Predefined::opt,&opt_parser);
}
void register_default_params()
{
	static CIndexParser parse_index;
	static CBKClassParser parse_bkclass;
	static CDemoParser parse_demo;
	static CWaitParser parse_wait;
	AltHeaderParser::register_param(Predefined::index,&parse_index);
	AltHeaderParser::register_param(Predefined::bkclass,&parse_bkclass);
	AltHeaderParser::register_param(Predefined::demo,&parse_demo);
	AltHeaderParser::register_param(Predefined::wait,&parse_wait);
}

////////////////////////////////////////////////////////////////////////////////
// CDummyGraph methods
Fail *CDummyGraph::Initialize()
{
	cout << "Initialize\n";
	m_indent+=2;
	return 0;
}
Fail *CDummyGraph::Realize()
{
	m_indent-=2;
	cout << "Realize\n";
	return 0; 
}
Fail *CDummyGraph::EnterSubgraph(token_t edge) 
{ 
	indent();
	cout << "EnterSubgraph " << edge << endl;
	m_indent+=2;
	return 0; 
}
void CDummyGraph::EnterSubgraph(int upcount)    
{
	indent();
	cout << "EnterSubgraph " << CPath(upcount) << endl;
	m_indent+=2; 
}
void CDummyGraph::LeaveSubgraph()  
{
	m_indent-=2;
	indent();
	cout << "LeaveSubgraph\n"; 
}
Fail *CDummyGraph::EnterBranch()
{
	indent();
	cout << "EnterBranch\n";
	m_indent+=2;
	return 0; 
}
Fail *CDummyGraph::LeaveBranch()
{
	m_indent-=2;
	indent();
	cout << "LeaveBranch\n";
	return 0; 
}
const EdgeList *CDummyGraph::cur_loc() const       
{
	static EdgeList elist; 
	return &elist; 
}
void CDummyGraph::indent()
{
	cout << setw(m_indent) << ""; 
}
Fail *CDummyGraph::AddFeature(char *szCmdName,token_t attr,const hCObject& feat)
{
	indent();
	cout << szCmdName << ' ' << attr << ' ' << feat << endl;
	return 0;
}
Fail *CDummyGraph::EnterOrNode(char *szCmdName,CAltParams *params)
{
	indent();
	cout << szCmdName << ' ' << params << endl;
	m_indent+=2;
	delete params;
	return 0;
}
Fail *CDummyGraph::LeaveOrNode(char *szCmdName)
{
	m_indent-=2;
	indent();
	cout << szCmdName << endl;
	return 0;
}
