#include "cfuf.hxx"
#include "cfuf_fd.hxx"
#include "cfuf_grammar.hxx"
#include "cfuf_misc.hxx"
#include "timer.hxx"
#include "b_lexer.hxx"
#include "common.hxx"

class COStream:public COutputStream
{
	ostream& m_os;
public:
	explicit COStream(ostream& os):m_os(os){}
	COutputStream& operator<<(const string& str)    { m_os << str;return *this; }
	COutputStream& operator<<(const char *str)      { m_os << str;return *this; }
	COutputStream& operator<<(char c)               { m_os << c;return *this; }
	COutputStream& operator<<(long l)               { m_os << l;return *this; }
	COutputStream& operator<<(const CMyObject& obj) { m_os << obj.toString();return *this; }
	COutputStream& operator<<(const CMyObject *obj) { m_os << (obj ? obj->toString() : "NULL");return *this; }
	void flush()                                    { m_os.flush(); }
};

#include <iomanip>

#ifdef WIN32
using std::cout;
using std::ios;
using std::flush;
#endif

void PrintLastElapsed(CTimer& timer)
{
	cout << '(' << timer.GetLastElapsedReal() << ',' << timer.GetLastElapsedRun() << ')';
}

struct CmdLine
{
	static bool bDots;
	static bool bTkdots;
	static bool bTest;
	static bool bVerbose;
	static bool bSurgeTypes;
	static int nUMax;
	static int nCount;
	static int nFrom;
};
bool CmdLine::bDots;
bool CmdLine::bTkdots;
bool CmdLine::bTest;
bool CmdLine::bVerbose;
bool CmdLine::bSurgeTypes;
int CmdLine::nUMax = 2500;
int CmdLine::nCount = 1000;
int CmdLine::nFrom = 1;

void Dump(CFUFFD *pFD,token_t tokFDName);
ugrammar_t ParseGrammar(CInputFileStream& grfile);

Helper::CmdLineParam param1("-dots",Helper::CmdLineParam::BOOL,0,&CmdLine::bDots);
Helper::CmdLineParam param2("-tkdots",Helper::CmdLineParam::BOOL,0,&CmdLine::bTkdots);
Helper::CmdLineParam param3("-test",Helper::CmdLineParam::BOOL,0,&CmdLine::bTest);
Helper::CmdLineParam param4("-verbose",Helper::CmdLineParam::BOOL,0,&CmdLine::bVerbose);
Helper::CmdLineParam param5("-surgetypes",Helper::CmdLineParam::BOOL,0,&CmdLine::bSurgeTypes);
Helper::CmdLineParam param6("-umax",Helper::CmdLineParam::INT | Helper::CmdLineParam::ADJACENT,5,&CmdLine::nUMax);
Helper::CmdLineParam param7("-n",Helper::CmdLineParam::INT | Helper::CmdLineParam::ADJACENT,2,&CmdLine::nCount);
Helper::CmdLineParam param8("-from",Helper::CmdLineParam::INT | Helper::CmdLineParam::ADJACENT,5,&CmdLine::nFrom);

main(int argc,char *argv[])
{
	Helper::CmdLineParam params[8] = {param1,param2,param3,param4,param5,param6,param7,param8};

	int nArgsDone = Helper::ParseCmdLine(argc,argv,params,sizeof(params)/sizeof(Helper::CmdLineParam),2,2);
	if (nArgsDone < 0)
	{
		cerr << "Usage: " << *argv << " [-dots] [-tkdots] [-test] [-verbose] [-surgetypes] [-umax#] [-n#] [-from#] grfile fdfile\n";
		return 1;
	}

	CFUF::Initialize();

	CInputFileStream grfile(argv[++nArgsDone]);
	if (!grfile.IsOK())
	{
		cerr << "File not found: " << argv[nArgsDone] << endl;
		return 1;
	}
	CInputFileStream fdfile(argv[++nArgsDone]);
	if (!fdfile.IsOK())
	{
		cerr << "File not found: " << argv[nArgsDone] << endl;
		return 1;
	}

	if (CmdLine::bVerbose)
	{
		g_pTheOutput = new COStream(cerr);
		CFUFMisc::SetTraceLevel(3);
	}

	g_pTheEnv->SetMaxUCount(CmdLine::nUMax);

	ugrammar_t gr;
	ufd_t fd;

	CTimer timer;
	uint good=0,bad=0;
	int exitCode = 0;

	try
	{
		if (CmdLine::bSurgeTypes)
		{
			Helper::SetSurgeTypes();
		}

		gr = ParseGrammar(grfile);
		if (gr.fail)
		{
			throw gr.fail;
		}
		if (!gr.val)
		{
			return 1;
		}

		g_pTheLexer->reset();
		g_pTheInput = &fdfile;

		Helper::SkipDefine(CmdLine::nFrom-1,Helper::SILENT);

		cout.setf(ios::fixed | ios::left);
		cout.precision(2);

		for (int i=CmdLine::nFrom;i<CmdLine::nCount+CmdLine::nFrom;++i)
		{
			token_t tokFDName = Helper::SkipDefine(1,Helper::POSITION);
			if (!tokFDName) 
			{
				break;
			}

			cout << "----[" << i << "] " << tokFDName << ' ' << flush;

			CLexer::s_bFlushTokenPutbackQueue = false;
			fd = CFUFFD::Parse(fdfile);
			CLexer::s_bFlushTokenPutbackQueue = true;
			if (fd.fail)
			{
				throw fd.fail;
			}

			timer.Go();
			ret::ubool_t b = fd.val->Unify(gr.val);
			timer.Stop();
			if (b.fail) 
			{
				throw b.fail;
			}

			PrintLastElapsed(timer);
			cout << " (" << CFUFFD::GetUCount() << ")";

			if (b.val)
			{
				++good;
				if (!CmdLine::bTest) 
				{
					Dump(fd.val,tokFDName);
				}
				cout << endl;
			}
			else
			{
				++bad;
				timer.CancelLast();
				cout << " Unification failed!\n";
			}

			fd.val->Destroy();
		}

		gr.val->Destroy();
		cout << "done.\n";

		if (good)
		{
			cout << "Successfully unified " << good << " out of " << good+bad << " fds.\n";
			cout << "Time spent in unification of " << good << " \"good\" fds:\n";
			cout << " real time " << timer.GetTotalElapsedReal() << " seconds, (" << timer.GetTotalElapsedReal()/good << " in average)\n";
			cout << "  run time " << timer.GetTotalElapsedRun() << " seconds, (" << timer.GetTotalElapsedRun()/good << " in average)\n";
		}

	}
	catch(Fail *e)
	{
		cerr << e << endl;
		delete e;
		if (gr.val)
		{
			gr.val->Destroy();
		}
		if (fd.val)
		{
			fd.val->Destroy();
		}
		exitCode = 1;
	}

	if (CmdLine::bVerbose)
	{
		delete g_pTheOutput;
	}
	CFUF::Shutdown();

	return exitCode;
}
ugrammar_t ParseGrammar(CInputFileStream& grfile)
{
	g_pTheInput = &grfile;
	if (!Helper::SkipDefine(1,Helper::POSITION))
	{
		if (CmdLine::bVerbose)
		{
			delete g_pTheOutput;
		}
		CFUF::Shutdown();
		return grammar_t(0);
	}

	CLexer::s_bFlushTokenPutbackQueue = false;
	ugrammar_t gr = CFUFGrammar::Parse(grfile);
	CLexer::s_bFlushTokenPutbackQueue = true;
	return gr;
}

void Dump(CFUFFD *pFD,token_t tokFDName)
{
	string strFileName("_");
	strFileName << tokFDName;
	CFDNode *pRoot = pFD->GetFD()->root();

	if (CmdLine::bDots)
	{
		Helper::DumpDot(strFileName,pRoot);
	}
	if (CmdLine::bTkdots)
	{
		Helper::DumpTcl(strFileName,pRoot);
	}
	Helper::DumpScm(strFileName,pRoot);
}
