#include "stdcfuf.h"
#include "b_tokhier.hxx"
#include "b_nodebase.hxx"

#include <typeinfo>
////////////////////////////////////////////////////////////////////////////////
// Fail objects for various types of unification failures.
//

// Returned when unification of a normal and a special types failed.
class NSFail:public Fail{
public:
  NSFail(const hCType& normal,const hCType& special):m_normal(normal),m_special(special){}
  string GetDesc() const;
private:
  hCType m_normal;
  hCType m_special;
};

// Returned when unification of the internal and under types failed.
// I do not want this case caught along with the previous (internal and normal).
class IUFail:public Fail{
public:
  explicit IUFail(const hCUnderType& type):m_type(type){}
  string GetDesc() const;
private:
  hCUnderType m_type;
};

// Returned when unification of the internal and given types failed.
class IGFail:public Fail{
public:
  bool IsInternalGiven() const { return true; }
  string GetDesc() const;
};

// Returned when unification failed because a feature was not given.
class GivenFail:public Fail{
public:
  GivenFail(const hCType& given,const hCType& normal):m_given(given),m_normal(normal){}
  string GetDesc() const;
private:
  hCType m_given,m_normal;
};

// ------------------------------------------------------------------------
//       | NIL   ANY  GIVEN  NONE  X1
// ------+------------------------------------
// NIL   | NIL   ANY  FAIL   NONE  X1
// ANY   | ANY   ANY  ANY    FAIL  X1
// GIVEN | FAIL  ANY  FAIL   FAIL  X1
// NONE  | NONE  FAIL FAIL   NONE  FAIL
// X2    | X2    X2   X2     FAIL  u(X1,X2)  <-- use type hierarchy.

static CFDNodeBase *node_this,*node_type;
////////////////////////////////////////////////////////////////////////////////////
// CType methods
//
ret::utype_t CType::UnifyWithSpecial(const hCType& type,CFDNodeBase *,CFDNodeBase *)
{
  return new SSFail(this,type.pointee());
}
ret::utype_t CType::UnifyWithInternal(const hCInternalType& type)   { return new NSFail(type.pointee(),this); }
ret::utype_t CType::UnifyWithUnder(const hCUnderType& type)         { return new NSFail(type.pointee(),this); }
ret::utype_t CType::UnifyWithStrictUnder(const hCStrictUnderType& type)  { return UnifyWithUnder(type.pointee()); }
ret::utype_t CType::UnifyWithQuoted(const hCQuotedType& type)       { return new NSFail(type.pointee(),this); }
ret::utype_t CType::UnifyWithAtom(const hCAtomType& type)           { return new NSFail(type.pointee(),this); }
ret::utype_t CType::UnifyWithNone(const hCNoneType& type)           { return new NSFail(type.pointee(),this); }
ret::utype_t CType::UnifyWithNil(const hCNilType&)
{
  return UnifyWithSpecial(g_pTheEnv->nil(),node_this,node_type);
}
ret::utype_t CType::UnifyWithAny(const hCAnyType&)
{
  return UnifyWithSpecial(g_pTheEnv->nil(),node_this,node_type);
}
ret::utype_t CType::UnifyWithGiven(const hCGivenType&)
{
  return UnifyWithSpecial(g_pTheEnv->nil(),node_this,node_type);
}
// By default unify assumes that both types are special
ret::utype_t CType::unify(const hCType& type,CFDNodeBase *n1,CFDNodeBase *n2)
{
  node_this = n2;
  node_type = n1;
  return type->UnifyWithSpecial(this,n1,n2);
}

////////////////////////////////////////////////////////////////////////////////////
// CNormalType methods
//
// By default unification of a normal and a special types fails.
ret::utype_t CNormalType::UnifyWithSpecial(const hCType& type,CFDNodeBase *,CFDNodeBase *)
{
  return new NSFail(this,type.pointee());
}
ret::utype_t CNormalType::UnifyWithInternal(const hCInternalType& type)    { return new INFail(this); }
ret::utype_t CNormalType::UnifyWithNil(const hCNilType& type)              { return ret::utype_t(this); }
ret::utype_t CNormalType::UnifyWithAny(const hCAnyType& type)              { return ret::utype_t(this); }
ret::utype_t CNormalType::UnifyWithNone(const hCNoneType& type)            { return new NNFail(type.pointee(),this); }
ret::utype_t CNormalType::UnifyWithQuoted(const hCQuotedType& type)        { return new NNFail(this,type.pointee()); }
// By default, unification of a normal and the given type succeeds
// if either the GIVEN semantics are old (not correct) or the normal type
// in question comes from the input.
ret::utype_t CNormalType::UnifyWithGiven(const hCGivenType& type)
{
  bool input_this = node_this ? node_this->input() : g_pTheEnv->IsInput();
  if (!input_this && g_pTheEnv->m_bCorrectGiven) return new GivenFail(type.pointee(),this);
  return ret::utype_t(this);
}

////////////////////////////////////////////////////////////////////////////////////
// CInternalType methods
//
// An internal type can have one of the two possible titles - nil or any.
// Result of the unification of two internal types is another internal type.
// It title is nil iff both titles are nil.
ret::utype_t CInternalType::UnifyWithInternal(const hCInternalType& type)
{
  token_t tknNewTitle = (m_tknTitle == type->m_tknTitle) ? m_tknTitle : Predefined::any;
  if (m_tknTitle == tknNewTitle) return ret::utype_t(this);
  return ret::utype_t(type.pointee());
}
ret::utype_t CInternalType::UnifyWithUnder(const hCUnderType& type)        { return new IUFail(type); }
ret::utype_t CInternalType::UnifyWithQuoted(const hCQuotedType& type)      { return new INFail(type.pointee()); }
ret::utype_t CInternalType::UnifyWithAtom(const hCAtomType& type)          { return new INFail(type.pointee()); }
ret::utype_t CInternalType::UnifyWithNone(const hCNoneType& type)          { return new INFail(type.pointee()); }
ret::utype_t CInternalType::UnifyWithGiven(const hCGivenType&)             { return new IGFail(); }
ret::utype_t CInternalType::UnifyWithAny(const hCAnyType& type)
{
  if (m_tknTitle == Predefined::any) return ret::utype_t(this);
  return g_pTheEnv->internal(1);
}
ret::utype_t CInternalType::unify(const hCType& type,CFDNodeBase *n1,CFDNodeBase *n2)
{
  node_this = n2;
  node_type = n1;
  return type->UnifyWithInternal(this);
}
string CInternalType::toString() const
{
  return TOK2STR(Predefined::internal);
}

////////////////////////////////////////////////////////////////////////////////////
// CQuotedType methods
//
ret::utype_t CQuotedType::UnifyWithQuoted(const hCQuotedType& type)
{
  if (m_tknTitle != type->m_tknTitle) return new NNFail(this,type.pointee());
  return ret::utype_t(this);
}
ret::utype_t CQuotedType::UnifyWithUnder(const hCUnderType& type)  { return new NNFail(this,type.pointee()); }
ret::utype_t CQuotedType::UnifyWithAtom(const hCAtomType& type)    { return new NNFail(this,type.pointee()); }
ret::utype_t CQuotedType::unify(const hCType& type,CFDNodeBase *n1,CFDNodeBase *n2)
{
  node_this = n2;
  node_type = n1;
  return type->UnifyWithQuoted(this);
}
string CQuotedType::toString() const
{
  string str;
  return str << '"' << m_tknTitle << '"';
}

////////////////////////////////////////////////////////////////////////////////////
// CUnderType methods
//
ret::utype_t CUnderType::UnifyUnderWithUnder(CUnderType *type,bool bStrict1,bool bStrict2)
{
  ret::utoken_t glb=g_pTheTokhier->get_glb(m_tknTitle,type->title());
  if (glb.fail) return glb.fail;
  if (!glb.val) return new NNFail(this,type);
  bool bStrict = (bStrict1 != bStrict2) ?
    (glb.val == (bStrict1 ? m_tknTitle : type->title())) : bStrict1;
  return ret::utype_t(bStrict ? new CStrictUnderType(glb.val) : new CUnderType(glb.val));
}
ret::utype_t CUnderType::UnifyUnderWithAtom(CAtomType *type,bool bStrict)
{
  ret::utoken_t glb=g_pTheTokhier->get_glb(m_tknTitle,type->title());
  if (glb.fail) return glb.fail;
  if (!glb.val || glb.val != type->title() || (glb.val == m_tknTitle && bStrict))
    return new NNFail(this,type);
  return ret::utype_t(type);
}
ret::utype_t CUnderType::UnifyWithUnder(const hCUnderType& type)
{
  return UnifyUnderWithUnder(type,false,false);
}
ret::utype_t CUnderType::UnifyWithStrictUnder(const hCStrictUnderType& type)
{
  return UnifyUnderWithUnder(type,false,true);
}
ret::utype_t CUnderType::UnifyWithAtom(const hCAtomType& type)
{
  return UnifyUnderWithAtom(type,false);
}
ret::utype_t CUnderType::UnifyWithInternal(const hCInternalType&)  { return new IUFail(this); }
ret::utype_t CUnderType::UnifyWithNil(const hCNilType& type)       { return new GivenFail(this,type.pointee()); }
ret::utype_t CUnderType::UnifyWithAny(const hCAnyType& type)       { return new GivenFail(this,type.pointee()); }
ret::utype_t CUnderType::UnifyWithGiven(const hCGivenType& type)   { return new GivenFail(this,type.pointee()); }
ret::utype_t CUnderType::unify(const hCType& type,CFDNodeBase *n1,CFDNodeBase *n2)
{
  node_this = n2;
  node_type = n1;
  return type->UnifyWithUnder(this);
}
string CUnderType::toString() const
{
  string str;
  return str << "#(under " << m_tknTitle << ")";
}

////////////////////////////////////////////////////////////////////////////////////
// CStrictUnderType methods
//
ret::utype_t CStrictUnderType::UnifyWithUnder(const hCUnderType& type)
{
  return UnifyUnderWithUnder(type,true,false);
}
ret::utype_t CStrictUnderType::UnifyWithStrictUnder(const hCStrictUnderType& type)
{
  return UnifyUnderWithUnder(type,true,true);
}
ret::utype_t CStrictUnderType::UnifyWithAtom(const hCAtomType& type)
{
  return UnifyUnderWithAtom(type,true);
}
ret::utype_t CStrictUnderType::unify(const hCType& type,CFDNodeBase *n1,CFDNodeBase *n2)
{
  node_this = n2;
  node_type = n1;
  return type->UnifyWithStrictUnder(this);
}
string CStrictUnderType::toString() const
{
  string str;
  return str << "#(sunder " << m_tknTitle << ")";
}

////////////////////////////////////////////////////////////////////////////////////
// CAtomType methods
//
ret::utype_t CAtomType::UnifyWithUnder(const hCUnderType& type)
{
  node_this = node_type;
  return type->UnifyWithAtom(this);   // redirect the request to the under type
}
ret::utype_t CAtomType::UnifyWithAtom(const hCAtomType& type)
{
  ret::utoken_t glb=g_pTheTokhier->get_glb(m_tknTitle,type->m_tknTitle);
  if (glb.fail) return glb.fail;
  if (!glb.val) return new NNFail(this,type.pointee());
  return ret::utype_t(new CAtomType(glb.val));
}
ret::utype_t CAtomType::unify(const hCType& type,CFDNodeBase *n1,CFDNodeBase *n2)
{
  node_this = n2;
  node_type = n1;
  return type->UnifyWithAtom(this);
}

////////////////////////////////////////////////////////////////////////////////////
// CNoneType methods
//
ret::utype_t CNoneType::UnifyWithUnder(const hCUnderType& type)     { return new NNFail(this,type.pointee()); }
ret::utype_t CNoneType::UnifyWithAtom(const hCAtomType& type)       { return new NNFail(this,type.pointee()); }
ret::utype_t CNoneType::UnifyWithAny(const hCAnyType& type)         { return new NNFail(this,type.pointee()); }
ret::utype_t CNoneType::UnifyWithGiven(const hCGivenType& type)     { return new GivenFail(type.pointee(),this); }
ret::utype_t CNoneType::UnifyWithNone(const hCNoneType&)            { return ret::utype_t(this); }
ret::utype_t CNoneType::unify(const hCType& type,CFDNodeBase *n1,CFDNodeBase *n2)
{
  node_this = n2;
  node_type = n1;
  return type->UnifyWithNone(this);
}

////////////////////////////////////////////////////////////////////////////////////
// CNilType methods
//
ret::utype_t CNilType::UnifyWithSpecial(const hCType& type,CFDNodeBase *,CFDNodeBase *)
{
  return type->UnifyWithSpecial(g_pTheEnv->nil(),node_type,node_this);
}
ret::utype_t CNilType::UnifyWithUnder(const hCUnderType& type)      { return new GivenFail(type.pointee(),this); }
ret::utype_t CNilType::UnifyWithQuoted(const hCQuotedType& type)    { return ret::utype_t(type.pointee()); }
ret::utype_t CNilType::UnifyWithAtom(const hCAtomType& type)        { return ret::utype_t(type.pointee()); }
ret::utype_t CNilType::UnifyWithInternal(const hCInternalType& type){ return ret::utype_t(type.pointee()); }
ret::utype_t CNilType::UnifyWithNone(const hCNoneType& type)        { return ret::utype_t(type.pointee()); }
ret::utype_t CNilType::UnifyWithAny(const hCAnyType& type)          { return ret::utype_t(type.pointee()); }
ret::utype_t CNilType::UnifyWithGiven(const hCGivenType& type)      { return new GivenFail(type.pointee(),this); }
ret::utype_t CNilType::unify(const hCType& type,CFDNodeBase *n1,CFDNodeBase *n2)
{
  node_this = n2;
  node_type = n1;
  return type->UnifyWithNil(this);
}

////////////////////////////////////////////////////////////////////////////////////
// CAnyType methods
//
ret::utype_t CAnyType::UnifyWithSpecial(const hCType& type,CFDNodeBase *,CFDNodeBase *)
{
  return type->UnifyWithSpecial(g_pTheEnv->nil(),node_type,node_this);
}
ret::utype_t CAnyType::UnifyWithUnder(const hCUnderType& type)     { return new GivenFail(type.pointee(),this); }
ret::utype_t CAnyType::UnifyWithQuoted(const hCQuotedType& type)   { return ret::utype_t(type.pointee()); }
ret::utype_t CAnyType::UnifyWithAtom(const hCAtomType& type)       { return ret::utype_t(type.pointee()); }
ret::utype_t CAnyType::UnifyWithNone(const hCNoneType& type)       { return new NNFail(type.pointee(),this); }
ret::utype_t CAnyType::UnifyWithInternal(const hCInternalType& type)
{
  node_this = node_type;
  return type->UnifyWithAny(this);
}
// If the GIVEN semantics is correct (i.e. not the old GIVEN), then the
// unification always fails.With the old semantics it succeeds if ANY comes
// from the input.
ret::utype_t CAnyType::UnifyWithGiven(const hCGivenType& type)
{
  bool input_this = node_this ? node_this->input() : g_pTheEnv->IsInput();
  if (!input_this || g_pTheEnv->m_bCorrectGiven) return new GivenFail(type.pointee(),this);
  return ret::utype_t(this);
}
ret::utype_t CAnyType::unify(const hCType& type,CFDNodeBase *n1,CFDNodeBase *n2)
{
  node_this = n2;
  node_type = n1;
  return type->UnifyWithAny(this);
}

////////////////////////////////////////////////////////////////////////////////////
// CGivenType methods
//
ret::utype_t CGivenType::UnifyWithSpecial(const hCType& type,CFDNodeBase *,CFDNodeBase *)
{
  return type->UnifyWithSpecial(g_pTheEnv->nil(),node_type,node_this);
}
ret::utype_t CGivenType::UnifyWithUnder(const hCUnderType& type)   { return new GivenFail(type.pointee(),this); }
ret::utype_t CGivenType::UnifyWithNone(const hCNoneType& type)     { return new GivenFail(this,type.pointee()); }
ret::utype_t CGivenType::UnifyWithNil(const hCNilType& type)       { return new GivenFail(this,type.pointee()); }
ret::utype_t CGivenType::UnifyWithGiven(const hCGivenType& type)   { return new GivenFail(this,type.pointee()); }
ret::utype_t CGivenType::UnifyWithInternal(const hCInternalType&)  { return new IGFail(); }
ret::utype_t CGivenType::UnifyWithQuoted(const hCQuotedType& type)
{
  node_this = node_type;
  return type->UnifyWithGiven(this);
}
ret::utype_t CGivenType::UnifyWithAtom(const hCAtomType& type)
{
  node_this = node_type;
  return type->UnifyWithGiven(this);
}
ret::utype_t CGivenType::UnifyWithAny(const hCAnyType& type)
{
  node_this = node_type;
  return type->UnifyWithGiven(this);
}
ret::utype_t CGivenType::unify(const hCType& type,CFDNodeBase *n1,CFDNodeBase *n2)
{
  node_this = n2;
  node_type = n1;
  return type->UnifyWithGiven(this);
}

////////////////////////////////////////////////////////////////////////////////
// toString() methods of Fail objects
//
string SSFail::GetDesc() const
{
  string str("Special type <");
  return str << m_type1 << " (" << typeid(*m_type1).name() << ")> and <"
             << m_type2 << " (" << typeid(*m_type2).name() << ")> are inconsistent";
}
string INFail::GetDesc() const
{
  string str("Normal type <");
  return str << m_type << "> is inconsistent with the internal type";
}
string NNFail::GetDesc() const
{
  string str("Normal types <");
  return str << m_type1 << "> and <" << m_type2 << "> are inconsistent";
}
string NSFail::GetDesc() const
{
  string str("Normal type <");
  return str << m_normal << "> and special type <" << m_special
             << " (" << typeid(*m_special).name() << ")> are inconsistent";
}
string IUFail::GetDesc() const
{
  string str("Normal type <");
  return str << m_type << "> is inconsistent with the internal type";
}
string IGFail::GetDesc() const
{
  return "Normal type <given> is inconsistent with the internal type";
}
string GivenFail::GetDesc() const
{
  char *aux = "";
  if (m_normal->final() && !m_normal->none()) aux = " (not input)";
  string str("Normal types <");
  return str << m_given << "> and <" << m_normal << aux << "> are inconsistent";
}
