#ifndef _BASE_DEFTYPES_
#define _BASE_DEFTYPES_

class CFDNodeBase; 
////////////////////////////////////////////////////////////////////////////////////
// Possible kind of types:
//  - atom type (like present,agent, etc...)
//  - (s)under type (like #(under clause), #(sunder np), etc ...)
//  - a double quoted string (like "Carol" for the lex attribute)
// Every type object must know how to unify within another type.
// The default implementation returns an empty type handle.If a type does not actually
// need unification it can fall back to the default implementation and hope that the
// return value is handled properly.
// It is very important to check the return value to see if the type performed any
// unification at all.
// Empty handle is returned only if unification is just not supported)
//                           CQuotedType     CNoneType
// CType --> CNormalType --> CUnderType      CNilType
//                           CAtomType ----> CAnyType
//                           CInternalType   CGivenType
// Default settings are:
// 1. final is true for {CNormalType,CQuotedType,CUnderType,CAtomType,CNoneType}
// 2. input is true for all types.
// 3. special is true for CType. (Special types should be derived from it)
#include "b_typefwd.hxx"

class CType:public CMyObject{
public:
  virtual bool final() const    { return false; }  // by default type is not final and so it is safe to mark it
  virtual bool special() const  { return true; }   // a type is special by default
  virtual bool internal() const { return false; }  // convenience functions
  virtual bool under() const    { return false; }
  virtual bool quoted() const   { return false; }
  virtual bool atom() const     { return false; }
  virtual bool nil() const      { return false; }
  virtual bool none() const     { return false; }
  virtual bool any() const      { return false; }
  virtual bool given() const    { return false; }
  virtual bool is_equal(CType *) const =0;
  virtual ret::utype_t unify(const hCType& type,CFDNodeBase * =0,CFDNodeBase * =0);
public:
  virtual ret::utype_t UnifyWithSpecial(const hCType&,CFDNodeBase *,CFDNodeBase *);
  virtual ret::utype_t UnifyWithInternal(const hCInternalType&);
  virtual ret::utype_t UnifyWithUnder(const hCUnderType&);
  virtual ret::utype_t UnifyWithStrictUnder(const hCStrictUnderType&);
  virtual ret::utype_t UnifyWithQuoted(const hCQuotedType&);
  virtual ret::utype_t UnifyWithAtom(const hCAtomType&);
  virtual ret::utype_t UnifyWithNone(const hCNoneType&);
  virtual ret::utype_t UnifyWithNil(const hCNilType&);
  virtual ret::utype_t UnifyWithAny(const hCAnyType&);
  virtual ret::utype_t UnifyWithGiven(const hCGivenType&);
protected:
  CType(){}
private:
  CType(const CType&);
};

class CNormalType:public CType{
public:
  bool final() const            { return true; }
  bool special() const          { return false; }
public:
  token_t title() const         { return m_tknTitle; }
  bool is_equal(CType *type) const {
    CNormalType *normal = cast<CNormalType>(type);
    return (normal &&                           // the second type must be derived from CNormalType
            m_tknTitle == normal->title() &&    // the titles must be the same
            typeid(*this) == typeid(*type));    // the types must be of the same kind
  }
protected:
  token_t m_tknTitle;      // the title of the type
protected:
  explicit CNormalType(token_t tknTitle):m_tknTitle(tknTitle){}
public:
  ret::utype_t UnifyWithSpecial(const hCType&,CFDNodeBase *,CFDNodeBase *);
  ret::utype_t UnifyWithInternal(const hCInternalType&);
  ret::utype_t UnifyWithNil(const hCNilType&);
  ret::utype_t UnifyWithAny(const hCAnyType&);
  ret::utype_t UnifyWithGiven(const hCGivenType&);
  ret::utype_t UnifyWithNone(const hCNoneType&);
  ret::utype_t UnifyWithQuoted(const hCQuotedType&);
};

class CInternalType:public CNormalType{
public:
  bool final() const            { return false; }
  bool internal() const         { return true; }
  bool any() const              { return m_tknTitle == Predefined::any; }
  bool is_equal(CType *type) const { return type->internal(); }
  ret::utype_t unify(const hCType&,CFDNodeBase * =0,CFDNodeBase * =0);
public:
  explicit CInternalType(token_t tknTitle=Predefined::nil):CNormalType(tknTitle){}
  string toString() const;
public:
  ret::utype_t UnifyWithInternal(const hCInternalType&);
  ret::utype_t UnifyWithUnder(const hCUnderType&);
  ret::utype_t UnifyWithQuoted(const hCQuotedType&);
  ret::utype_t UnifyWithAtom(const hCAtomType&);
  ret::utype_t UnifyWithNone(const hCNoneType&);
  ret::utype_t UnifyWithAny(const hCAnyType&);
  ret::utype_t UnifyWithGiven(const hCGivenType&);
};

class CQuotedType:public CNormalType{
public:
  bool quoted() const           { return true; }
  ret::utype_t unify(const hCType& type,CFDNodeBase * =0,CFDNodeBase * =0);
public:
  explicit CQuotedType(token_t tknTitle):CNormalType(tknTitle){}
  string toString() const;
public:
  ret::utype_t UnifyWithUnder(const hCUnderType&);
  ret::utype_t UnifyWithQuoted(const hCQuotedType&);
  ret::utype_t UnifyWithAtom(const hCAtomType&);
};

class CUnderType:public CNormalType{
public:
  bool under() const            { return true; }
  bool given() const            { return true; }
  ret::utype_t unify(const hCType& type,CFDNodeBase * =0,CFDNodeBase * =0);
public:
  CUnderType(token_t tknTitle):CNormalType(tknTitle){}
  string toString() const;
public:
  ret::utype_t UnifyWithInternal(const hCInternalType&);
  ret::utype_t UnifyWithUnder(const hCUnderType&);
  ret::utype_t UnifyWithStrictUnder(const hCStrictUnderType&);
  ret::utype_t UnifyWithAtom(const hCAtomType&);
  ret::utype_t UnifyWithNil(const hCNilType&);
  ret::utype_t UnifyWithAny(const hCAnyType&);
  ret::utype_t UnifyWithGiven(const hCGivenType&);
protected:
  ret::utype_t UnifyUnderWithUnder(CUnderType *type,bool bStrict1,bool bStrict2);
  ret::utype_t UnifyUnderWithAtom(CAtomType *type,bool bStrict);
};

class CStrictUnderType:public CUnderType{
public:
  ret::utype_t unify(const hCType& type,CFDNodeBase * =0,CFDNodeBase * =0);
public:
  CStrictUnderType(token_t tknTitle):CUnderType(tknTitle){}
  string toString() const;
public:
  ret::utype_t UnifyWithUnder(const hCUnderType&);
  ret::utype_t UnifyWithStrictUnder(const hCStrictUnderType&);
  ret::utype_t UnifyWithAtom(const hCAtomType&);
};

class CAtomType:public CNormalType{
public:
  bool atom() const             { return true; }
  ret::utype_t unify(const hCType& type,CFDNodeBase * =0,CFDNodeBase * =0);
public:
  explicit CAtomType(token_t tknTitle):CNormalType(tknTitle){}
  string toString() const       { return TOK2STR(m_tknTitle); }
public:
  ret::utype_t UnifyWithUnder(const hCUnderType&);
  ret::utype_t UnifyWithAtom(const hCAtomType&);
};

class CNoneType:public CAtomType{
public:
  bool none() const             { return true; }
  ret::utype_t unify(const hCType& type,CFDNodeBase * =0,CFDNodeBase * =0);
public:
  CNoneType():CAtomType(Predefined::none){}
public:
  ret::utype_t UnifyWithUnder(const hCUnderType&);
  ret::utype_t UnifyWithAtom(const hCAtomType&);
  ret::utype_t UnifyWithAny(const hCAnyType&);
  ret::utype_t UnifyWithGiven(const hCGivenType&);
  ret::utype_t UnifyWithNone(const hCNoneType&);
};

class CNilType:public CAtomType{
public:
  bool nil() const              { return true; }
  bool final() const            { return false; }
  ret::utype_t unify(const hCType& type,CFDNodeBase * =0,CFDNodeBase * =0);
public:
  CNilType():CAtomType(Predefined::nil){}
public:
  ret::utype_t UnifyWithSpecial(const hCType&,CFDNodeBase *,CFDNodeBase *);
  ret::utype_t UnifyWithUnder(const hCUnderType&);
  ret::utype_t UnifyWithQuoted(const hCQuotedType&);
  ret::utype_t UnifyWithAtom(const hCAtomType&);
  ret::utype_t UnifyWithInternal(const hCInternalType&);
  ret::utype_t UnifyWithNone(const hCNoneType&);
  ret::utype_t UnifyWithAny(const hCAnyType&);
  ret::utype_t UnifyWithGiven(const hCGivenType&);
};

class CAnyType:public CAtomType{
public:
  bool any() const              { return true; }
  bool final() const            { return false; }
  ret::utype_t unify(const hCType& type,CFDNodeBase * =0,CFDNodeBase * =0);
public:
  CAnyType():CAtomType(Predefined::any){}
public:
  ret::utype_t UnifyWithSpecial(const hCType&,CFDNodeBase *,CFDNodeBase *);
  ret::utype_t UnifyWithUnder(const hCUnderType&);
  ret::utype_t UnifyWithQuoted(const hCQuotedType&);
  ret::utype_t UnifyWithAtom(const hCAtomType&);
  ret::utype_t UnifyWithInternal(const hCInternalType&);
  ret::utype_t UnifyWithNone(const hCNoneType&);
  ret::utype_t UnifyWithGiven(const hCGivenType&);
};

class CGivenType:public CAtomType{
public:
  bool given() const            { return true; }
  bool final() const            { return false; }
  ret::utype_t unify(const hCType& type,CFDNodeBase * =0,CFDNodeBase * =0);
public:
  CGivenType():CAtomType(Predefined::given){}
public:
  ret::utype_t UnifyWithSpecial(const hCType&,CFDNodeBase *,CFDNodeBase *);
  ret::utype_t UnifyWithUnder(const hCUnderType&);
  ret::utype_t UnifyWithQuoted(const hCQuotedType&);
  ret::utype_t UnifyWithAtom(const hCAtomType&);
  ret::utype_t UnifyWithInternal(const hCInternalType&);
  ret::utype_t UnifyWithNone(const hCNoneType&);
  ret::utype_t UnifyWithNil(const hCNilType&);
  ret::utype_t UnifyWithAny(const hCAnyType&);
  ret::utype_t UnifyWithGiven(const hCGivenType&);
};

// Returned when unification of two special types failed.
class SSFail:public Fail{
public:
  SSFail(const hCType& type1,const hCType& type2):m_type1(type1),m_type2(type2){}
  string GetDesc() const;
private:
  hCType m_type1;
  hCType m_type2;
};

// Returned when unification of two normal types failed.
class NNFail:public Fail{
public:
  NNFail(const hCType& type1,const hCType& type2):m_type1(type1),m_type2(type2){}
  string GetDesc() const;
private:
  hCType m_type1,m_type2;
};

// Returned when unification of the internal and a normal types failed.
class INFail:public Fail{
public:
  explicit INFail(const hCType& type):m_type(type){}
  bool IsInternalNormal() const { return true; }
  string GetDesc() const;
private:
  hCType m_type;
};

#endif
