#ifndef _BASE_HANDLE_
#define _BASE_HANDLE_

#include <typeinfo>

#if !defined(NDEBUG) && defined(WIN32)
using std::cout;
using std::hex;
using std::dec;
#endif

template<class T> class Handle;
template<> class Handle<CMyObject>;

////////////////////////////////////////////////////////////////////////////////
// Template class for smart pointers-to-T objects;
// T must support the RCObject interface
// Implements implicit cast from Handle<T> to Handle<CMyObject>
//
template<class T>
class Handle
{
public:
	Handle(T *realPtr = 0):m_pointee(realPtr)             { init(); }
	Handle(const Handle& rhs):m_pointee(rhs.m_pointee)    { init(); }
	~Handle()
	{
		if (m_pointee) 
		{
			m_pointee->removeReference();
		}
	}
	Handle& operator=(const Handle& rhs)
	{
		reset(rhs.m_pointee);
		return *this; 
	}
	void reset(T *realPtr)
	{
		if (m_pointee != realPtr)
		{
			if (m_pointee) 
			{
				m_pointee->removeReference();
			}
			m_pointee = realPtr;
			init();
		}
	}
	operator bool() const                                 { return m_pointee != 0; }
	bool operator!() const                                { return m_pointee == 0; }
	T *pointee() const                                    { return m_pointee; }
	operator T*() const                                   { return m_pointee; }
	T *operator->() const                                 { return m_pointee; }
	T& operator*() const                                  { return *m_pointee; }
	operator Handle<CMyObject>() const;
#ifndef NDEBUG
	void dump() const 
	{
		cout << "HANDLE DUMP: " << typeid(*this).name() << " = [";
		cout << hex << int(m_pointee) << " (" << typeid(*m_pointee).name() << ") "
			<< (m_pointee ? m_pointee->refCount() : 0) << " ]\n" << dec;
	}
#endif
private:   
	T *m_pointee;

	void init()
	{   
		if (m_pointee == 0)
		{
			return;
		}
		m_pointee->addReference();
	}
};

////////////////////////////////////////////////////////////////////////////////
// Specialization of the above Handle<T> template class for T being CMyObject
//
template<>
class Handle<CMyObject>{
public:
	Handle(CMyObject *realPtr = 0):m_pointee(realPtr)     { init(); }
	Handle(const Handle& rhs):m_pointee(rhs.m_pointee)    { init(); }
	~Handle()                                   
	{
		if (m_pointee) 
		{
			m_pointee->removeReference();
		}
	}
	Handle& operator=(const Handle& rhs)                  
	{
		reset(rhs.m_pointee);
		return *this; 
	}
	void reset(CMyObject *realPtr)
	{
		if (m_pointee != realPtr)
		{
			if (m_pointee) 
			{
				m_pointee->removeReference();
			}
			m_pointee = realPtr;
			init();
		}
	}
	operator bool() const                                 { return m_pointee != 0; }
	bool operator!() const                                { return m_pointee == 0; }
	CMyObject *pointee() const                            { return m_pointee; }
	operator CMyObject*() const                           { return m_pointee; }
	CMyObject *operator->() const                         { return m_pointee; }
	CMyObject& operator*() const                          { return *m_pointee; }
#ifndef NDEBUG
	void dump() const 
	{
		cout << "HANDLE DUMP: " << typeid(*this).name() << " = [";
		cout << hex << int(m_pointee) << " (" << typeid(*m_pointee).name() << ") "
			<< (m_pointee ? m_pointee->refCount() : 0) << " ]\n" << dec;
	}
#endif
private:   
	CMyObject *m_pointee;

	void init()
	{   
		if (m_pointee == 0) 
		{
			return;
		}
		m_pointee->addReference();
	}
};

template<class T>
inline Handle<T>::operator Handle<CMyObject>() const
{
  return Handle<CMyObject>(m_pointee); 
}

////////////////////////////////////////////////////////////////////////////////
// Used to cast Handle<Src> to Handle<Dst>
//
template <class Dst,class Src>
inline Handle<Dst> cast_h2h(const Handle<Src>& src_handle)
{
	Dst *dst_ptr = dynamic_cast<Dst *>(src_handle.pointee());     // get the pointee of Handle<Src> and cast it to Dst*
	return Handle<Dst>(dst_ptr);                                  // return the respective Handle<Dst>
}

#endif
