#ifndef _PARSER_LEXER_
#define _PARSER_LEXER_

#include <stack.h>
#include <deque.h>

#ifdef getc
#undef getc
#endif
#ifdef putc
#undef putc
#endif

/*
  Reads a well parenthesized expression from the input stream.
  Fails if the input ends prematurely.
  The input is converted to a list like in scheme, containing
  tokens (keys in a hash set).
*/

struct Delim{
  static void init();
  static bool delim2token(int c,token_t& token);
  static bool is_delim(token_t token);
  static bool is_delim(int c);
  
  static token_t single_quote;
  static token_t double_quote;
  static token_t ws;
  static token_t open_paren;
  static token_t close_paren;
  static token_t open_curly;
  static token_t close_curly;
  static token_t tilde;
  static token_t colon;
  static token_t bar;
  static token_t at;
  static token_t percent;
  static token_t roof;
  static token_t star;
private:
  static int s_is_delim[INTDIV(256)];
  static token_t s_tokens[256];
};

#include "b_cyque.hxx"

class CLexer{
  ////////////////////////////////////////////////////////////////////////////////
  // All the read characters are cached in the error log to be available in case of 
  // an error.The error log is a circular queue.
  class CErrLog{
  public:
    CErrLog():m_chars(g_pTheEnv->m_nErrLogLen){}
    void reset()        { m_chars.resize(g_pTheEnv->m_nErrLogLen); }
    void put(char c)    { m_chars.push_back(c); }
    void unput()        { m_chars.pop_back(); }
    string get() const;
  private:
    cyque<char> m_chars;
  };
  ////////////////////////////////////////////////////////////////////////////////
  // I defined this class for mere convenience.It acts like a buffer which knows to
  // reset itself.
  class CBuffer{
  public:
    CBuffer():m_len(g_pTheEnv->m_nMaxTokenLen+2),m_buf(new char[m_len]){}
    ~CBuffer(){ delete[] m_buf; }
    void reset();
    char& operator[](int i){ return m_buf[i]; }
    const char *c_str() const { return m_buf; }
  private:
    int m_len;
    char *m_buf;
  };

public:
  CLexer();
  ~CLexer();
  void reset(){
    destroy();
    construct();
  }
  bool get(token_t& token) throw (Fail *);
  bool get(int& c);
  void putback(token_t token);
  void unget(){
    ungetc();
    m_errlog.unput();
  }
  bool ok() const       { return g_pTheInput->IsOK(); }
  string errlog() const { return m_errlog.get(); }

private:
  bool m_delim_next;
  bool m_is_string;
  TokenList m_putback;
  char *m_putback_str;     // --+
  int m_putback_len;       //   +-> these are for get(int&) in case a token is waiting in the putback queue
  int m_putback_i;         // --+
  CBuffer m_buf;
  CErrLog m_errlog;
  
private:
  bool get_delim(token_t& token);
  bool get_token(token_t& token) throw (Fail *);
  int read();
  void destroy();
  void construct();
  CLexer(const CLexer&);
  CLexer& operator=(const CLexer&);
  int getc()    { return g_pTheInput->Get(); }
  void ungetc() { g_pTheInput->Unget(); }

public:
  static bool s_bFlushTokenPutbackQueue;
};

extern CLexer *g_pTheLexer;

#endif
