#include "stdcfuf.h"
#include "b_lexer.hxx"
#include "b_tokhier.hxx"
#include <assert.h>
#include "b_debug_tracer.hxx"

RCObject::~RCObject(){}         // define the pure virtual destructor of RCObject

Env *g_pTheEnv = NULL;
CLexer *g_pTheLexer = NULL;
CTokenHier *g_pTheTokhier = NULL;

CInterp *g_pTheInterp = NULL;
CInputStream *g_pTheInput = NULL;
COutputStream *g_pTheOutput = NULL;

CInterp g_nullInterp;
CInputStream g_nullInput;
COutputStream g_nullOutput;

// maximum size of a Fail derived object is nFailObjMaxSize bytes
#ifdef WIN32
static const uint nFailObjMaxSize = 20;
#else
static const uint nFailObjMaxSize = 16;
#endif

// there can be no more than 10 simultaneous Fail objects (actual figure is 3)
static const uint nFailObjMaxNum = 10;

union Cell             // Any Fail object should fit in a Cell
{
  char space[nFailObjMaxSize];
  Cell *next;
  Cell():next(0){}
};
static Cell buf[nFailObjMaxNum];// the buffer of Fail objects
static Cell *avail = buf;       // pointer to the first available cell
static uint n = 0;              // the number of allocated cells (for statistics and consistency check)
class InitFailBuf
{
public:
	InitFailBuf()
	{
		assert(nFailObjMaxSize == sizeof(Cell));
		Cell *ptr = buf;
		for (uint i=nFailObjMaxNum - 1;i;--i,++ptr) 
		{
			ptr->next = ptr+1;
		}
	}
	~InitFailBuf()
	{
		assert(n == 0);
	}
};
static InitFailBuf init_fail_buf; // this object Initializes the cell buffer
/////////////////////////////////////////////////////////////////////////////////
// Fail methods
//
void *Fail::operator new(size_t size)
{
	if (size > nFailObjMaxSize)       // be flexible about the size, allow for future extensions
	{
		cerr << "Warning: Fail object of size " << size << " is allocated.\n";
		return ::operator new(size);
	}
	++n;
	assert(avail);
	void *p = static_cast<void *>(avail);
	avail = avail->next;
	return p;
}
void Fail::operator delete(void *p)
{
	static uint start = reinterpret_cast<int>(buf);
	static uint end = reinterpret_cast<int>(buf+nFailObjMaxNum);
	uint cur = reinterpret_cast<int>(p);
	if (start <= cur && cur < end)
	{
		Cell *cell = static_cast<Cell *>(p);
		cell->next = avail;
		avail = cell;
		--n;
	}
	else
	{
		cerr << "Warning: out-of-bounds Fail object is deallocated.\n";
		::operator delete(p);
	}
}

Symbols::symhash_t Symbols::s_symbols;
/////////////////////////////////////////////////////////////////////////////////
// Symbols methods
//
void Symbols::clean()
{
	while (s_symbols.size())
	{
		char *str = const_cast<char *>(*s_symbols.begin());
		s_symbols.erase(s_symbols.begin());
		delete[] str;
	}
}
char *Symbols::insert(const char *str)
{
	symhash_t::iterator found = s_symbols.find(str);
	if (found!=s_symbols.end()) return (char *)*found;
	int len = strlen(str) + 1;
	char *dup = new char[len];
	memcpy(dup,str,len);
	s_symbols.insert(dup);
	return dup;
}

/////////////////////////////////////////////////////////////////////////////////
// EdgeList methods
//
string EdgeList::GetString() const
{
	if (empty())
	{
		return "";
	}
	string str;
	const_iterator itElt=begin();
	for (uint i=size()-1;i;--i,++itElt) str << *itElt << ' ';
	return str << *itElt;
}

/////////////////////////////////////////////////////////////////////////////////
// CPath methods
//
CPath::CPath(int upcount):m_upcount(upcount)
{
	reserve(3);
}
CPath::CPath(int upcount,const EdgeList& edges):
	EdgeList(edges),m_upcount(upcount){}
CPath::CPath(int upcount,const iterator& begin,const iterator& end):
	EdgeList(begin,end),m_upcount(upcount){}
string CPath::toString() const
{
	if (m_upcount==CUR && size()==1) return TOK2STR(front());

	string str;

	if (m_upcount != CUR)
	{
		str << '{';
		if (m_upcount > CUR)
		{
			str << '^';
			if (m_upcount>1) 
			{
				str << m_upcount;
			}
			if (size()) 
			{
				str << ' ';
			}
		}
		str << EdgeList::GetString() << '}';
	}
	else if (size()) 
	{
		str << '(' << EdgeList::GetString() << ')';
	}
	return str;
}

/////////////////////////////////////////////////////////////////////////////////
// InterruptFail functions
//
string InterruptFail::GetDesc() const { return "Interrupted by the user"; }

/////////////////////////////////////////////////////////////////////////////////
// Miscellaneous functions
//
string MakeStringFromInt(int _n)
{
	const int BUFSIZE = 20;
	static char szBuf[BUFSIZE];
	szBuf[BUFSIZE-1] = '\0';

	int64 n = _n;
	if (_n < 0) 
	{
		n = -n;
	}

	uint i = BUFSIZE-1;
	do
	{
		--i;
		szBuf[i] = '0'+n%10;
		n/=10;
	}
	while (i && n);
	assert(i);

	if (_n < 0)
	{
		szBuf[--i] = '-';
	}

	return string(szBuf+i);
}
