#ifndef _BASE_OBJECTS_
#define _BASE_OBJECTS_

#include "b_symbols.hxx"
#include "b_predefined.hxx"
#include <vector.h>
#include <string>

////////////////////////////////////////////////////////////////////////////////
// base class for reference-counted objects
//
class RCObject{
public:
  void addReference() const     { ++m_refCount; }
  void removeReference() const  { if (--m_refCount == 0) delete this; }
  int refCount() const          { return m_refCount; }    // this one is for diagnostics
protected:   
  RCObject():m_refCount(0){}
  RCObject(const RCObject& rhs):m_refCount(0){}
  RCObject& operator=(const RCObject& rhs){ return *this; }
  virtual ~RCObject() = 0;   
private:   
  mutable int m_refCount;   
};

////////////////////////////////////////////////////////////////////////////////
// base class for most of the small objects and all which can be accessed
// through handles.
//
class CMyObject:public RCObject{
public:
  virtual ~CMyObject(){}
  virtual string toString() const =0;
};

#include "b_handle.hxx"

typedef Handle<CMyObject> hCObject;

template <class T>
inline T *cast(CMyObject *obj)            { return dynamic_cast<T *>(obj); }

////////////////////////////////////////////////////////////////////////////////////
// Address is something that can be on the left side of an attribute-value pair.
// It is either an arc or a path
typedef CMyObject CAddr;

class CArc:public CAddr{
protected:
  token_t m_title;
public:
  CArc(){}
  explicit CArc(token_t title):m_title(title){}
  token_t title() const          { return m_title; }
  void set_title(token_t title)  { m_title = title; }
  string toString() const               { return TOK2STR(m_title); }
};

class EdgeList:public vector<token_t>{
public:
  EdgeList(){}
  template<class InputIterator>
  EdgeList(InputIterator first,InputIterator last):vector<token_t>(first,last){}
  EdgeList(const_iterator first, const_iterator last):vector<token_t>(first,last){}
  EdgeList(const EdgeList& r):vector<token_t>(r){}
  string GetString() const;
};
typedef EdgeList TokenList;
typedef hash_set<token_t,hashtoken,eqtokens> TokenHash;

class CPath:public CAddr,public EdgeList{
public:
  enum {ABS=-1,CUR=0};
protected:
  int m_upcount;
public:
  explicit CPath(int upcount=ABS);
  CPath(int upcount,const EdgeList& edges);
  CPath(int upcount,const iterator& begin,const iterator& end);
  void set_upcount(int up)      { m_upcount = up; }
  int upcount() const           { return m_upcount; }
  string toString() const;
};

inline CArc *obj2arc(CMyObject *obj)      { return cast<CArc>(obj); }
inline CPath *obj2path(CMyObject *obj)    { return cast<CPath>(obj); }

typedef Handle<CAddr> hCAddr;
typedef Handle<CArc> hCArc;
typedef Handle<CPath> hCPath;
////////////////////////////////////////////////////////////////////////////////////

string MakeStringFromInt(int n);

inline string& operator<<(string& str,CMyObject *obj)           { return str+=(obj ? obj->toString() : "NULL"); }
inline string& operator<<(string& str,const CMyObject& obj)     { return str+=obj.toString(); }
inline string& operator<<(string& str1,const string& str2)      { return str1+=str2; }
inline string& operator<<(string& str,char c)                   { return str+=c; }
inline string& operator<<(string& str,char *s)                  { return str+=s; }
inline string& operator<<(string& str,int n)                    { return str+=MakeStringFromInt(n); }

inline ostream& operator<<(ostream& os,CMyObject *obj)          { return os << (obj ? obj->toString() : "NULL"); }
inline ostream& operator<<(ostream& os,const CMyObject& obj)    { return os << obj.toString(); }

inline void delete_obj(CMyObject *obj){ delete obj; }

template <class T>
inline ostream& operator<<(ostream& os,const Handle<T>& handle)  { return os << handle.pointee(); }
template <class T>
inline string& operator<<(string& str,const Handle<T>& handle)   { return str << handle.pointee(); }

#include "b_env.hxx"
#include "b_fail.hxx"
#include "b_deftypes.hxx"

class CInputStream{             // Generic input stream for parsing inputs
public:
  virtual ~CInputStream(){}
  virtual int Get()   { return -1; }    // get the next character in stream or EOF
  virtual void Unget(){}                // undo the last get (multiple consecutive calls to unget() have the same effect as one call)
  virtual bool IsOK() { return false; } // true if the stream is not past end of file and no failures have occurred (good() in istreams)
};

const char ENDL = '\n';
class COutputStream{
public:
  virtual ~COutputStream(){}
  virtual COutputStream& operator<<(const string&)        { return *this; }
  virtual COutputStream& operator<<(const char *str)      { return *this; }
  virtual COutputStream& operator<<(char c)               { return *this; }
  virtual COutputStream& operator<<(long l)               { return *this; }
  virtual COutputStream& operator<<(const CMyObject& obj) { return *this; }
  virtual COutputStream& operator<<(const CMyObject *obj) { return *this; }
  virtual void flush(){}
};

class CInterp			// Generic interpreter for control and extern constructs
{
public:
	virtual ~CInterp(){}
	virtual bool EvalControl(const char *const exp)          { return true; }
	virtual CInputStream *EvalExtern(const char *const exp)  { return NULL; }
};

extern Env *g_pTheEnv;
extern COutputStream *g_pTheOutput;
extern CInputStream *g_pTheInput;
extern CInterp *g_pTheInterp;

extern CInterp g_nullInterp;
extern CInputStream g_nullInput;
extern COutputStream g_nullOutput;

class CSwitchInputStream
{
public:
	CSwitchInputStream(CInputStream *pNewInputStream)
	{
		m_pOldInputStream = g_pTheInput;
		g_pTheInput = pNewInputStream;
	}
	~CSwitchInputStream()
	{
		g_pTheInput = m_pOldInputStream;
	}
private:
	CInputStream *m_pOldInputStream;
};

#endif
