#ifndef _BASE_TOKHIER_
#define _BASE_TOKHIER_

#include "b_bits.hxx"
#include <hash_map.h>
#include <algo.h>
#include <assert.h>

/*
  Tokens hash map contains pairs <token_t& title,int irow>, where token is the key and irow is the data.
  The irow is the index of the token's row in the bit matrix.Given two tokens their row indices are
  retrieved, the respective rows in the matrix are then combined to yield another row.
  Bits hash map contains pairs <int *row,token_t& token>, where row is the key and token is the data.
  The row as received above is then looked up in the bits hash map and if it is there then the found
  token is the requested one.
  The token strings are shared by the two hash maps, as well as the rows are shared between the bits
  hash map and the bits matrix (which is the rightful owner of them).
  So the bits hash must not delete its keys and data when it is deleted.
*/

// Comparison function for the bit rows
struct eqrows{
  CBitMatrix *m_pBits;
  eqrows(CBitMatrix *bits=NULL):m_pBits(bits){}
  bool operator()(const int *row1, const int *row2) const{
    return equal(row1,row1+m_pBits->size_int(),row2);
  }
};

// hash function for the bit rows
struct hashrow{
  CBitMatrix *m_pBits;
  hashrow(CBitMatrix *bits=NULL):m_pBits(bits){}
  size_t operator()(const int *row) const{
    size_t acc=0;
    for (int i=m_pBits->size_int()-1;i>=0;i--) acc^=row[i];
    return acc;
  }
};

struct TokenRealizedFail:public Fail{
  string GetDesc() const { return "the type system has already been Realized"; }
};
struct TokenNotYetFail:public Fail{
  string GetDesc() const { return "the type system has not been Realized yet"; }
};
struct TokenTooManyFail:public Fail{
  string GetDesc() const { return "too many types"; }
};
struct TokenCycleFail:public Fail{
  string GetDesc() const { return "the last pair has closed a cycle"; }
};

typedef hash_map<const int*, token_t, hashrow, eqrows> CBitrow2token;
typedef hash_map<token_t, int, hashtoken, eqtokens> CToken2index;

class CTokenHier{
public:
  explicit CTokenHier(int max=0);
  ~CTokenHier();
  void reset(int max=0){
    destroy();
    construct(max ? max : m_nTokensCount);
  }
  int count() const { return m_nTokensCount; }
  int capacity() const { return m_nMaxTokens; }
  Fail *insert_pair(token_t parent,token_t child);
  void Realize();
  ret::utoken_t get_glb(token_t,token_t);
private:
  int m_nMaxTokens;             // the type system capacity
  int m_nTokensCount;           // the current token count
  bool m_bRealized;             // true if the system was Realized
  CToken2index *m_phmTokens;    // maps tokens to the rows indexes in the bit matrix
  CBitMatrix *m_pBits;          // the bit matrix (each row is a type)
  int *m_result;                // the bitrow resulting from unifying two tokens (use m_phmBits to get the token)
  eqrows m_eqrows;
  hashrow m_hashrow;
  CBitrow2token *m_phmBits;     // maps bitrows to tokens
private:
  ret::uint_t add_token(token_t);
  void destroy();
  void construct(int max);
};

extern CTokenHier *g_pTheTokhier;

#endif
