#include "stdcfuf.h"

#define SORT

#include "b_debug_tracer.hxx"
#include "c_extfd.hxx"
#include "c_cset.hxx"
#include "b_bkclass.hxx"
#include "c_fdnode2path.hxx"
#include "c_genericset.hxx"
#include "t_tracer_l3.hxx"

static CFDNode *s_from = 0;

class FDAddCsetFail:public FDCsetFail{
public:
  FDAddCsetFail(const hCCsetType& _ctype,const hCCset& _cset):ctype(_ctype),cset(_cset){}
  string GetDesc() const {
    string str("Csets ");
    return str << ctype << " and " << cset << " are inconsistent!";
  }
private:
  hCCsetType ctype;
  hCCset cset;
};

class FDUnifyCsetsFail:public FDCsetFail{
public:
  FDUnifyCsetsFail(const hCCsetType& _ctype1,const hCCsetType& _ctype2):ctype1(_ctype1),ctype2(_ctype2){}
  string GetDesc() const {
    string str("Csets ");
    return str << ctype1 << " and " << ctype2 << " are inconsistent!";
  }
private:
  hCCsetType ctype1,ctype2;
};

static Fail *fail;
////////////////////////////////////////////////////////////////////////////////
// CCsetType methods
//
/*
  Cset unification definition:
   - cset1 = (abs1,base1,inc1,dec1)
   - cset2 = (abs2,base2,inc2,dec2)
  1. abs1 == () and abs2 == ()
     return ((),(base1 U base2),(inc1 U inc2),(dec1 U dec2))
  2. abs1 != () and abs2 == ()
     if ((inc1 U inc2) is included in abs1) and 
        ((dec1 U dec2) is mutually exclusive with abs1)
     then return (abs1,(),(),())
     else fail
  3. abs1 == () and abs2 != ()
     Same as 2, but abs1 and abs2 switch roles.
  4. abs1 != () and abs2 != ()
     if (abs1 != abs2) then fail
     else same as 2
*/
// Copy the cset r.Node references of the original are resolved from scratch in the copy.
// m_owner field of the copy is set to 0 (the caller is responsible for calling set_owner method)
CCsetType::CCsetType(CFDNode *owner,const CCsetType& r):m_owner(owner)
{
	fail = 0;
	s_from = r.m_owner;
	copy_component(m_abs,r.m_abs);
	copy_component(m_base,r.m_base);
	copy_component(m_inc,r.m_inc);
	copy_component(m_dec,r.m_dec);
}
CCsetType::CCsetType(CFDNode *owner,const hCCset& cset):m_owner(owner)
{
	fail = 0;
	init_component(m_abs,cset->abs());
	init_component(m_base,cset->base());
	init_component(m_inc,cset->inc());
	init_component(m_dec,cset->dec());
}
CCsetType::CCsetType(const hCCsetType& ctype,const hCCset& cset):m_owner(ctype->m_owner)
{
  fail = 0;
  CTypeCset cset1(ctype);
  COrigCset cset2(dynamic_cast<CExtFD *>(m_owner->fd()),cset);
  if (!unify(cset1,cset2)) fail = new FDAddCsetFail(ctype,cset);
}
CCsetType::CCsetType(const hCCsetType& ctype1,const hCCsetType& ctype2):m_owner(ctype1->m_owner)
{
  fail = 0;
  CTypeCset cset1(ctype1);
  CTypeCset cset2(ctype2);
  if (!unify(cset1,cset2)) fail = new FDUnifyCsetsFail(ctype1,ctype2);
}
ret::utype_t CCsetType::create(CFDNode *owner,const hCCset& cset)
{
  hCType res_type(new CCsetType(owner,cset));
  if (fail) return fail;
  return res_type;
}
ret::utype_t CCsetType::special_unify(const hCCset& cset)
{
  DBG_TRACE3(Tracer_l3::special_unify(this,cset.pointee()));
  hCType res_type(new CCsetType(this,cset));
  if (fail) return fail;
  return res_type;
}
ret::utype_t CCsetType::unify_with_special(const hCType& type,CFDNodeBase *,CFDNodeBase *)
{
  if (type->nil()) return ret::utype_t(this);
  CCsetType *ctype = cast<CCsetType>(type);
  if (!ctype) return new SSFail(this,type);
  DBG_TRACE3(Tracer_l3::special_unify(this,ctype));
  hCType res_type(new CCsetType(this,ctype));
  if (fail) return fail;
  return res_type;
}
// The cset is void at this point.It will be initialized step by step following the unification
// of the two generic csets.Both generic csets may be either of the type CCsetType (this is the
// case when CExtFD::AddPath() causes two csets to be unified) or one of CCsetType and the other
// of CCset (this is the case of CExtFD::AddSpecial()).
bool CCsetType::unify(GenericCset& cset1,GenericCset& cset2)
{
  CVisitedBitmap visited(m_owner->fd()->count());   // this is the master map
  GenericComponent& abs1 = cset1.abs();
  GenericComponent& abs2 = cset2.abs();
  bool ok = true;
  if (abs1.size()) ok = construct_abs(abs1,abs2,visited);
  else if (abs2.size()) ok = construct_abs(abs2,abs1,visited);
  if (!ok) return false;
  if (m_abs.empty()){                   // if the resulting absolute component is void - proceed with plan 1
    proceed_1(cset1,cset2);             // which should always succeed
    return true;
  }
  return proceed_2(cset1,cset2,visited);// else proceed with plan 2
}
/*
  Master absolute component is known to be not empty.Slave is allowed either be empty are set-equal
  to the master.Each cell in the slave is checked in the visited map (which in effect is a bitmap of
  the master).Every slave cell must be present in the master map and there must be precisely the same
  number of them as the number of master cells.
*/  
bool CCsetType::construct_abs(GenericComponent& master,GenericComponent& slave,CVisitedBitmap& visited)
{
  set_component(visited,m_abs,master);  // Initialize the absolute component with the master
  if (!slave.size()) return true;       // done if the slave is void
  uint count=0;
  CVisitedBitmap visited2(visited.size());  // prepare the slave map (tells if already saw a slave node)
  GenUnifyCsetIterator& slave_it = slave.begin();
  for (uint i=slave.size();i;--i,slave_it.advance()){  // iterate over the slave
    int id = slave_it.node()->group_id();  // get the unique id of the current slave node
    if (visited2[id]) continue;         // continue if this slave node has already been seen
    visited2[id]=true;                  // else update the slave map
    if (visited[id] && count < m_abs.size()) ++count;  // we succeed if the slave node is present in the
    else return false;                  // master map and there are less slave nodes than the master has
  }
  return count == m_abs.size();         // the number of slave nodes must be equal to that of the master
}
/////////////////////////////////////////////////////////////////////////////////
// Plan 1: 
// Compute the union of base,inc and dec components of the two csets.
void CCsetType::proceed_1(GenericCset& cset1,GenericCset& cset2)
{
  unite_components(m_base,cset1.base(),cset2.base());
  unite_components(m_inc,cset1.inc(),cset2.inc());
  unite_components(m_dec,cset1.dec(),cset2.dec());
}
void CCsetType::unite_components(CCsetTypeComponent& dst,GenericComponent& src1,GenericComponent& src2)
{
  CVisitedBitmap visited(m_owner->fd()->count());  // Initialize the map 
  set_component(visited,dst,src1); // first copy src1 and create the respective map
  set_component(visited,dst,src2); // then copy src2 using the map to avoid copying the same cells twice
}
// Plan 1 is done.
/////////////////////////////////////////////////////////////////////////////////
// Plan 2:
// Validate that the union of:
//   1) inc components is set-included in the abs component;
//   2) dec components is mutually exclusive with the abs component.
bool CCsetType::proceed_2(GenericCset& cset1,GenericCset& cset2,CVisitedBitmap& visited)
{
  return (validate_inclusion(visited,cset1.inc()) &&
          validate_inclusion(visited,cset2.inc()) &&
          validate_mutex(visited,cset1.dec()) &&
          validate_mutex(visited,cset2.dec()));
}
bool CCsetType::validate_inclusion(CVisitedBitmap& visited,GenericComponent& slave)
{
  GenUnifyCsetIterator& slave_it = slave.begin();
  for (uint i=slave.size();i;--i,slave_it.advance()){   // iterate over the slave
    int id = slave_it.node()->group_id();               // get the unique id of the current slave node
    if (!visited[id]) return false;                     // fail if it is absent from the master map
  }
  return true;
}
bool CCsetType::validate_mutex(CVisitedBitmap& visited,GenericComponent& slave)
{
  GenUnifyCsetIterator& slave_it = slave.begin();
  for (uint i=slave.size();i;--i,slave_it.advance()){   // iterate over the slave
    int id = slave_it.node()->group_id();               // get the unique id of the current slave node
    if (visited[id]) return false;                      // fail if it is present in the master map
  }
  return true;
}
// Plan 2 is done.
/////////////////////////////////////////////////////////////////////////////////
void CCsetType::init_component(CCsetTypeComponent& dst,const GenCsetComponent& _src)
{
	const CCsetComponent& src = dynamic_cast<const CCsetComponent&>(_src);
	CVisitedBitmap visited(m_owner->fd()->count());
	CCsetComponent::const_iterator src_it=src.begin();
	CExtFD *hostfd = dynamic_cast<CExtFD *>(m_owner->fd());
	assert(hostfd);
	for (uint i=src.size();i;--i,++src_it)
	{
		CFDNode *node = hostfd->SoftPeekAddr(*src_it);  // might add new nodes to the graph
		int id=node->group_id();
		if (visited[id]) continue;
		visited[id]=true;
		dst.push_back(node);
	}
}
// Initializes the dst cset component from the src using the visited map as the reference.
// Only those source cells which are not marked in the reference map as already visited
// get copied to dst.Every added cell is reflected in the visited map.
void CCsetType::set_component(CVisitedBitmap& visited,CCsetTypeComponent& dst,GenericComponent& src)
{
  GenUnifyCsetIterator& src_it = src.begin();
  for (uint i=src.size();i;--i,src_it.advance()){ // iterate over the src component
    int id = src_it.node()->group_id();         // get the unique id of the current src node
    if (visited[id]) continue;                  // skip it if dst component already has it
    visited[id]=true;                           // else update the visited map
    dst.push_back(src_it.node());               // and update the dst component
  }
}
void CCsetType::copy_component(CCsetTypeComponent& dst,const CCsetTypeComponent& src)
{
  dst.reserve(src.size());
  CCsetTypeComponent::const_iterator src_it = src.begin();
  CExtFD *hostfd = dynamic_cast<CExtFD *>(m_owner->fd());
  assert(hostfd);
  for (uint i=src.size();i;--i,++src_it){
    CPath *path = Node2Path::get_rel_path(s_from,*src_it,false);// do not fix the relative path
    CFDNode *dst_node = hostfd->SoftPeekPath(path);    // get the respective node in the current fd
    dst.push_back(dst_node);
  }
}
string CCsetType::toString() const
{
  assert(m_owner);
  s_from = m_owner;
  return GenCset::str();
}
Fail *CCsetType::add_self(CExtFD *fd) const
{
  CCset cset;   // no need to define a dynamic object, since no one is going to keep it, but a handle
  cset.addReference();  // prevent the handle of AddSpecial() from destroying it
  reverse_init_component(cset.m_abs,m_abs);
  reverse_init_component(cset.m_base,m_base);
  reverse_init_component(cset.m_inc,m_inc);
  reverse_init_component(cset.m_dec,m_dec);
  return fd->AddSpecial(Predefined::cset,hCCset(&cset));
}
void CCsetType::reverse_init_component(CCsetComponent& dst,const CCsetTypeComponent& src) const
{
  dst.reserve(src.size());
  CCsetTypeComponent::const_iterator src_it = src.begin();
  for (uint i=src.size();i;--i,++src_it){
    CPath *path = Node2Path::get_rel_path(m_owner,*src_it,false);// do not fix the relative path
    dst.push_back(new CPath(*path));
  }
}
bool CCsetType::is_equal(CType *type) const
{
  CCsetType *ctype = cast<CCsetType>(type);
  if (!ctype) return false;
  CFDNode *dstroot = ctype->m_owner->fd()->root();
  return (is_equal_component(m_abs,ctype->m_abs,dstroot) &&
          is_equal_component(m_base,ctype->m_base,dstroot) &&
          is_equal_component(m_inc,ctype->m_inc,dstroot) &&
          is_equal_component(m_dec,ctype->m_dec,dstroot));
}
// Here I have to compare the node references in different host fds.
// I could have compared absolute paths, but there are may be more
// than one absolute path to a node.So I do this:
//  - get one absolute path to a node from the source component
//  - get the node at this path in the destination fd
//  - compare that node to the current destination node
bool CCsetType::is_equal_component(const CCsetTypeComponent& src,
                                   const CCsetTypeComponent& dst,
                                   CFDNode *dstroot) const
{
  if (src.size() != dst.size()) return false;
  CCsetTypeComponent::const_iterator src_it = src.begin();
  CCsetTypeComponent::const_iterator dst_it = dst.begin();
  for (uint i=src.size();i;--i,++src_it,++dst_it){
    CPath *srcpath = Node2Path::get_abs_path(*src_it);  // get an absolute path to the current source node
    CFDNode *checknode = dstroot->FindDescendant(srcpath->begin(),srcpath->end(),true);  // get the check node
    if (!checknode || !checknode->is_equiv(*dst_it))    // fail if the check node and the current dst node differ
      return false;    
  }
  return true;
}

#ifdef SORT
#include <algo.h>

static bool comp(const string& s1,const string& s2)
{
  return strcmp(s1.c_str(),s2.c_str()) < 0;
}
#endif

/////////////////////////////////////////////////////////////////////////////////
// CCsetTypeComponent methods
//
string CCsetTypeComponent::str() const
{
	if (empty())
	{
		return "";
	}
	string str;
#ifdef SORT
	vector<string> sorted;
	sorted.reserve(size());
	{
		const_iterator itCsetNode=begin();
		for (uint i = size();i;--i,++itCsetNode) 
		{
			sorted.push_back(Node2Path::get_rel_path(s_from,*itCsetNode,true)->toString());
		}
	}
	sort(sorted.begin(),sorted.end(),comp);
	vector<string>::const_iterator it=sorted.begin();
	for (uint i = size() - 1;i;--i,++it)
	{
		str << *it << ' ';
	}
	return str << *it;
#else
	const_iterator itCsetNode=begin();
	for (uint i = size() - 1;i;--i,++itCsetNode)
	{
		str << Node2Path::get_rel_path(s_from,*itCsetNode,true) << ' ';
	}
	return str << Node2Path::get_rel_path(s_from,*itCsetNode,true);
#endif
}

////////////////////////////////////////////////////////////////////////////////
// FDCsetFail methods
//
FDCsetFail::FDCsetFail()
{
  assert(BKClass::fail_bk() == 0);            // assert that the failure log is clean
  BKClass::add_fail_bk(Predefined::cset);     // acknowledge the failed edge
}
