#include "stdcfuf.h"
#include "c_fdnode.hxx"
#include "c_barefd.hxx"
#include "c_fdwalk.hxx"
#include "c_fdcmp.hxx"

class CCompareFDs{
  struct FailInfo{
    FailInfo(token_t edge){
      FDCompare::s_failPath[FailInfo::s_nSide].push_back(edge);
    }
    ~FailInfo(){
      if (!s_bFailed) FDCompare::s_failPath[FailInfo::s_nSide].pop_back();
    }
    static void Init(uint nSide){
      assert(!s_bFailed);
      s_nSide = nSide;
    }
    bool Failed(CFDNode *pNode){
      FDCompare::s_pFailNode[s_nSide] = pNode;
      s_bFailed = true;
      return false;
    }
    static bool s_bFailed;
    static uint s_nSide;
  };
  
public:
  enum {MASTER = 0,SLAVE = 1};
  typedef pair<token_t,CFDNode *> child_type;

  CCompareFDs(CFDNode *pNodeMaster,CFDNode *pNodeSlave);

  bool m_bEqual;

  static bool comp(const child_type& x,const child_type& y){
    return TOK2STR(x.first) > TOK2STR(y.first);
  }
private:
  vector<bool> m_nil[2];                // bitmaps of nil nodes for the two fds to be compared
  hash_map<int,int> m_master2slave;     // holds master->slave equality relationship (given master get slave)
  hash_map<int,int> m_slave2master;     // holds slave->master equality relationship (given slave get master)

  bool IsEqual(CFDNode *pNodeMaster,CFDNode *pNodeSlave);
  void CollectChildren(CFDNode *pParent,vector<child_type>& rColl);
  // These functions verify that a node or a collection of nodes is not ground
  bool IsNil(vector<child_type>& coll,uint nSide/* MASTER or SLAVE */);
  bool IsNil(const child_type& child);
};

bool CCompareFDs::FailInfo::s_bFailed = false;
uint CCompareFDs::FailInfo::s_nSide = (uint)-1;
/////////////////////////////////////////////////////////////////////////////////
// CCompareFDs methods
//
CCompareFDs::CCompareFDs(CFDNode *pNodeMaster,CFDNode *pNodeSlave)
{
  m_nil[MASTER].insert(m_nil[MASTER].end(),pNodeMaster->fd()->count(),false);
  m_nil[SLAVE].insert(m_nil[SLAVE].end(),pNodeSlave->fd()->count(),false);
  m_bEqual = IsEqual(pNodeMaster,pNodeSlave);
}
bool CCompareFDs::IsEqual(CFDNode *pNodeMaster,CFDNode *pNodeSlave)
{
	vector<child_type> collChildren[2];
	CollectChildren(pNodeMaster,collChildren[MASTER]); // collect and sort the master children
	CollectChildren(pNodeSlave,collChildren[SLAVE]);   // collect and sort the slave children

	FDCompare::s_pFailNode[MASTER] = 0;   // A zeroed s_pFailNode[i] means there was no failure on the side i
	FDCompare::s_pFailNode[SLAVE] = 0;
	uint i;

	while (collChildren[MASTER].size() && collChildren[SLAVE].size())      // iterate on the children
	{
		child_type child[2];
		token_t edge[2];

		for (i = 0; i < 2; ++i)						// for MASTER and SLAVE do:
		{
			child[i] = collChildren[i].back();      // get a child node
			edge[i] = child[i].first;               // get the child's edge
			FDCompare::s_failPath[i].push_back(edge[i]);  // update the path to the child (relative to the roots given to the constructor)
		}

		if (edge[MASTER] == edge[SLAVE])            // the edges are the same
		{
			CGroupData *pGDataChild[2];               // child group data
			CType *pTypeChild[2];                     // child types
			int gid[2];                               // group ids

			for (i = 0; i < 2; ++i)                      // for MASTER and SLAVE do:
			{
				pGDataChild[i] = &child[i].second->data();      // get the group data of the respective child
				pTypeChild[i] = pGDataChild[i]->GetType();     // get the child's type
				gid[i] = (pTypeChild[i]->final() ?              // get the child's group id: final nodes are considered  
				child[i].second->id() :               // individually as opposed to others, for which respective
				pGDataChild[i]->id());                // node groups are taken (this matches m_bSplitPaths setting)
			}

			hash_map<int,int>::const_iterator foundMaster = m_master2slave.find(gid[MASTER]); // get history
			hash_map<int,int>::const_iterator  foundSlave = m_slave2master.find(gid[SLAVE]);
			if (foundMaster == m_master2slave.end() && foundSlave == m_slave2master.end()) // if no history on the two nodes
			{
				m_master2slave[gid[MASTER]] = gid[SLAVE];                 // update the history for the master and the slave
				m_slave2master[gid[SLAVE]] = gid[MASTER];

				bool cont = (pTypeChild[MASTER]->is_equal(pTypeChild[SLAVE]) ||
				pTypeChild[MASTER]->internal() && pTypeChild[SLAVE]->nil() ||    // make (a ()) and (a nil) be equal
				pTypeChild[MASTER]->nil() && pTypeChild[SLAVE]->internal());

				if (!cont)                                              // the types differ - 
				{
					FDCompare::s_pFailNode[MASTER] = child[MASTER].second;// set the fail nodes
					FDCompare::s_pFailNode[SLAVE] = child[SLAVE].second;
					return false;                                         // exit with failure
				}

				if (!IsEqual(child[MASTER].second,child[SLAVE].second))
				{
					return false;
				}
			}
			else if (foundMaster == m_master2slave.end() ||          // partial or 
					foundMaster->second != gid[SLAVE])             // not matching history
			{
				FDCompare::s_pFailNode[MASTER] = child[MASTER].second;  // set the fail nodes
				FDCompare::s_pFailNode[SLAVE] = child[SLAVE].second;
				return false;                                           // yields failure
			}

			for (i = 0; i < 2; ++i)				// all ok - for MASTER and SLAVE do:
			{
				collChildren[i].pop_back();     // proceed to the next master-slave child pair
				FDCompare::s_failPath[i].pop_back(); // undo the path change
			}
		}
		else
		{
			uint nSide = TOK2STR(edge[MASTER]) < TOK2STR(edge[SLAVE]) ? MASTER : SLAVE; // get the leading side (MASTER or SLAVE)
			FDCompare::s_failPath[nSide].pop_back();  // undo the fail path change (IsNil takes care of it)
			FailInfo::Init(nSide);
			if (!IsNil(child[nSide])) 
			{
				return false;   // verify that it is non ground
			}
			collChildren[nSide].pop_back();           // if indeed - proceed to the next child on this side
		}
	}

	if (collChildren[MASTER].size())
	{
		return IsNil(collChildren[MASTER],MASTER);
	}
	if (collChildren[SLAVE].size()) 
	{
		return IsNil(collChildren[SLAVE],SLAVE);
	}
	return true;
}
bool CCompareFDs::IsNil(const child_type& child)
{
  FailInfo f(child.first);

  {     // do not carry unnecessary stuff on stack during recursive calls
    CGroupData& gdata = child.second->data();
    int gid = gdata.id();
    if (m_nil[FailInfo::s_nSide][gid]) return true;
    
    CType *pType = gdata.GetType();
    if (!pType->nil() && (!pType->internal() || pType->any())) // an internal node may be ANY too
      return f.Failed(child.second);
    
    m_nil[FailInfo::s_nSide][gid] = true;
    if (pType->nil()) return true;
  }
  
  for (CFDChildIterator it(child.second);!it.is_past_the_end();it.advance())
    if (!IsNil(child_type(it->first->title(),it->second))) return false;
  
  return true;
}
bool CCompareFDs::IsNil(vector<child_type>& coll,uint nSide/* MASTER or SLAVE */)
{
  FailInfo::Init(nSide);
  while (coll.size() && IsNil(coll.back())) coll.pop_back();
  return coll.empty();
}
void CCompareFDs::CollectChildren(CFDNode *pParent,vector<child_type>& rColl)
{
  for (CFDChildIterator it(pParent);!it.is_past_the_end();it.advance())
    rColl.push_back(child_type(it->first->title(),it->second));
  sort(rColl.begin(),rColl.end(),comp);
}

CFDNode *FDCompare::s_pFailNode[2];
EdgeList FDCompare::s_failPath[2];
////////////////////////////////////////////////////////////////////////////////
// FDCompare methods
//
bool FDCompare::IsEqual(CFDNode *pRoot0,const EdgeList& path0,
                        CFDNode *pRoot1,const EdgeList& path1)
{
  InitPath(s_failPath[0],path0);
  InitPath(s_failPath[1],path1);
  
  CCompareFDs compare(pRoot0,pRoot1);
#ifndef NDEBUG
  bool bFailed = (FDCompare::s_pFailNode[0] || FDCompare::s_pFailNode[1]);
  assert(compare.m_bEqual == !bFailed);
#endif
  return compare.m_bEqual;
}
void FDCompare::InitPath(EdgeList& rDstPath,const EdgeList& srcPath)
{
  rDstPath.erase(rDstPath.begin(),rDstPath.end());
  rDstPath.insert(rDstPath.end(),srcPath.begin(),srcPath.end());
}
