#include "stdcfuf.h"

// #define F_TRACE

#include "b_debug_tracer.hxx"
#include "c_fditerator.hxx"
#include "c_barefd.hxx"
#include "c_fddump.hxx"
#include "c_tkdot.hxx"
#include <algo.h>
#ifdef WIN32
#include <ostream.h>
#else
#include <ostream.h>
#endif

struct Dump
{
  virtual void Prefix(ostream& os) const {}
  virtual void Suffix(ostream& os) const {}
  virtual void OutputNode(ostream& os,int idNode,const char *szLabel,CType *pType) const =0;
  virtual void OutputEdge(ostream& os,int idNode1,int idNode2,const char *szLabel) const =0;
  virtual void Summary(ostream& os,int nodeCountActual,int nodeCountDisplayed,int edgeCountDisplayed) const =0;
};

struct DumpTkdot:Dump
{
  void Prefix(ostream& os) const { OutputTKDot::Prefix(os); }
  void Suffix(ostream& os) const { OutputTKDot::Suffix(os); }
  void OutputNode(ostream& os,int idNode,const char *szLabel,CType *pType) const
  {
    os << "set n(" << idNode << ")\t[$g addnode label \"" << szLabel << "\"";
    if (pType->internal()) os << " width .15 height .15";
    os << "]\n";
  }
  void OutputEdge(ostream& os,int idNode1,int idNode2,const char *szLabel) const
  {
    os << "$n(" << idNode1 << ") addedge $n(" << idNode2 << ")\tlabel \"" << szLabel << "\"\n";
  }
  void Summary(ostream& os,int nodeCountActual,int nodeCountDisplayed,int edgeCountDisplayed) const
  {
    os << "#[Total nodes: " << nodeCountActual << "]\n";        // the number of actual CFDNode objects in the graph
    os << "#[Graph nodes: " << nodeCountDisplayed << "]\n";     // the number of nodes in the graph displayed
    os << "#[Graph edges: " << edgeCountDisplayed << "]\n\n";   // the number of edges in the graph
  }
};

struct DumpDot:Dump
{
  void Prefix(ostream& os) const { os << "digraph G {\n"; }
  void Suffix(ostream& os) const { os << "}\n"; }
  void OutputNode(ostream& os,int idNode,const char *szLabel,CType *pType) const
  {
    os << "  n" << idNode << "\t[label=\"" << szLabel << "\"";
    if (pType->internal()) os << " width = .15 height = .15";
    os << "];\n";
  }
  void OutputEdge(ostream& os,int idNode1,int idNode2,const char *szLabel) const
  {
    os << "  n" << idNode1 << " -> n" << idNode2 << "\t[label=\"" << szLabel << "\"];\n";
  }
  void Summary(ostream& os,int nodeCountActual,int nodeCountDisplayed,int edgeCountDisplayed) const
  {
    os << "/*\n   [Total nodes: " << nodeCountActual << "]\n";          // the number of actual CFDNode objects in the graph
    os << "   [Graph nodes: " << nodeCountDisplayed << "]\n";           // the number of nodes in the graph displayed
    os << "   [Graph edges: " << edgeCountDisplayed << "]\n*/\n";       // the number of edges in the graph
  }
};

static void DoDump(ostream& os,CFDNode *root,bool bDoingAll,Dump *pDumpObj);

void FDDump::fd2tkdot(ostream& os,CFDNode *root,bool bDoingAll)
{
  static DumpTkdot tkdotDump;
  DoDump(os,root,bDoingAll,&tkdotDump);
}
void FDDump::fd2dot(ostream& os,CFDNode *root,bool bDoingAll)
{
  static DumpDot dotDump;
  DoDump(os,root,bDoingAll,&dotDump);
}

void DoDump(ostream& os,CFDNode *root,bool bDoingAll,Dump *pDumpObj)
{
  pDumpObj->Prefix(os);
  ///////////////////////////////////////////////////////////////////////////////
  // List all the nodes first
  
  vector<CFDNode *> nodes;                      // holds the fd nodes
  nodes.reserve(root->fd()->count());           // reserve enough space for all the nodes
  for (CFDGraphIterator it(root);!it.is_past_the_end();it.advance()){   // iterate on the fd graph
    hCType type = it->data().GetType();                // get the node type
    if (!bDoingAll && type->special()) continue;     // skip special nodes if so required
    string label;
    if (type->internal()) label = type->any() ? "*" : "";
    else if (type->quoted()){
      CQuotedType *quoted = cast<CQuotedType>(type);    // check if the node is of quoted type
      assert(quoted);
      label << "\\\"" << quoted->title() << "\\\"";     // to handle the double quotes specially
    }else label = type->toString();
    pDumpObj->OutputNode(os,it->group_id(),label.c_str(),type);
    nodes.push_back(it);                // cache the node for later use
  }
  os << endl;
  
  // End of the node list
  ///////////////////////////////////////////////////////////////////////////////
  // List all the edges next

  bool (CFDChildIterator::*halter)() const = bDoingAll ? &CFDChildIterator::is_past_the_end : &CFDChildIterator::is_special;
  int edgeCount=0;
    
  for (vector<CFDNode *>::iterator nd_it=nodes.begin();nd_it!=nodes.end();++nd_it){     // iterate the cached nodes
    for (CFDChildIterator ch_it(*nd_it);!(ch_it.*halter)();ch_it.advance()){            // visite the children of each node
      ++edgeCount;                                                                      // update the edge count
      pDumpObj->OutputEdge(os,(*nd_it)->group_id(),ch_it->second->group_id(),ch_it->first->title());
    }
  }
  
  // End of the edge list
  ///////////////////////////////////////////////////////////////////////////////
  pDumpObj->Suffix(os);
  
  pDumpObj->Summary(os,root->fd()->count(),nodes.size(),edgeCount);
}
