#include "stdcfuf.h"

// #define F_TRACE
// #define CMD_TRACE_ON

#include "b_debug_tracer.hxx"
#include "c_fdnode.hxx"
#include "c_barefd.hxx"
#include "c_special.hxx"
#include "c_fdunify.hxx"
#include "t_tracer_l3.hxx"

#ifdef WIN32
#pragma warning(disable : 4355)
#endif

#define NO_COMPACT_LINKS

#ifdef STATS_2
void type_stats(uint all,uint fail);
#endif
#ifdef STATS_3
void link_stats(uint all,uint fail);
#endif
#ifdef STATS_4
void node_stats(uint size,bool is_valid);
#endif
#ifdef STATS_5
void merge_stats(uint *ids,uint size)
{
#ifndef STATS_5_N
  static const uint N = 20;
#else
  static const uint N = STATS_5_N;
#endif
  static bool first = true;
  if (size > N){
    if (first) cout << endl;
    first = false;
    cout << "* " << size << ": ";
    for (uint i=0;i<size;++i) cout << ids[i] << ' ';
    cout << endl;
  }
}
#endif
#ifdef STATS_6
void cnext_stats(int n);
void ufnext_stats(int n);
void stats6_acknoledge();
struct Stats6{
  int& count;
  Stats6(int& _count):count(_count){}
  ~Stats6(){ cnext_stats(count); }
};
#endif

// Used to skip the nodes and arcs which are invalid
// (i.e. cancelled during unification)
static inline bool IsChildOK(const CFDNode::ChildrenList::iterator& it)
{
  return bool(it->second->data().GetType()) && it->first->IsDefined();
}

////////////////////////////////////////////////////////////////////////////////
// CGroupData methods
//
CGroupData::CGroupData(CFDNode *owner,const hCType& init_type):m_owner(owner),
  m_marked(0),m_chook(0)
{
  F_TRACE0();
  
  reserve(g_pTheEnv->m_nInitTypesCap);
  push_back(TypeSlot(init_type));
  if (init_type->any()) m_owner->fd()->any_stack().push(m_owner);
  DBG_TRACE3(Tracer_l3::new_node(init_type,m_owner));
}
void CGroupData::remove()
{
  F_TRACE0();

  assert(m_marked == 0);
  m_chook = 0;                  // clear the copy hook field
  erase(begin(),end());         // erase all the types
}
hCType CGroupData::cleanup()
{
  F_TRACE0();

  assert(m_marked == 0);
  m_chook = 0;                  // clear the copy hook field
  hCType type = GetType();     // get the node's type
  if (type){                    // if it is valid
    erase(begin(),end());       // erase all the types
    push_back(TypeSlot(type,CClock::INPUT_TICK));  // Initialize the types with the current type value
  }
  return type;
}
int CGroupData::id()
{
  return m_owner->id();
}
hCType CGroupData::GetType()
{
  F_TRACE0();
  
#ifdef STATS_6
  stats6_acknoledge();
#endif
#ifdef STATS_2
  uint tseqlen = size();
  uint vsuflen = 0;
#endif
  while (size() && !g_pTheEnv->m_clock.IsCurrent(back().first)){
#ifdef STATS_2
    ++vsuflen;
#endif
    pop_back();
  }
#ifdef STATS_2
  type_stats(tseqlen,vsuflen);
#endif
  return empty() ? hCType() : back().second;
}
void CGroupData::SetType(const hCType& newtype)
{
  F_TRACE1(newtype);
  
  const hCType& prev = GetType();
  if (newtype->any() && (!prev || !prev->any()))
    m_owner->fd()->any_stack().push(m_owner);
  UpdateType(newtype);
}
void CGroupData::UpdateType(const hCType& type)
{
  F_TRACE0();

  if (empty()){
    push_back(TypeSlot(type));
    DBG_TRACE3(Tracer_l3::new_node(type,m_owner));
  }
  assert(g_pTheEnv->m_clock.IsCurrent(back().first));
  DBG_TRACE3(Tracer_l3::update_node(back().second,type,m_owner));
  if (back().first == g_pTheEnv->m_tickCurrent) back().second = type;
  else{
#ifndef NO_COMPACT_TYPES
    const hCType& prev = back().second;
    if (prev->special() || !prev->is_equal(type))
#endif
      push_back(TypeSlot(type));
  }
}

////////////////////////////////////////////////////////////////////////////////
// CLinks methods
//
CLinks::CLinks(CFDNode *owner):m_init_link(owner,owner)
{
  F_TRACE0();
  
  reserve(g_pTheEnv->m_nInitLinksCap);
  push_back(LinkSlot(m_init_link));
}
void CLinks::cleanup()
{
  F_TRACE0();
  
  m_init_link = get_link();
  erase(begin(),end());
  push_back(LinkSlot(m_init_link,CClock::INPUT_TICK));
}
Link& CLinks::get_link()
{
#ifdef STATS_3
  uint lseqlen = size();
  uint lsuflen = 0;
#endif
  while (size() && !g_pTheEnv->m_clock.IsCurrent(back().first)){
#ifdef STATS_3
    ++lsuflen;
#endif
    pop_back();
  }
#ifdef STATS_3
  link_stats(lseqlen,lsuflen);
#endif
  if (empty()) push_back(LinkSlot(m_init_link));
  return back().second;
}
void CLinks::set_cnext(CFDNode *cnext)
{
  Link& link = get_link();
  if (back().first == g_pTheEnv->m_tickCurrent) link.first = cnext;
  else{
#ifndef NO_COMPACT_LINKS
    if (link.first != cnext)
#endif
      push_back(LinkSlot(Link(cnext,link.second)));
  }
}
void CLinks::set_ufnext(CFDNode *ufnext)
{
  Link& link = get_link();
  if (back().first == g_pTheEnv->m_tickCurrent) link.second = ufnext;
  else{
#ifndef NO_COMPACT_LINKS
    if (link.second != ufnext)
#endif
      push_back(LinkSlot(Link(link.first,ufnext)));
  }
}

////////////////////////////////////////////////////////////////////////////////
// CFDNode methods
//
CFDNode::CFDNode(CBareFD *fd,const hCType& m_type,CArc_c *parent_arc,CFDNode *parent_node):
  m_fd(fd),m_link(this),m_id(m_fd->m_count),m_tstamp(m_id),m_parent(parent_arc,parent_node),m_data(this,m_type)
{
  ++m_fd->m_count;
}
CFDNode::~CFDNode()
{
  F_TRACE0();

  --m_fd->m_count;
  while (m_children.size()){
    CFDNode *node;
    {                   // enclose as much as possible of the locals in a separate block
      CArc_c *arc;      // this is because "delete node" is recursive and I do not want to
      ChildrenList::iterator begin = m_children.begin();  // eat up the stack space just like that
      arc = begin->first;          // get the arc leading to the child
      node = begin->second;        // get the child node
      m_children.erase(begin);          // erase the entry for the child
      delete arc;                       // delete the arc
    }
    delete node;        // delete the node (this a recursive call to destructor)
  }
}
////////////////////////////////////////////////////////////////////////////////
// Cleans the node (as the name suggests), removing unification history
// from the node's links and data.In every group only the representative is allowed
// to have data, all the "vassal" nodes are stripped clean.
// No nodes or arcs are ever deleted.This is because patterns and cset can have references
// to nodes scattered all around the graph.Physical deletion of nodes makes consistency
// maintaining too much of a trouble.
// Both functions are made private, so that only friends can apply them (on the root
// of the graph only).
void CFDNode::cleanup()
{
  F_TRACE0();

  m_link.cleanup();             // cleanup the links first
  m_input = true;               // the node becomes input (declared in CFDNodeBase)
  if (m_link.ufnext() == this){ // if this is a group representative
    hCType type = m_data.cleanup();     // cleanup the data
    if (CSpecial *special = cast<CSpecial>(type))   // if this is a special type
      special->cleanup();       // cleanup the special type
#ifdef STATS_4
    {
      uint csize = 1;
      for (CFDNode *node = m_link.cnext();node != this;++csize,node = node->m_link.cnext());
      node_stats(csize,bool(type));
    }
#endif
#ifdef STATS_5
    {
      static uint ids[1000];
      ids[0] = m_id;
      uint csize = 1;
      for (CFDNode *node = m_link.cnext();node != this;++csize,node = node->m_link.cnext())
        ids[csize] = node->m_id;
      merge_stats(ids,csize);
    }
#endif
  }else                         // else - this is a "vassal" node
    m_data.remove();            // remove its data (it is taken from the representative anyway)
    
  ChildrenList::iterator ch_it = m_children.begin();    // get the children iterator
  for (uint i=m_children.size();i;--i,++ch_it){         // iterate on the children
    ch_it->second->cleanup();                           // cleanup the child node recursively
    ch_it->first->cleanup();                            // cleanup the child arc
  }
}
////////////////////////////////////////////////////////////////////////////////
// Theoretically Union-Find algorithm with path compression is more effective
// than without it.But in practice it is too much of an overhead to implement
// path compression.Roughly 1% of calls to representative() has to go over one
// link to reach the group representative.Of those calls 99% require exactly
// two links to be followed.So there is no penalty to omit the path compression.
CFDNode *CFDNode::representative()
{
  F_TRACE0();
  
  CFDNode *node = this;
  CFDNode *_ufnext = m_link.ufnext();
#ifdef STATS_6
  int count = 1;
#endif
  while (_ufnext != node){
    node = _ufnext;
    _ufnext = node->m_link.ufnext();
#ifdef STATS_6
    ++count;
#endif
  }
#ifdef STATS_6
  ufnext_stats(count);
#endif
  return _ufnext;
}
bool CFDNode::HasAnyChildren()
{
  if (!data().GetType()->internal()) return false;
#ifdef STATS_6
  int count = 0;
  Stats6 stats(count);
#endif
  for (CFDGroupIterator itGroup(this);!itGroup.is_past_the_end();itGroup.advance()){
#ifdef STATS_6
    ++count;
#endif
    ChildrenList::iterator itChild=itGroup->m_children.begin();
    for (uint i = itGroup->m_children.size();i;--i,++itChild)
      if (IsChildOK(itChild)) return true;
  }
  data().SetType(g_pTheEnv->nil());
  return false;
}
CFDNode *CFDNode::FindChild(CArc_c *arc,bool insist)
{
  F_TRACE1(arc);

  if (!insist && !data().GetType()->internal()) return NULL;
#ifdef STATS_6
  int count = 0;
  Stats6 stats(count);
#endif
  for (CFDGroupIterator it(this);!it.is_past_the_end();it.advance()){
#ifdef STATS_6
    ++count;
#endif
    ChildrenList::iterator found=it->m_children.find(arc);
    if (found!=it->GetChildEndIt() && IsChildOK(found))
      return found->second;
  }
  return NULL;
}
// Follows the edge list until the destination node is found or the list is
// exhausted.If the destination node exists it is returned, otherwise NULL is.
// If insist is false (the default) stops at the first final node
CFDNode *CFDNode::FindDescendant(const EdgeList::const_iterator& begin,
                                  const EdgeList::const_iterator& end,
                                  bool insist)
{
  F_TRACE0();
  
  EdgeList::const_iterator e_it = begin;
  CFDNode *cur = this;
  for (uint i = end - begin;i;--i,++e_it){
    CFDNode *next = cur->FindChild(*e_it,insist);
    if (!next) return cur->data().GetType()->none() ? cur : 0;
    cur = next;
  }
  return cur;
}
CFDNode *CFDNode::AddChild(CArc_c *arc,const hCType& type)
{
  F_TRACE2(arc,type);
  
  ChildrenList::iterator found=m_children.find(arc);
  if (found==GetChildEndIt()){                         // the arc is totally new
    CArc_c *charc=new CArc_c(arc->title());
    return m_children[charc] = new CFDNode(m_fd,type,charc,this);
  }else{                                        // the arc already exists (though in different clock)
    found->second->data().SetType(type);  // introduce the child into the current clock
    found->first->update();
    return found->second;
  }
}
void CFDNode::cancel_node(const hCType& type)
{
  F_TRACE0();
  
  data().RemoveType(type);    // type should be either none or nil
  
  // for (CFDChildIterator it(this);!it.is_past_the_end();it.advance())
    // it->first->undefine();
}
CFDNode *CFDNode::connect(CFDNode *node)
{
  DBG_TRACE3(Tracer_l3::merge_nodes(this,node));
  CFDNode *cnext1 = m_link.cnext();
  CFDNode *cnext2 = node->m_link.cnext();
  CFDNode *repr1 = representative();
  CFDNode *repr2 = node->representative();
  m_link.set_cnext(cnext2);
  node->m_link.set_cnext(cnext1);
  uint marked1 = repr1->m_data.GetMark();
  uint marked2 = repr2->m_data.GetMark();
  if (marked1 == marked2 && repr1->m_tstamp < repr2->m_tstamp || marked1 > marked2){
    // cout << repr2->m_tstamp() << " (" << repr2->data().GetType() << ") " << "--> "
         // << repr1->m_tstamp() << " (" << repr1->data().GetType() << ")\n";
    repr2->m_link.set_ufnext(repr1);
    repr1->set_input(repr2->input());
    return repr1;
  }else{
    // cout << repr1->m_tstamp() << " (" << repr1->data().GetType() << ") " << "--> "
         // << repr2->m_tstamp() << " (" << repr2->data().GetType() << ")\n";
    repr1->m_link.set_ufnext(repr2);
    repr2->set_input(repr1->input());
    return repr2;
  }
}
bool CFDNode::any()
{
  hCType type = data().GetType();
  if (!type || !type->any()) return false;      // if the node is not even typed as ANY - done
  if (!type->internal()) return true;           // if ANY but not internal - done
  ret::utype_t u=StaticUnify::unify_i_g(this);       // unify the subgraph with GIVEN
  if (!u.fail){                                 // if ok -
    data().SetType(u.val);                     // the node is not really ANY - update its type
    return false;                               // and the node is not ANY
  }
  delete u.fail;                                // the node is really ANY
  return true;
}

////////////////////////////////////////////////////////////////////////////////
// CFDParentIterator methods
//
void CFDParentIterator::advance()
{
  if (m_cur && ((m_cur=m_cur->m_link.cnext())==m_start ||     // if either reached the start (looped through the whole group)
                m_cur->m_parent.first == NULL))               // or the parent is NULL
    m_cur=NULL;                                               // transit the iterator into past-the-end state
}

////////////////////////////////////////////////////////////////////////////////
// CFDChildIterator methods
//
void CFDChildIterator::reset(CFDNode *node)
{
  m_start = m_cur = node;
  m_it = m_cur->GetChildBeginIt();
  while (!m_special_queue.empty()) m_special_queue.pop();
  next();
}
void CFDChildIterator::advance()
{
	F_TRACE0();

	if (m_cur)
	{
		++m_it;
		next();
	}
	else if (m_special_queue.size())
	{
		m_special_queue.pop();
		if (m_special_queue.size())
		{
#ifdef WIN32
			memcpy(&m_child,&m_special_queue.front(),sizeof(m_child));
#else
			m_child=m_special_queue.front();
#endif
		}
	}
}
void CFDChildIterator::next()
{
  F_TRACE0();

  for (;;){
    while (m_it==m_cur->GetChildEndIt()){
      m_cur=m_cur->m_link.cnext();
      if (m_cur==m_start){
        m_cur=NULL;
        if (m_special_queue.size())
		{
#ifdef WIN32
			memcpy(&m_child,&m_special_queue.front(),sizeof(m_child));
#else
			m_child=m_special_queue.front();
#endif
		}
        return;
      }
      m_it = m_cur->GetChildBeginIt();
    }
    if (IsChildOK(m_it)){
      if (m_it->second->data().GetType()->special()){
        m_special_queue.push(*m_it);
      }else{
#ifdef WIN32
			memcpy(&m_child,&*m_it,sizeof(m_child));
#else
	        m_child=*m_it;
#endif
        return;
      }
    }
    ++m_it;
  }
}
