#ifndef _COMPILER_FDNODE_
#define _COMPILER_FDNODE_

#include "b_objects.hxx"
#include "c_fdbase.hxx"
#include "b_nodebase.hxx"
#include <hash_map.h>
#include <queue.h>

class CFDNode;

struct TypeSlot:pair<tick_t,hCType>{
  explicit TypeSlot(const hCType& type):pair<tick_t,hCType>(g_pTheEnv->m_tickCurrent,type){}
  TypeSlot(const hCType& type,tick_t uid):pair<tick_t,hCType>(uid,type){}
};

class CGroupData:private vector<TypeSlot>{
  CFDNode *m_owner;     // the data owner node
  uint m_marked;        // non zero if the node is a constituent
  CFDNode *m_chook;     // used to speed copy_subfd
public:
  CGroupData(CFDNode *owner,const hCType& init_type);
  hCType cleanup();
  void remove();
  void SetType(const hCType&);
  void RemoveType(const hCType& type){ UpdateType(type); }
  hCType GetType();
  void Mark()                   { ++m_marked; }
  void Unmark()                 { --m_marked; }
  uint GetMark() const          { return m_marked; }
  void SetHook(CFDNode *copy)   { m_chook = copy; }
  CFDNode *GetHook() const      { return m_chook; }
  int id();             // defined below, under the CFDNode class definition
private:
  void UpdateType(const hCType& type);
};

struct Link:public pair<CFDNode *,CFDNode *>{
  Link(CFDNode *cnext,CFDNode *ufnext):pair<CFDNode *,CFDNode *>(cnext,ufnext){}
};
struct LinkSlot:pair<tick_t,Link>{
  explicit LinkSlot(const Link& link):pair<tick_t,Link>(g_pTheEnv->m_tickCurrent,link){}
  LinkSlot(const Link& link,tick_t uid):pair<tick_t,Link>(uid,link){}
};

class CLinks:private vector<LinkSlot>{
  Link m_init_link;
public:
  explicit CLinks(CFDNode *owner);
  void cleanup();
  CFDNode *cnext()                      { return get_link().first; }
  CFDNode *ufnext()                     { return get_link().second; }
  void set_cnext(CFDNode *cnext);
  void set_ufnext(CFDNode *ufnext);
private:
  Link& get_link();
};

struct hasharc{
  size_t operator()(CArc_c *arc) const {
    return hashtoken()(arc->title());
  }
};
struct eqarcs{
  bool operator()(CArc_c *arc1,CArc_c *arc2) const {
    return eqtokens()(arc1->title(),arc2->title());  // both tokens must point to the same string
  }
};

class CBareFD;
class CFDNode:public CFDNodeBase{
public:
  typedef hash_map<CArc_c *,CFDNode *,hasharc,eqarcs> ChildrenList;
  typedef pair<CArc_c *,CFDNode *> child_type;
  typedef child_type parent_type;
  friend class CFDChildIterator;
  friend class CFDGroupIterator;
  friend class CFDParentIterator;
  friend class CGroupData;
  friend class CFD;     // must have access to the cleanup() method
private:
  CBareFD *m_fd;        // must be defined before CGroupData object
  CLinks m_link;
  ChildrenList m_children;
  int m_id;             // individual node id, sequential, identifies graph nodes
  int m_tstamp;         // individual node time stamp, can differ from id (for example when copying an fd)
  parent_type m_parent;
  CGroupData m_data;
public:
  CFDNode(CBareFD *fd,const hCType& m_type,CArc_c *parent_arc=0,CFDNode *parent_node=0);
  ~CFDNode();
  CBareFD *fd() const { return m_fd; }
  bool HasAnyChildren();
  CFDNode *FindChild(token_t edge,bool insist=false)  { return FindChild(dummy_arc(edge),insist); }
  CFDNode *FindChild(CArc_c *arc,bool insist=false);
  CFDNode *FindDescendant(const EdgeList::const_iterator& begin,
                           const EdgeList::const_iterator& end,bool insist=false);
  CFDNode *AddChild(token_t edge,const hCType& type)  { return AddChild(dummy_arc(edge),type); }
  CFDNode *AddChild(CArc_c *arc,const hCType&);
  void cancel_node(const hCType& type);
  const parent_type& parent() const { return m_parent; }
  CGroupData& data()            { return representative()->m_data; }
  CLinks& link()                { return m_link; }
  int id() const                { return m_id; }
  int group_id()                { return representative()->id(); }
  int tstamp() const            { return m_tstamp; }
  void set_tstamp(int tstamp)   { m_tstamp = tstamp; }
  int group_tstamp()            { return representative()->tstamp(); }
  bool is_equiv(CFDNode *node)  { return representative() == node->representative(); }
  CFDNode *connect(CFDNode *node); // connects <this> and <node> and returns the representative of the new group
  CFDNode *representative();
  bool any();           // true if the node is really ANY
private:
  void cleanup();       // should be called only on the root of the fd
  ChildrenList::iterator GetChildBeginIt(){
    return data().GetType()->internal() ? m_children.begin() : m_children.end();
  }
  ChildrenList::iterator GetChildEndIt(){ return m_children.end(); }
  CArc_c *dummy_arc(token_t edge){
    static CArc_c dummy_arc;
    dummy_arc.set_title(edge);
    return &dummy_arc;
  }
  void set_input(const CInput& r) { m_input = r; }
};

class CFDGroupIterator{
public:
  typedef CFDNode value_type;
  typedef value_type& reference;
  typedef value_type *pointer;
private:
  CFDNode *m_start;
  CFDNode *m_cur;
public:
  explicit CFDGroupIterator(CFDNode *start):m_start(start),m_cur(m_start){}
  void advance()                { if (m_cur && (m_cur=m_cur->m_link.cnext())==m_start) m_cur=NULL; }
  bool is_past_the_end() const  { return m_cur==NULL; }
  reference operator*() const   { return *m_cur; }
  pointer operator->() const    { return m_cur; }
  operator pointer() const      { return m_cur; }
};

class CFDParentIterator{
public:
  typedef CFDNode::parent_type value_type;
  typedef value_type& reference;
  typedef value_type *pointer;
private:
  CFDNode *m_start;
  CFDNode *m_cur;
public:
  explicit CFDParentIterator(CFDNode *start = 0):m_start(start),m_cur(start){}
  void reset(CFDNode *start)    { m_start = m_cur = start; }
  void advance();
  bool is_past_the_end() const  { return m_cur==NULL || m_cur->m_parent.first == NULL; }
  reference operator*() const   { return m_cur->m_parent; }
  pointer operator->() const    { return &(m_cur->m_parent); }
  operator pointer() const      { return &(m_cur->m_parent); }
};

class CFDChildIterator{
public:
  typedef CFDNode::child_type value_type;
  typedef value_type& reference;
  typedef value_type *pointer;
  typedef CFDNode::ChildrenList::iterator ch_iterator;
private:
  CFDNode *m_start;
  CFDNode *m_cur;
  ch_iterator m_it;
  queue<value_type> m_special_queue;
  value_type m_child;
public:
  CFDChildIterator():m_start(0),m_cur(0){}
  explicit CFDChildIterator(CFDNode *node){ reset(node); }
  void reset(CFDNode *node);
  void advance();
  bool is_special() const       { return !m_cur; }
  bool is_past_the_end() const  { return !m_cur && m_special_queue.empty(); }
  reference operator*() const   { return const_cast<reference>(m_child); }
  pointer operator->() const    { return const_cast<pointer>(&m_child); }
  operator pointer() const      { return const_cast<pointer>(&m_child); }
private:
  void next();
};

template <class T>
inline T *cast(CFDNode *obj)            { return dynamic_cast<T *>(obj); }

#endif
