#include "stdcfuf.h"
#include "c_extfd.hxx"
#include "c_fdnode2path.hxx"
#include <queue.h>

////////////////////////////////////////////////////////////////////////////////
// Helper classes for get_rel_path().
//
// CReverseNode and CReverseGraph implement creation of the reversed graph with
// dst being the root.
class CReverseNode:public RCObject{
public:
  CReverseNode(CFDNode *proto_node,CArc_c *proto_arc,CReverseNode *rnode =0);
  CPath *get_path(int upcount,bool fix);
  CFDNode *proto_node() const           { return m_proto_node; }
  void add_rparent(CReverseNode *rnode) { m_rparents.push_back(rnode); }
private:
  CFDNode *m_proto_node;
  CArc_c *m_proto_arc;
  vector<CReverseNode *> m_rparents;
};

CReverseNode::CReverseNode(CFDNode *proto_node,CArc_c *proto_arc,CReverseNode *rnode):
  m_proto_node(proto_node),m_proto_arc(proto_arc)
{
  m_rparents.push_back(rnode);
}
CPath *CReverseNode::get_path(int upcount,bool fix)
{
  static CPath path;
  path.erase(path.begin(),path.end());
  
  path.set_upcount(upcount);
  CReverseNode *cur = this;
  while (cur->m_proto_arc){
    path.push_back(cur->m_proto_arc->title());
    cur = cur->m_rparents.front();
  }
  if (fix && upcount == 1 && path.size() == 1) path.set_upcount(CPath::CUR);
  return &path;
}

typedef Handle<CReverseNode> hCReverseNode;

class CReverseGraph{
public:
  CReverseGraph(CFDNode *root);
  CReverseNode *find(int group_id);
private:
  hash_map<int,hCReverseNode> m_nodehash;
private:
  void build(CReverseNode *rnode);
};

CReverseGraph::CReverseGraph(CFDNode *root)
{
  CReverseNode *rroot = new CReverseNode(root,0);
  m_nodehash[root->group_id()] = rroot;
  build(rroot);
}
void CReverseGraph::build(CReverseNode *rnode)
{
  for (CFDParentIterator it(rnode->proto_node());!it.is_past_the_end();it.advance()){
    int parent_id = it->second->group_id();
    CReverseNode *rchild;
    hash_map<int,hCReverseNode>::iterator rfound = m_nodehash.find(parent_id);
    if (rfound != m_nodehash.end()){
      rchild = rfound->second;
      rchild->add_rparent(rnode);
    }else{
      rchild = new CReverseNode(it->second,it->first,rnode);
      m_nodehash[parent_id] = rchild;
      build(rchild);
    }
  }
}
CReverseNode *CReverseGraph::find(int group_id)
{
  hash_map<int,hCReverseNode>::iterator rfound = m_nodehash.find(group_id);
  return (rfound == m_nodehash.end()) ? NULL : rfound->second.pointee();
}

// CAncestorIterator implements BFS iterator of all the src ancestors.
// The src node itself is included.
class CAncestorIterator{
public:
  typedef CFDNode value_type;
  typedef value_type& reference;
  typedef value_type *pointer;
private:
  CFDNode *m_root;
  CVisitedBitmap m_visited;
  queue<pair<CFDNode *,int> > m_queue;
public:
  CAncestorIterator(CFDNode *root);
  void advance()                { if (!is_past_the_end()) next(); }
  bool is_past_the_end() const  { return m_queue.empty(); }
  reference operator*() const   { return *(m_queue.front().first); }
  pointer operator->() const    { return m_queue.front().first; }
  operator pointer() const      { return m_queue.front().first; }
  int level() const             { return m_queue.front().second; }
private:
  void next();
};

CAncestorIterator::CAncestorIterator(CFDNode *root):
  m_root(root),m_visited(1024)
{
  m_queue.push(make_pair(m_root,0));
}
void CAncestorIterator::next()
{
  CFDNode *cur = m_queue.front().first;
  int level = m_queue.front().second;
  m_queue.pop();
  m_visited[cur->group_id()]=true;
  ++level;
  for (CFDParentIterator it(cur);!it.is_past_the_end();it.advance()){
    int id = it->second->id();
    if (!m_visited[id]){
      m_queue.push(make_pair(it->second,level));
      m_visited[id]=true;
    }
  }
}

////////////////////////////////////////////////////////////////////////////////
// Node2Path methods
//
CPath *Node2Path::get_abs_path(CFDNode *node)
{
  static CPath path(CPath::ABS);
  path.erase(path.begin(),path.end());
  
  CFDNode *cur=node;
  while (cur->group_id() > 0){
    CFDNode::parent_type parent(cur->parent());
    path.push_back(parent.first->title());
    cur = parent.second;
  }
  reverse(path.begin(),path.end());
  return &path;
}
CPath *Node2Path::get_rel_path(CFDNode *src,CFDNode *dst,bool fix)
{
  assert(src);
  // First build the reverse graph with dst as its root.
  CReverseGraph rgraph(dst);
  // Next step depends on the circumstances.
  // If the current location reference (g_pTheEnv->m_curLocRef) is not empty, then
  // I consult it to get the turn node for the relative path, else I find it
  // myself, any match is ok.
  if (g_pTheEnv->m_curLocRef.GetSize()){
    // Go through the current location reference until found a node that is present
    // in the reverse graph.
    CCurLocRef::LocIter cur = g_pTheEnv->m_curLocRef.Begin();
    uint level = 0;
    for (uint i=g_pTheEnv->m_curLocRef.GetSize();i;--i,++cur,++level)
      if (CReverseNode *rnode = rgraph.find(*cur)) return rnode->get_path(level,fix);
  }else{
    // Iterate over the ancestors of the source until found one that is present
    // in the reverse graph.
    for (CAncestorIterator it(src);!it.is_past_the_end();it.advance())
      if (CReverseNode *rnode = rgraph.find(it->group_id())) return rnode->get_path(it.level(),fix);
  }
  assert(false);
  return 0;
}
