#include "stdcfuf.h"

// #define F_TRACE

#include "b_debug_tracer.hxx"
#include "c_fdnode.hxx"
#include "c_fditerator.hxx"
#include "c_fdunify.hxx"
#include "b_bkclass.hxx"

struct GroundFail:public Fail{
  string GetDesc() const       { return "Cannot cancel a ground fd"; }
};
struct NonGroundFail:public Fail{
  string GetDesc() const       { return "Cannot unify <given> with a non ground fd"; }
};

typedef bool (*Predicate)(CFDNode *);

static vector<CFDNode *> m_cache;
static Fail *unify_graphs(CFDNode *,CFDNode *);
static bool i_f_node_ok(CFDNode *node);
static bool i_g_bad_node(CFDNode *node);
static bool is_fd_ok(CFDNode *,Predicate);

ret::unode_t StaticUnify::unify_nodes(CFDNode *node1,CFDNode *node2)
{
  F_TRACE2(node1,node2);

  node1 = node1->representative();
  node2 = node2->representative();
  if (node1 == node2) return node1;

  hCType type1 = node1->data().GetType();
  hCType type2 = node2->data().GetType();
  ret::utype_t u = type1->unify(type2,node1,node2);  // unify the types of the two nodes
  if (u.fail && u.fail->IsInternalNormal()){    // retry if internal-normal failure
    delete u.fail;                              // delete the failure object
    u = (type1->internal() ?                    // unify internal node with the final one
         StaticUnify::unify_i_f(node1,type2) :
         StaticUnify::unify_i_f(node2,type1));
  }
  if (!u.fail && type1->internal() && type2->internal()) // if OK and both nodes are internal
    u.fail = unify_graphs(node1,node2);       // unify the respective subgraphs
  if (u.fail){
    if (BKClass::fail_bk() == 0){
      for (CFDParentIterator p_it1(node1);!p_it1.is_past_the_end();p_it1.advance()) // iterate on the parents of the current node
        BKClass::add_fail_bk(p_it1->first->title()); // acknowledge the failed parent edges
      for (CFDParentIterator p_it2(node2);!p_it2.is_past_the_end();p_it2.advance()) // iterate on the parents of the current node
        BKClass::add_fail_bk(p_it2->first->title()); // acknowledge the failed parent edges
    }
    return u.fail;
  }
  
  if (node1->is_equiv(node2)) return node1;     // might exit here if unification path included loops
  
  CFDNode *repr = node1->connect(node2);
  if (u.val) repr->data().SetType(u.val);      // can happen if a type does not support unification
  return repr;
}
Fail *unify_graphs(CFDNode *node1,CFDNode *node2)
{
  F_TRACE2(node1,node2);
  
  vector<pair<CFDNode *,CFDNode *> > matches;
  matches.reserve(4);

  // Unification is split in two parts:
  // 1. Collect the matching nodes
  // 2. Unify the matching nodes pair after pair.
  // The reason is that unification might merge two nodes one of which is the one which children are being
  // iterated over.This is an unwanted effect.So, do the iteration first and only then unify.
  for (CFDChildIterator it(node1);!it.is_past_the_end();it.advance()){  // iterate on the children of node1
    CFDNode *child_node2 = node2->FindChild(it->first);        // check if node2 has a child on the same arc
    // cout << it->first << ": (" << it->second << ',' << child_node2 << ")\n";
    if (child_node2){                           // if it does - cancel one of the arcs (this also prevents
      it->first->undefine();                    // from repeating the unification route again)
      matches.push_back(make_pair(it->second,child_node2));     // save the matching nodes for later unification
    }
  }
  if (g_pTheEnv->m_bInterrupted) throw new InterruptFail;
  
  vector<pair<CFDNode *,CFDNode *> >::const_iterator cur = matches.begin();
  for (uint i=matches.size();i;--i,++cur){
    // cout << i << ": (" << cur->first << ',' << cur->second << ")\n";
    ret::unode_t u =
      StaticUnify::unify_nodes(cur->first,cur->second); // and unify the matching nodes recursively
    if (u.fail) return u.fail;
  }
  return 0;
}
ret::utype_t StaticUnify::unify_i_f(CFDNode *in_node,const hCType& leaf)
{
  F_TRACE2(in_node,leaf);

  assert(leaf->final());
  if (!is_fd_ok(in_node,i_f_node_ok)) return new GroundFail;
  
  // The subgraph is not ground - may proceed with the cancellation
  for (uint i = m_cache.size();i;--i){
    m_cache.back()->cancel_node(g_pTheEnv->none());
    m_cache.pop_back();
  }
  // Check if the <in_node>'s type can be set to <leaf>
  // This may fail, if <leaf> is not <none> and there is a parent node of <in_node> which
  // has just been cancelled (thus its type is <none>)
  if (!leaf->none()){
    for (CFDParentIterator p_it(in_node);!p_it.is_past_the_end();p_it.advance()){ // iterate on the parents of <in_node>
      const hCType& parent_type = p_it->second->data().GetType();
      if (parent_type->none()) return new GroundFail;
      assert(parent_type->internal());
    }
  }
  return leaf;
}
bool i_f_node_ok(CFDNode *node)       // returns true if a node obeys following criteria
{
  F_TRACE1(node);

  CGroupData& g_data = node->data();
  hCType type = g_data.GetType();
  return (!(type->any() ||              // not ANY node (since it will have to be instantiated eventually)
            type->final()) ||           // and not a final node
          type->none());                // or it is a none node
}
ret::utype_t StaticUnify::unify_i_g(CFDNode *in_node)
{
  F_TRACE1(in_node);
  
  if (is_fd_ok(in_node,i_g_bad_node))   // if every node below in_node is bad
    return new NonGroundFail;           // failed - cannot unify with given
  return in_node->data().GetType();
}
bool i_g_bad_node(CFDNode *node)        // the node is bad for the GIVEN criteria
{                                       // if it is either not final or none
  F_TRACE1(node);

  hCType type = node->data().GetType();
  return !type->final() || type->none();
}
bool is_fd_ok(CFDNode *root,Predicate pred)
{
  F_TRACE1(root);

  m_cache.erase(m_cache.begin(),m_cache.end());
  for (CFDGraphIterator it(root);!it.is_past_the_end();it.advance()){
    m_cache.push_back(it);
    if (!pred(it)) return false;
  }
  return true;
}
