#include "stdcfuf.h"
#include "c_barefd.hxx"
#include "c_fdwalk.hxx"

bool ch_less(const CFDWalk::child_type& x,const CFDWalk::child_type& y)
{
  return strcmp(TOK2STR(x.first),TOK2STR(y.first)) < 0;
}
bool ch_greater(const CFDWalk::child_type& x,const CFDWalk::child_type& y)
{
  return strcmp(TOK2STR(x.first),TOK2STR(y.first)) > 0;
}

const CFDWalk::order_t CFDWalk::NONE=0;
const CFDWalk::order_t CFDWalk::ASCENDING=ch_less;
const CFDWalk::order_t CFDWalk::DESCENDING=ch_greater;
const CFDWalk::iter_halter_t CFDWalk::DO_ALL    = &CFDChildIterator::is_past_the_end;
const CFDWalk::iter_halter_t CFDWalk::DO_NORMAL = &CFDChildIterator::is_special;

/////////////////////////////////////////////////////////////////////////////////
// CFDWalk methods
//
// All the paths have the given root node as its origin, even if this node is not
// the real root of its fd.This means that a path {a b c} originates at the given node
// and NOT at the fd root.The user who supplied the root node can easily compute the
// real absolute path.
CFDWalk::CFDWalk(CFDNode *root):m_visited(root->fd()->count())
{
  assert(!g_pTheEnv->m_curLocRef.GetSize());
  m_pUnparser = 0;
  int rootid = root->group_id();
  g_pTheEnv->m_curLocRef.Push(rootid);  // Initialize the current location reference with the root index
  m_visited[rootid]=new EdgeList;       // set the path for the root node
  m_pNode = root;
}
void CFDWalk::ignore(token_t feature)
{
  m_ignored.push_back(feature);
}

struct PrepareWalk{
  ~PrepareWalk(){
    g_pTheEnv->m_curLocRef.Reset();     // reset the current location reference
  }
};

Fail *CFDWalk::walk(CUnparseFD *pUnparser)
{
  assert(!m_pUnparser);
  m_pUnparser = pUnparser;
  m_halter = m_pUnparser->IsDoingAll() ? DO_ALL : DO_NORMAL;
  if (int nOrder = m_pUnparser->GetOrder())
    m_order = (nOrder == CUnparseFD::DESCENDING) ? DESCENDING : ASCENDING;
  else m_order = NONE;
    
  hCType type = m_pNode->data().GetType();
  Fail *f = 0;
  PrepareWalk prepare;                       // ensure the consistent state, even if an exception is thrown
  if (type->internal()){                     // this might take some time
    f = m_pUnparser->StartInternal(m_pNode);
    if (!f) f = WalkFD(m_pNode);             // can throw an exception
    if (!f) f = m_pUnparser->EndInternal();
  }else f = type->special() ? m_pUnparser->StartEndSpecial(m_pNode) : m_pUnparser->StartEndLeaf(m_pNode);
  return f;
}
Fail *CFDWalk::WalkFD(CFDNode *root)
{
  Fail *f = 0;
  if (m_order){                                                                 // if the children must be sorted before traversed
    deque<child_type> children;                                                 // holds the children of the root
    for (CFDChildIterator ch_it(root);!(ch_it.*m_halter)();ch_it.advance())     // iterate on the root's children
      children.push_back(child_type(ch_it->first->title(),ch_it->second));      // and save them to the queue
    sort(children.begin(),children.end(),m_order);                              // sort the children
    
    while (children.size() && !f){                              // iterate on the children
      f = WalkAttrValue(children.front().first,               // child node's feature
                          children.front().second);             // the child node itself
      children.pop_front();
    }
  }else{
    // the children are walked in an arbitrary manner - quicker and uses less memory
    // special nodes are guaranteed to come after normal ones
    for (CFDChildIterator ch_it(root);!(ch_it.*m_halter)() && !f;ch_it.advance()) // iterate on the root's children
      f = WalkAttrValue(ch_it->first->title(),ch_it->second);
  }
  return f;
}
Fail *CFDWalk::WalkAttrValue(token_t edge,CFDNode *node)
{
  if (g_pTheEnv->m_bInterrupted) throw new InterruptFail;
  
  if (find(m_ignored.begin(),m_ignored.end(),edge) != m_ignored.end()) return 0;

  Fail *f = 0;
  CGroupData& gr_data = node->data();           // get the group data
  g_pTheEnv->m_curLocRef.Push(gr_data.id());        // update the current location reference
  hCType type = gr_data.GetType();             // get the node type
  EdgeList *path = m_visited[gr_data.id()];     // the first path this node was seen at
  if (path)                                     // if the node has already been visited
    f = (g_pTheEnv->m_bSplitPaths && type->final() ?  // split paths if permittable and allowed
         m_pUnparser->AddLeaf(edge,node,path) :    // i.e. add a leaf
         m_pUnparser->AddPath(edge,node,path));    // else add a path
  else{                                         // if the node is seen the first time
    m_curloc.push_back(edge);                   // follow the feature - update the current location
    path = m_visited[gr_data.id()] = new EdgeList(m_curloc); // save the location for node future references
    if (type->internal()){                      // if the node is internal
      f = m_pUnparser->EnterSubgraph(edge,node,path);  // enter its subgraph
      if (!f) f = WalkFD(node);                // then recursively walk the subfd
      if (!f) f = m_pUnparser->LeaveSubgraph();    // leave the subgraph
    }else f = m_pUnparser->AddLeaf(edge,node,path);// all that's left is to add a leaf
    m_curloc.pop_back();                        // restore the current location
  }
  g_pTheEnv->m_curLocRef.Pop();                     // restore the previous location reference
  return f;
}
