#include "stdcfuf.h"

// #define F_TRACE

#include "b_debug_tracer.hxx"
#include "c_extfd.hxx"
#include "c_pattern.hxx"
#include "b_bkclass.hxx"
#include "c_fdnode2path.hxx"
#include "c_cell.hxx"
#include "t_tracer_l3.hxx"
#include <assert.h>

static Fail *fail;
static CFDNode *s_from;

inline int is_bit_set(int n,int pos){ return n & (1<<pos); }

class FDAddPatternFail:public FDPatternFail{
public:
  FDAddPatternFail(const hCPatternType& _ptype,const hCPattern& _pattern):
    ptype(_ptype),pattern(_pattern){}
  string GetDesc() const;
private:
  hCPatternType ptype;
  hCPattern pattern;
};

class FDUnifyPatternsFail:public FDPatternFail{
public:
  FDUnifyPatternsFail(const hCPatternType& _ptype1,const hCPatternType& _ptype2):
    ptype1(_ptype1),ptype2(_ptype2){}
  string GetDesc() const;
private:
  hCPatternType ptype1,ptype2;
};

////////////////////////////////////////////////////////////////////////////////
// CNodeCell and CMergeNodeCell str() methods
//
string CNodeCell::str(CFDNode *from) const
{
  return Node2Path::get_rel_path(from,m_node,true)->toString();
}
string CMergeNodeCell::str(CFDNode *from) const
{
  string str("(* ");
  return str << CNodeCell::str(from) << ')';
}

////////////////////////////////////////////////////////////////////////////////
// CellList methods
//
CellList::CellList(int capacity)
{
  reserve(capacity);
}
string CellList::toString() const
{
  if (empty()) return "()";
  string str("(");
  const_iterator cur=begin();
  for (uint i=size()-1;i;--i,++cur) str << (*cur)->str(s_from) << ' ';
  return str << (*cur)->str(s_from) << ')';
}

////////////////////////////////////////////////////////////////////////////////
// PatternList methods
//
PatternList::PatternList(const hCellList& first)
{
  push_back(first);
}

////////////////////////////////////////////////////////////////////////////////
// Constraint methods
//
Constraint::Constraint(int capacity)
{
  reserve(capacity);
}
string Constraint::str() const
{
  string _str("(");
  const_iterator cur=begin();
  for (uint i=0;i<size()-1;++i,++cur)
    _str << Node2Path::get_abs_path(*cur) << ' ';
  return _str << Node2Path::get_abs_path(*cur) << ')';
}

////////////////////////////////////////////////////////////////////////////////
// PatternValues methods
//
PatternValues::PatternValues(int capacity):
  m_nconstr(0)
{
  reserve(capacity);
}
PatternValues::~PatternValues()
{
  vector<Constraint *>::iterator it=m_constraints.begin();
  for (uint i=0;i<m_constraints.size();++i,++it) delete *it;   // delete constraints
}
void PatternValues::push_dots(const hCCell& dots)
{
  BaseClass::push_back(dots);
  m_constraints.push_back(0);
}
void PatternValues::push_back(const hCCell& cell,Constraint *c)
{
  BaseClass::push_back(cell);
  if (c) ++m_nconstr;
  m_constraints.push_back(c);
}
void PatternValues::pop_back()
{
  if (m_constraints.back()){
    --m_nconstr;
    delete m_constraints.back();        // delete constraint
  }
  BaseClass::pop_back();
  m_constraints.pop_back();
}
string PatternValues::toStringLong() const
{
  string str;
  str << '(' << toString();
  if (m_nconstr==0) return str << " nil)";
  str << ' ';
  vector<Constraint *>::const_iterator cur_c=m_constraints.begin();
  char delim='(';
  for (uint i=m_constraints.size();i;--i,++cur_c){
    if (*cur_c){
      str << delim << (*cur_c)->str();
      delim=' ';
    }
  }
  return str << "))";
}

////////////////////////////////////////////////////////////////////////////////
// CPatternType::ConstraintHash methods
//
/*
  When merging cells in patterns one must know how many of them are actually
  different and how many of them are mergeable.For example:
  (b ...) U (f ...) U ((* f) ...) U (b ...) is ok, since of the cells {b,f,(* f),b}
  there are only two different and one mergeable.
  There allowed to be at most one non mergeable cell.
*/
CPatternType::ConstraintHash::ConstraintHash(int ndims):
  BaseClass(ndims),m_nmergeable(0){}
void CPatternType::ConstraintHash::merge(const hCCell& cell)
{
  if (!m_first) m_first=cell;
  bool is_m = cell->is_mergeable();
  pair<iterator,bool> res = insert(value_type(cell->node(),is_m));
  if (is_m){
    if (!m_first->is_mergeable()) m_first=cell;
    if (res.second) ++m_nmergeable;
    else if (!res.first->second){
      res.first->second = true;
      ++m_nmergeable;
    }
  }
}
/*
  A constraint must be returned if the cell is actually a merger of different
  cells.
*/
Constraint *CPatternType::ConstraintHash::constraint() const
{
  if (size() < 2) return NULL;
  Constraint *c = new Constraint(size());
  for (const_iterator it=begin();it!=end();++it) c->push_back(it->first);
  return c;
}

////////////////////////////////////////////////////////////////////////////////
// CPatternType::LocalDimMap methods
//
CPatternType::LocalDimMap::LocalDimMap(int ndims):m_ndims(ndims){}
CPatternType::LocalDimMap::~LocalDimMap()
{
  for (iterator it=begin();it!=end();++it) delete it->second;
}
void CPatternType::LocalDimMap::insert(CFDNode *node,int dim)
{
  int key = node->group_id();
  iterator found = find(key);
  if (found == end()){
    vector<bool> *node_set = new vector<bool>(m_ndims,false);
    (*node_set)[dim]=true;
    BaseClass::insert(value_type(key,node_set));
  }else (*found->second)[dim]=true;
}

////////////////////////////////////////////////////////////////////////////////
// CPatternType::GlobalDimMap methods
//
/*
  Another crucial requirement for pattern unification result is that there be two
  identical cells in it if and only if there is an original pattern having these same cells.
  For example (b a) U ((* a) (* f)) could yield the pattern ((* a) (* f)) with the
  constraints ((b a) (a f)), but because of the constraint (a f) the result is actually
  ((* a) (* a)), which is forbidden, since no original pattern has 'a' twice.
  The unification (b a) U ((* a) (* f)) U (a a), however, is ok.
  To enforce this restraint a map of all the cells in all the patterns first created.
  There is an entry for every cell and it contains the multiset of dimensions where this
  cell is present (all the patterns are numbered from 0 to n-1, where the cells from the
  pattern i are said to be present in i's dimension).In our example we have three patterns
  {(b a),((* a) (* f))} and the map will look like this:
                      [a,{0,1}],[b,{0}],[f,{1}]
  (The multiset of dimensions is implemented using an array int[number of dimensions]
   with the index being the dimension in question and the value telling how many times
   the cell is present in this dimension.So the multiset {0,1,2,2} is represented by
   the array {1,1,2})
  To unify the patterns {(b a),((* a) (* f))} I have to merge the cells b(0) and a(1) first,
  then I update the global map.I get the map [a,{0}],[f,{1}] and see that 'a' is left in the dimension
  0 but I used 'a' of the dimension 1,which means that the unification result will have two 'a' which
  come from different dimensions, which is violation of the above restraint.If I add the pattern (a,a)
  I will have:
   - the global map before the merger: [a,{0,1,2,2}],[b,{0}],[f,{1}]
   - merging b(0),a(1,2)
   - the global map after the merger: [a,{0,2}],[f,{1}]
  I used 'a' from the dimension 2 and there is still an 'a' from that same dimension, so that's
  ok, no violation is imminent.
*/
CPatternType::GlobalDimMap::GlobalDimMap():m_ndims(0){}
void CPatternType::GlobalDimMap::init(PatternList *plist,int ndims)
{
	m_ndims=ndims;
	PatternList::const_iterator cur_pat = plist->begin();
	for (int i=0;i<m_ndims;++i,++cur_pat)
	{
		PatternValues::const_iterator cur_val = (*cur_pat)->begin();
		for (uint j=(*cur_pat)->size();j;--j,++cur_val)
		{
			if ((*cur_val)->is_dots()) continue;
			insert((*cur_val)->node(),i);
		}
	}
}
void CPatternType::GlobalDimMap::reset()
{
	while (size())
	{
		iterator itFirst = begin();
		delete[] itFirst->second;
		erase(itFirst);
	}
	m_ndims = 0;
}
void CPatternType::GlobalDimMap::insert(CFDNode *node,int dim)
{
  int key = node->group_id();
  iterator found = find(key);
  if (found == end()){
    int *node_set = new int[m_ndims];
    memset(node_set,0,sizeof(int)*m_ndims);
    node_set[dim]=1;
    BaseClass::insert(value_type(key,node_set));
  }else (found->second)[dim]++;
}
/*
  Updates the global map after the merger.Gets the update map as an argument.
  Update map is a map of dimension sets (not multisets) keyed by cells.
  This function decides whether we will violate the described restraint.
  Returns false if the violation is unavoidable, else returns true.
*/  
bool CPatternType::GlobalDimMap::update(CPatternType::LocalDimMap& upd_map)
{
  for (LocalDimMap::iterator it=upd_map.begin();it!=upd_map.end();++it){
    int *cur_set = find(it->first)->second;
    vector<bool>& upd_set = *it->second;
    int left=0;                        
    int mset_size=0;                   
    for (int dim=0;dim<m_ndims;++dim){ 
      if (upd_set[dim]){               
        --(cur_set[dim]);              
        left+=cur_set[dim];
      }
      mset_size+=cur_set[dim];
    }
    if (!left && mset_size){
      restore(upd_map.begin(),++it);
      return false;
    }
  }
  return true;
}
/*
  Undoes the deeds of the update() method.Get the range in the update map.
*/  
void CPatternType::GlobalDimMap::restore(CPatternType::LocalDimMap::iterator begin,
                                         CPatternType::LocalDimMap::iterator end)
{
  for (CPatternType::LocalDimMap::iterator it=begin;it!=end;++it){
    int *cur_set = find(it->first)->second;
    vector<bool>& rst_set = *it->second;
    for (int dim=0;dim<m_ndims;++dim) if (rst_set[dim]) ++(cur_set[dim]);
  }
}

////////////////////////////////////////////////////////////////////////////////
// CPatternType::UStartFrame methods
//
CPatternType::UStartFrame::UStartFrame(cl_iterator *itlist,PatternList *plist,int ndims):
  start(0),prev_end(false)
{
  for (int i=0;i<ndims;++i)
    if (itlist[i]!=(*plist)[i]->end() && (*itlist[i])->is_dots()) dots.push_back(i);
  ndots=dots.size();
  n = (1<<ndots)-1;
  if (ndots && ndots == ndims) ++start;
}

////////////////////////////////////////////////////////////////////////////////
// CPatternType::UContFrame methods
//
CPatternType::UContFrame::UContFrame(int ndims):
  save(ndims,false),local_dim(ndims){}

////////////////////////////////////////////////////////////////////////////////
// CPatternType methods
//
// Creates a pattern type from the pattern object received from the parser.Should never fail.
CPatternType::CPatternType(CFDNode *owner,const hCPattern& pattern):
  m_owner(owner),m_mergeable(pattern->is_mergeable())
{
  F_TRACE1(pattern);

  fail = 0;
  bool ok = init(new PatternList(pattern2cells(pattern)),0);
  assert(ok);
}
// Called when one has to unify a pattern (as received from the parser)
// with an already existing pattern type.If all is ok a new pattern type
// is returned which is the unification of the parser given pattern and
// the pattern type.The pattern list is shared by both pattern types.
CPatternType::CPatternType(CPatternType *ptype,const hCPattern& pattern):
  m_owner(ptype->m_owner),m_mergeable(ptype->m_mergeable || pattern->is_mergeable())
{
  F_TRACE2(ptype,pattern);
  
  // cerr << "!  Found pattern: " << ptype << endl;
  // cerr << "! Adding pattern: " << pattern << endl;
  fail = 0;
  assert(ptype->sane());
  ptype->m_plist->push_back(pattern2cells(pattern));
  if (!init(ptype->m_plist,1)) fail = new FDAddPatternFail(ptype,pattern);
}
CPatternType::CPatternType(CPatternType *ptype1,const hCPatternType& ptype2):
  m_owner(ptype1->m_owner),m_mergeable(ptype1->m_mergeable || ptype2->m_mergeable)
{
  F_TRACE2(ptype1,ptype2);
  
  fail = 0;
  assert(ptype1->sane() && ptype2->sane());
  ptype1->extend(ptype2);
  if (!init(ptype1->m_plist,ptype2->m_ndims)) fail = new FDUnifyPatternsFail(ptype1,ptype2);
}
void CPatternType::extend(const hCPatternType& ptype)
{
  F_TRACE1(ptype);

  m_plist->reserve(m_ndims + ptype->m_ndims);
  PatternList::const_iterator it = ptype->m_plist->begin();
  for (int i=ptype->m_ndims;i;--i,++it) m_plist->push_back(*it);
}
CPatternType::CPatternType(CFDNode *owner,const CPatternType& r):m_owner(owner),m_mergeable(r.m_mergeable)
{
  F_TRACE1(r);

  fail = 0;
  assert(r.sane());
  CExtFD *hostfd = dynamic_cast<CExtFD *>(owner->fd());
  hCellList cells(new CellList(r.m_cur.size()));        // will hold the copies of the r pattern value cells
  PatternValues::const_iterator src_it = r.m_cur.begin();   // iterate on the r pattern value
  for (uint i=r.m_cur.size();i;--i,++src_it){           // for each r pattern value cell (source cell)
    CFDNode *dst_node = 0;
    if (CFDNode *src_node = (*src_it)->node()){         // get the source cell node
      CPath *path = Node2Path::get_rel_path(r.m_owner,src_node,false);    // get its path relative to the r owner
      dst_node = hostfd->SoftPeekPath(path);          // get the destination node in the hostfd (this is the respective copy)
    }
    cells->push_back((*src_it)->clone(dst_node));       // store the copy of the source cell
  }
  bool ok = init(new PatternList(cells),0);             // Initialize the pattern
  assert(ok);
}
CPatternType::~CPatternType()
{
  F_TRACE0();
  
  delete[] m_itlist;
  while (m_ext--) m_plist->pop_back();
}
ret::utype_t CPatternType::create(CFDNode *owner,const hCPattern& pattern)
{
  F_TRACE1(pattern);

  hCType res_type(new CPatternType(owner,pattern));
  if (fail) return fail;
  return res_type;
}
ret::utype_t CPatternType::special_unify(const hCPattern& pattern)
{
  F_TRACE1(pattern);
  
  DBG_TRACE3(Tracer_l3::special_unify(this,pattern.pointee()));
  hCType res_type(new CPatternType(const_cast<CPatternType *>(this),pattern));
  if (fail) return fail;
  return res_type;
}
ret::utype_t CPatternType::unify_with_special(const hCType& type,CFDNodeBase *,CFDNodeBase *)
{
  F_TRACE1(type);
  
  if (type->nil()) return ret::utype_t(this);
  CPatternType *ptype = cast<CPatternType>(type);
  if (!ptype) return new SSFail(this,type);
  DBG_TRACE3(Tracer_l3::special_unify(this,ptype));
  hCType res_type(new CPatternType(const_cast<CPatternType *>(this),ptype));
  if (fail) return fail;
  return res_type;
}
// Initializes a pattern type from the supplied pattern list.
// Might fail, since it is actually unifying the patterns in the list to get the
// first result.If there is none - the patterns are inconsistent and the constructor
// fails.The pattern list is likely to be shared across many pattern types, so special
// care is taken of the memory management.
bool CPatternType::init(const hPatternList& plist,int ext)
{
  F_TRACE0();

  m_ndims=plist->size();
  m_ext=ext;
  m_plist=plist;
  m_itlist=new cl_iterator[m_ndims];
  m_dim_tbl.init(plist,m_ndims);

  for (int i=0;i<m_ndims;++i) m_itlist[i]=(*plist)[i]->begin();
  return advance();
}
void CPatternType::cleanup()
{
  // First create a fresh new cell list from the pattern values
  hCellList cells(new CellList(m_cur.size()));
  PatternValues::const_iterator pv_it = m_cur.begin();
  for (uint i=m_cur.size();i;--i,++pv_it) cells->push_back(*pv_it);

  // Next remove the trash (unification history, etc...)
  delete[] m_itlist;
  m_cur.erase(m_cur.begin(),m_cur.end());
  m_dim_tbl.reset();
  m_ustart_stack.flush();
  m_ucont_stack.flush();
  
  // Next reInitialize the pattern with the just created cell list
  bool ok = init(new PatternList(cells),0);
  assert(ok);
}
/*
  Transforms a parser given pattern to a list of cells which is what
  pattern lists consist of.
*/
hCellList CPatternType::pattern2cells(const hCPattern& pattern)
{
  F_TRACE1(pattern);
  
  static hCCell dots_cell(new CDotsCell);
 
  hCellList cells(new CellList(pattern->size()));
  CPattern::const_iterator pat_it = pattern->begin();
  CExtFD *hostfd = dynamic_cast<CExtFD *>(m_owner->fd());
  assert(hostfd);
  for (uint i=pattern->size();i;--i,++pat_it)
    if (obj2dots(*pat_it)) cells->push_back(dots_cell);
    else if (obj2mergeable_addr(*pat_it)){
      CAddr *addr = cast<CMergeableAddr>(*pat_it)->addr();
      cells->push_back(new CMergeNodeCell(hostfd->SoftPeekAddr(addr)));
    }else{
      CAddr *addr = cast<CAddr>(*pat_it);
      cells->push_back(new CNodeCell(hostfd->SoftPeekAddr(addr)));
    }
  return cells;
}
Fail *CPatternType::add_self(CExtFD *fd) const
{
  static hCDots dots(new CDots);
  CPattern pattern;   // no need to define a dynamic object, since no one is going to keep it, but a handle
  pattern.addReference();  // just prevent the handle of AddSpecial() from destroying it
  pattern.reserve(m_cur.size());
  hCObject obj;
  PatternValues::const_iterator pv_it = m_cur.begin();
  for (uint i=m_cur.size();i;--i,++pv_it){
    if (CFDNode *node = (*pv_it)->node()){
      CPath *path = Node2Path::get_rel_path(m_owner,node,false);    // do not fix the relative path
      if ((*pv_it)->is_mergeable()) obj = new CMergeableAddr(new CPath(*path));
      else obj = new CPath(*path);
    }else obj = dots;
    pattern.push_back(obj);
  }
  return fd->AddSpecial(Predefined::pattern,hCPattern(&pattern));
}
/*
  The actual pattern unification algorithm.
  Consists of two parts.
  First part chooses what cells are to be merged in every step.
  Second part does the merger.
*/
bool CPatternType::advance()
{
  F_TRACE0();
  
  if (!m_ustart_stack.empty()) goto __next;
__unify_start_ent:
  m_ustart_stack.push(m_us_f=new UStartFrame(m_itlist,m_plist,m_ndims));
  if (m_us_f->start==1) m_cur.push_dots(*m_itlist[m_us_f->dots.front()]);

  for (;m_us_f->n >= m_us_f->start;--(m_us_f->n)){
    int i;
    for (i=0;i<m_us_f->ndots;++i)
      if (is_bit_set(m_us_f->n,i)) ++(m_itlist[m_us_f->dots[m_us_f->ndots-1-i]]);
    bool ok;
    if (is_end(ok)){
      if (!m_us_f->prev_end){
        m_us_f->prev_end=true;
        return true;
      }
    }else if (ok){
      goto __unify_cont_ent;
    __unify_cont_ret:        
      m_us_f = m_ustart_stack.top();
    }
  __next:      
    for (i=0;i<m_us_f->ndots;++i)
      if (is_bit_set(m_us_f->n,i)) --(m_itlist[m_us_f->dots[m_us_f->ndots-1-i]]);
  }
  
  if (m_us_f->start==1) m_cur.pop_back();
  m_ustart_stack.pop();
  
  if (m_ustart_stack.empty()) return false;
  goto __unify_start_ret;
__unify_cont_ent:
  
  m_ucont_stack.push(m_uc_f=new UContFrame(m_ndims));
  {
    ConstraintHash res(m_ndims);        // result of unifying pattern cells currently being pointed at

    // cerr << "[ " << flush;
    for (int dim=0;dim<m_ndims;++dim){  // iterate on the patterns
      assert(m_itlist[dim]!=(*m_plist)[dim]->end());
      hCCell& cell = *m_itlist[dim];
      if (!cell->is_dots()){
        // cerr << '(' << cell << ',' << cell->node()->group_id() << ',' << cell->node()->data().GetType() << ')' << flush;
        m_uc_f->local_dim.insert(cell->node(),dim);
        res.merge(cell);
        m_uc_f->save[dim]=true;
      }
    }
    // cerr << " ]\n";
    if (!res.is_consistent() || !m_dim_tbl.update(m_uc_f->local_dim)){
      m_ucont_stack.pop();
      goto __unify_cont_ret;
    }
    m_cur.push_back(res.cell(),res.constraint());
  }
  uint dim;
  for (dim=0;dim<m_uc_f->save.size();++dim)
    if (m_uc_f->save[dim]) ++(m_itlist[dim]);
  
  goto __unify_start_ent;
__unify_start_ret:
  
  m_uc_f = m_ucont_stack.top();
  for (dim=0;dim<m_uc_f->save.size();++dim)
    if (m_uc_f->save[dim]) --(m_itlist[dim]);
  m_cur.pop_back();
  m_dim_tbl.restore(m_uc_f->local_dim);
  m_ucont_stack.pop();
  goto __unify_cont_ret;
}
bool CPatternType::is_end(bool& ok)
{
  int end_count=0,end2_count=0;
  for (int i=0;i<m_ndims;++i)
    if (m_itlist[i]==(*m_plist)[i]->end()) ++end_count;
    else if ((*m_itlist[i])->is_dots() && m_itlist[i]+1==(*m_plist)[i]->end()) ++end2_count;
  if (end_count + end2_count == m_ndims) return true;
  ok = (end_count==0);
  return false;
}
string CPatternType::toString() const
{
  assert(m_owner);
  s_from = m_owner;
  return m_cur.toString();
}
string CPatternType::toStringLong() const
{
  assert(m_owner);
  s_from = m_owner;
  return m_cur.toStringLong();
}
// Here I have to compare the node references in different host fds.
// I could have compared absolute paths, but there are may be more
// than one absolute path to a node.So I do this:
//  - get one absolute path to a node from the source component
//  - get the node at this path in the destination fd
//  - compare that node to the current destination node
bool CPatternType::is_equal(CType *type) const
{
  CPatternType *ptype = cast<CPatternType>(type);
  if (!ptype) return false;
  assert(m_ext == 0 && m_ndims == int(m_plist->size()) &&
         ptype->m_ext == 0 && ptype->m_ndims == int(ptype->m_plist->size()));
  CFDNode *dstroot = ptype->m_owner->fd()->root();
  PatternValues::const_iterator src_it = m_cur.begin();
  PatternValues::const_iterator dst_it = ptype->m_cur.begin();
  for (uint i=m_cur.size();i;--i,++src_it,++dst_it){
    if (typeid(**src_it) != typeid(**dst_it)) return false;     // make sure the cells are of the same type
    if ((*src_it)->is_dots()) continue;                         // proceed to the next pair of cells if at dots
    CPath *srcpath = Node2Path::get_abs_path((*src_it)->node());// get an absolute path to the current source node
    CFDNode *checknode = dstroot->FindDescendant(srcpath->begin(),srcpath->end(),true);  // get the check node in <dstfd>
    if (!checknode || !checknode->is_equiv((*dst_it)->node()))  // if the check node and the current dst node differ -
      return false;                                             // fail
  }
  return true;
}

////////////////////////////////////////////////////////////////////////////////
// CConstraintIterator methods
//
CConstraintIterator::CConstraintIterator(PatternValues& pv):
  m_constraints(pv.m_constraints),m_cur(m_constraints.begin())
{
  next();
}
void CConstraintIterator::next()
{
  while (!(is_past_the_end() || *m_cur)) ++m_cur;
}

////////////////////////////////////////////////////////////////////////////////
// CPatternIterator methods
//
CPatternIterator::CPatternIterator(CPatternType *ptype):
  m_pv(ptype->m_cur),m_it(m_pv.begin())
{
  next();
}
void CPatternIterator::next()
{
  for (;!is_past_the_end();++m_it)
    if (!(*m_it)->is_dots()){
      m_cur=dynamic_cast<CNodeCell *>(m_it->pointee());
      return;
    }
  m_cur=0;
}

////////////////////////////////////////////////////////////////////////////////
// FDPatternFail methods
//
FDPatternFail::FDPatternFail()
{
  assert(BKClass::fail_bk() == 0);              // assert that the failure log is clean
  BKClass::add_fail_bk(Predefined::pattern);  // acknowledge the failed edge
}

////////////////////////////////////////////////////////////////////////////////
// toString() methods of the Fail family of classes
//
string FDAddPatternFail::GetDesc() const
{
  string str("Patterns ");
  return str << ptype << " and " << pattern << " are inconsistent!";
}
string FDUnifyPatternsFail::GetDesc() const
{
  string str("Patterns ");
  return str << ptype1 << " and " << ptype2 << " are inconsistent!";
}
