#ifndef _COMPILER_PATTERN_
#define _COMPILER_PATTERN_

#include "b_pattern.hxx"
#include "c_cell.hxx"
#include "c_special.hxx"

typedef Handle<CCell> hCCell;

struct CellList:public CMyObject,public vector<hCCell>
{
	explicit CellList(int capacity=0);
	string toString() const;
};

typedef CellList::iterator cl_iterator;

typedef Handle<CellList> hCellList;

struct PatternList:public CMyObject,public vector<hCellList>
{
	explicit PatternList(const hCellList& first);
	string toString() const             { return ""; }
};

/*
  A constraint is a list of nodes to be unified in order for some particular
  pattern cell to be valid.There are any constraints at all only if there are
  mergeable cells.
*/
class Constraint:public vector<CFDNode *>
{
public:
	explicit Constraint(int capacity=0);
	string str() const;
};

/*
  The actual pattern.Consists of two components:
   - the list of cells (dots,normal and mergeable)
   - the list of constraints needed by this pattern
*/
class PatternValues:public CellList
{
	typedef CellList BaseClass;
	vector<Constraint *> m_constraints;
	int m_nconstr;
	friend class CConstraintIterator;
public:
	explicit PatternValues(int capacity=0);
	~PatternValues();
	void push_dots(const hCCell& dots);
	void push_back(const hCCell& cell,Constraint *c=0);
	void pop_back();
	string toStringLong() const;
};

/*
  Data stack template.
  The original algorithm employed two mutually recursive functions.
  After converting it to the respective iterative one all the local data of each one
  of the two functions was moved to the corresponding data stack.
*/
template <class Frame>
struct FrameStack:public stack<Frame *>
{
	typedef stack<Frame *> BaseClass;
	~FrameStack() 
	{
		flush(); 
	}
	void flush()  
	{
		while (BaseClass::size()) pop(); 
	}
	void pop()
	{
		delete BaseClass::top();
		BaseClass::pop();
	}
};

/*
  These are required by CPatternType::ConstraintHash.
*/
struct nodehash
{
	size_t operator()(CFDNode *node) const { return node->group_id(); }
};
struct eqnodes
{
	bool operator()(CFDNode *node1,CFDNode *node2) const 
	{
		return node1->is_equiv(node2);
	}
};

class CConstraintIterator;

/*
  The pattern type itself, the one which gets stored in a node's data.
  Public constructor accepts pattern as received from the parser.
  This pattern starts the private pattern list.
  Private constructor (gets called when method CExtFD::AddSpecial() or
  CExtFD::AddPath() is invoked) accepts a list of patterns to be unified.
  The same list of patterns may be shared accross multiple pattern type instances.
  This is to save space and avoid copying the same lists.
*/
class CPatternType:public CSpecial,public CType
{
	typedef Handle<PatternList> hPatternList;
	friend class CPatternIterator;
public:
	class ConstraintHash:public hash_map<CFDNode *,bool,nodehash,eqnodes>
	{
		typedef hash_map<CFDNode *,bool,nodehash,eqnodes> BaseClass;
		int m_nmergeable;
		hCCell m_first;
	public:
		explicit ConstraintHash(int ndims);
		void merge(const hCCell& cell);
		bool is_consistent() const  { return size() - m_nmergeable < 2; }
		hCCell cell() const         { return m_first; }
		Constraint *constraint() const;
	};
	class LocalDimMap:public hash_map<int,vector<bool> *>
	{
		typedef hash_map<int,vector<bool> *> BaseClass;
		int m_ndims;
	public:
		explicit LocalDimMap(int ndims);
		~LocalDimMap();
		void insert(CFDNode *node,int dim);
	};
	class GlobalDimMap:public hash_map<int,int *>
	{
		typedef hash_map<int,int *> BaseClass;
		int m_ndims;
	public:
		GlobalDimMap();
		~GlobalDimMap(){ reset(); }
		void init(PatternList *plist,int ndims);
		void reset();
		bool update(LocalDimMap& upd_map);
		void restore(LocalDimMap& rst_map){ restore(rst_map.begin(),rst_map.end()); }
	private:
		void insert(CFDNode *node,int dim);
		void restore(LocalDimMap::iterator begin,LocalDimMap::iterator end);
	};
	struct UStartFrame
	{
		vector<int> dots;
		int ndots;
		int start;
		int n;
		bool prev_end;
		UStartFrame(cl_iterator *itlist,PatternList *plist,int dims);
	};
	struct UContFrame
	{
		vector<bool> save;
		LocalDimMap local_dim;
		explicit UContFrame(int ndims);
	};
private:
	int m_ndims;
	int m_ext;
	hPatternList m_plist;
	cl_iterator *m_itlist;
	PatternValues m_cur;
	GlobalDimMap m_dim_tbl;
	FrameStack<UStartFrame> m_ustart_stack;
	FrameStack<UContFrame> m_ucont_stack;
	UStartFrame *m_us_f;
	UContFrame *m_uc_f;
	CFDNode *m_owner;
	bool m_mergeable;
public:
	static ret::utype_t create(CFDNode *owner,const hCPattern& pattern);
	ret::utype_t special_unify(const hCPattern& pattern);
	ret::utype_t unify_with_special(const hCType&,CFDNodeBase *,CFDNodeBase *);
	void cleanup();
	bool advance();
	CConstraintIterator constraint_iterator();
	string toString() const;
	string toStringLong() const;
	const PatternValues& values() const   { return m_cur; }
	bool is_mergeable() const             { return m_mergeable; }
	hCType clone(CFDNode *owner) const    { return new CPatternType(owner,*this); }
	Fail *add_self(CExtFD *fd) const;
	bool is_equal(CType *type) const;
	token_t feature() const               { return Predefined::pattern; }
private:
	CPatternType(CFDNode *owner,const CPatternType&);
	CPatternType(CFDNode *owner,const hCPattern& pattern);
	CPatternType(CPatternType *ptype,const hCPattern& pattern);
	CPatternType(CPatternType *ptype1,const hCPatternType& ptype2);
	~CPatternType();
	bool init(const hPatternList& plist,int ext);
	void extend(const hCPatternType& ptype);
	hCellList pattern2cells(const hCPattern& pattern);
	bool is_end(bool& ok);
	bool sane() const { return m_ndims == int(m_plist->size()); }
};

class CConstraintIterator
{
	typedef vector<Constraint *> Container;
	typedef Container::iterator iterator;
public:
	typedef Constraint& reference;
	typedef Constraint *pointer;
private:
	Container& m_constraints;
	iterator m_cur;
public:
	explicit CConstraintIterator(PatternValues& pv);
	void advance()                { ++m_cur;next(); }
	bool is_past_the_end() const  { return m_cur==m_constraints.end(); }
	reference operator*() const   { return **m_cur; }
	pointer operator->() const    { return *m_cur; }
	operator pointer() const      { return *m_cur; }
private:
	void next();
};

class CPatternIterator
{
	typedef PatternValues::iterator iterator;
public:
	typedef CNodeCell& reference;
	typedef CNodeCell *pointer;
private:
	PatternValues& m_pv;
	iterator m_it;
	CNodeCell *m_cur;
public:
	explicit CPatternIterator(CPatternType *ptype);
	void advance()                { ++m_it;next(); }
	bool is_past_the_end() const  { return m_it==m_pv.end(); }
	reference operator*() const   { return *m_cur; }
	pointer operator->() const    { return m_cur; }
	operator pointer() const      { return m_cur; }
private:
	void next();
};

inline CConstraintIterator CPatternType::constraint_iterator(){ return CConstraintIterator(m_cur); }

class FDPatternFail:public Fail
{
public:
	FDPatternFail();
};

inline CPatternType *obj2pattern_type(CMyObject *obj) { return cast<CPatternType>(obj); }

#endif
