#include "stdcfuf.h"
#include "cfuf.hxx"
#include "cfuf_fd.hxx"
#include "cfuf_grammar.hxx"
#include "ctrlc.hxx"

#include "p_special.hxx"
#include "p_pattern.hxx"
#include "p_cset.hxx"
#include "p_fset.hxx"
#include "p_control.hxx"
#include "c_fditerator.hxx"
#include "c_fdpprint.hxx"
#include "c_fdnode2path.hxx"
#include "c_special.hxx"
#include "u_unify.hxx"
#include "u_fdmanip.hxx"
#include "u_fd.hxx"
#include "u_gr.hxx"

#include <fstream.h>

#ifdef WIN32
using std::ofstream;
#define GET_FUNC_ADDR(func) func
#else
#define GET_FUNC_ADDR(func) &func
#endif

uint CFUFFD::s_nUCount = 0;
////////////////////////////////////////////////////////////////////////////////
// CFUFFD methods
//
ufd_t CFUFFD::Parse(CInputStream& inputStream)
{
  g_pTheLexer->reset();
  g_pTheInput = &inputStream;
  CFD *pFD = new CFD;
  
  try{
    CShieldCtrlC shield_ctrlc;
    ParseFD::set_fd(pFD);       // attach the new fd to the parser
    ///////////////////////////////////////////////////////////////////////////////
    // Register the fset,cset and pattern features.
    // Ignore the tpattern and control features silently (just skip them).
    SpecialFeatureParser<FsetParser,CFset,CFD> fset_parser;             // create fset parser
    SpecialFeatureParser<CsetParser,CCset,CFD> cset_parser;             // create cset parser
    SpecialFeatureParser<PatternParser,CPattern,CFD> pattern_parser;    // create pattern parser 
    SpecialFeatureParser<ControlParser,CControl,CFD> control_parser;    // create control parser 
    ParseFD::register_feature(Predefined::fset,&fset_parser);           // register the fset feature
    ParseFD::register_feature(Predefined::cset,&cset_parser);           // register the cset feature
    ParseFD::register_feature(Predefined::pattern,&pattern_parser);     // register the pattern feature
    ParseFD::register_feature(Predefined::control,&control_parser);     // register the control feature
    FeatureSkipper skip;                                                // create feature skipper
    ParseFD::register_feature(Predefined::tpattern,&skip);              // register the tpattern feature (skip)
    
    ParseFD::unregister_feature(Predefined::alt);
    ParseFD::unregister_feature(Predefined::ralt);
    ParseFD::unregister_feature(Predefined::opt);
    ///////////////////////////////////////////////////////////////////////////////
    Fail *f = ParseFD::parse(); // parse the fd
    assert(pFD->root());        // make sure the root exists, even if parsing fails
    // There are failures, which depend on FD.
    // So the FD cannot be deleted even if parsing fails.
    return ufd_t(new CFUFFD(pFD->root()),f);
  }catch(Fail *e){
    delete pFD;         // can safely delete fd now, since fatal errors do not depend on fd
    return e;           // an exceptional condition (like Ctrl+C or invalid input)
  }
}
fd_t CFUFFD::MakeEmpty()
{
  CFD *pFD = new CFD;
  pFD->Initialize();
  pFD->Realize();
  return new CFUFFD(pFD->root());
}
void CFUFFD::Destroy()
{
  {
    CDisableCtrlC disable_ctrlc;
    delete this;
  }
}
bool CFUFFD::IsEmpty()
{
  for (CFDGraphIterator it(m_pNode);!it.is_past_the_end();it.advance()){
    CType *type = it->data().GetType();
    assert(type && !type->given());
    if (type->final()) return false;
  }
  return true;
}
token_t CFUFFD::GetSpecialAttr()
{
  CType *type = m_pNode->data().GetType();
  assert(type && !type->given() && !type->nil());
  CSpecial *special = dynamic_cast<CSpecial *>(type);
  return special ? special->feature() : 0;
}
Fail *CFUFFD::Unparse(CUnparseFD& unparser)
{
  Fail *f;
  try{
    CShieldCtrlC shield_ctrlc;
    CFDWalk walker(m_pNode);
    f = walker.walk(&unparser);
  }catch(Fail *e){ f = e; }
  
  return f;
}
Fail *CFUFFD::Unparse(CUnparseFD& unparser,const EdgeList& lstIgnored)
{
  Fail *f;
  try{
    CShieldCtrlC shield_ctrlc;
    CFDWalk walker(m_pNode);
    EdgeList::const_iterator it = lstIgnored.begin();
    for (uint i = lstIgnored.size();i;--i,++it) walker.ignore(*it);
    f = walker.walk(&unparser);
  }catch(Fail *e){ f = e; }
  
  return f;
}
ret::ubool_t CFUFFD::Unify(grammar_t gr)
{
	try
	{
		CShieldCtrlC shield_ctrlc;
		CUnifyEngine u;                             // create the unifier object
		ret::ubool_t b = u.exec(m_path,m_pNode,gr->GetGrammar()); // actually unify
		assert(!b.fail);
		s_nUCount = u.ucount();                     // export the count of visited branches
		return b;                                   // return either #t or #f depending on the unification outcome
	}
	catch(Fail *e)
	{
		return e; 
	}
}
void CFUFFD::PrettyPrint(char *szFDName,char *szFileName)
{
	CDisableCtrlC disable_ctrlc;
	ofstream outfile(szFileName);
	outfile << "(define " << szFDName << " '";
	CPrettyPrintFD pprinter(m_pNode);
	pprinter.display(true,outfile,10 + strlen(szFDName),false);
	outfile << ")\n";
}
CType *CFUFFD::GetType()
{
	CType *type = m_pNode->data().GetType();
	assert(type && !type->given() && !type->nil());
	return type;
}
fd_t CFUFFD::FindChild(token_t edge)
{
  return new CFUFFD(m_pNode->FindChild(edge),m_path,edge,true);
}
fd_t CFUFFD::FindDescendant(const EdgeList& path)
{
  return new CFUFFD(m_pNode->FindDescendant(path.begin(),path.end()),m_path,path);
}
pair<token_t,fd_t> CFUFFD::GetFirstChild()
{
  return GetRelative(GET_FUNC_ADDR(CFUFFD::ProtGetFirstChild),true);
}
pair<token_t,fd_t> CFUFFD::GetNextChild()
{
  return GetRelative(GET_FUNC_ADDR(CFUFFD::ProtGetNextChild),true);
}
pair<token_t,fd_t> CFUFFD::GetFirstParent()
{
  return GetRelative(GET_FUNC_ADDR(CFUFFD::ProtGetFirstParent),false);
}
pair<token_t,fd_t> CFUFFD::GetNextParent()
{
  return GetRelative(GET_FUNC_ADDR(CFUFFD::ProtGetNextParent),false);
}
fd_t CFUFFD::Copy()
{
  CDisableCtrlC disable_ctrlc;
  return new CFUFFD(FDManip::copy_subfd(m_pNode)->root());
}
void CFUFFD::Delete()
{
  CDisableCtrlC disable_ctrlc;
  FDManip::delete_subfd(m_pNode);
}
fd_t CFUFFD::Cut()
{
  CDisableCtrlC disable_ctrlc;
  CFD *copyFD = FDManip::copy_subfd(m_pNode);
  FDManip::delete_subfd(m_pNode);
  return new CFUFFD(copyFD->root());
}
Fail *CFUFFD::Paste(fd_t srcFD)
{
  Fail *f;
  try{
    CShieldCtrlC shield_ctrlc;
    f = FDManip::paste_subfd(m_pNode,m_path,srcFD->GetRoot());
  }catch(Fail *e){ f = e; }
  return f;
}
bool CFUFFD::IsEqual(fd_t fd)
{
  CDisableCtrlC disable_ctrlc;
  return FDCompare::IsEqual(m_pNode,m_path,fd->GetRoot(),fd->GetPath());
}
////////////////////////////////////////////////////////////////////////////////
// Create an empty fd if pNode is NULL or has the type of nil
bool CFUFFD::InitEmpty(CFDNode *pNode,bool bMergeable)
{
  m_pNode = pNode;
  m_bMergeable = bMergeable;
  assert(!m_pNode || GetFD()->extra);

  if (m_pNode && !m_pNode->data().GetType()->nil()) return false;
  
  CFD *fd = new CFD;
  fd->Initialize();
  fd->Realize();
  m_pNode = fd->root();
  GetFD()->extra = new CExtra(1);
  return true;
}
// Create an independent FD (no previous references exist) if pNodePath is NULL
CFUFFD::CFUFFD(CFDNode *pNode,CPath *pNodePath,bool bMergeable)
{
  if (!pNodePath){      // creating an independent FD
    m_pNode = pNode;
    m_bMergeable = bMergeable;
    assert(m_pNode && GetFD()->extra == 0 && !m_pNode->data().GetType()->nil());
    GetFD()->extra = new CExtra(1);
  }else if (!InitEmpty(pNode,bMergeable)){
    m_path.insert(m_path.end(),pNodePath->begin(),pNodePath->end());
    GetFDExtra()->AddRef();
  }
}
// Create another reference to an existing FD (through the node pNodeChild, being down by pathChild)
CFUFFD::CFUFFD(CFDNode *pNodeChild,const EdgeList& pathMaster,const EdgeList& pathChild)
{
  if (!InitEmpty(pNodeChild)){
    GetFDExtra()->AddRef();
    m_path.reserve(pathMaster.size() + pathChild.size());
    m_path.insert(m_path.end(),pathMaster.begin(),pathMaster.end());
    m_path.insert(m_path.end(),pathChild.begin(),pathChild.end());
  }
}
// Create another reference to an existing FD (through the node pNode, being down (bChild) or up (!bChild) by edge)
CFUFFD::CFUFFD(CFDNode *pNode,const EdgeList& pathMaster,token_t edge,bool bChild)
{
  if (!InitEmpty(pNode)){
    GetFDExtra()->AddRef();
    if (bChild){
      m_path.reserve(pathMaster.size() + 1);
      m_path.insert(m_path.end(),pathMaster.begin(),pathMaster.end());
      m_path.push_back(edge);
    }else{
      assert(pathMaster.size());
      if (edge == pathMaster.back())
        m_path.insert(m_path.end(),pathMaster.begin(),pathMaster.end() - 1);
      else{
        CPath *pNodePath = Node2Path::get_abs_path(pNode);
        m_path.insert(m_path.end(),pNodePath->begin(),pNodePath->end());
      }
    }
  }
}
CFUFFD::~CFUFFD()
{
  CExtra *pExtra = GetFDExtra();
  pExtra->RemRef();             // decrement the FD reference count
  if (pExtra->GetRef() == 0)    // if the last reference to the fd is removed
    delete GetFD();             // delete the whole fd
}
pair<token_t,fd_t> CFUFFD::GetRelative(CFUFFD::GetRelativeFunc f,bool bChild)
{
  pair<CArc_c *,CFDNode *> relative;
  (this->*f)(relative);
  if (!relative.first) return make_pair((token_t)0,new CFUFFD(0,m_path,0,bChild));
  return make_pair(relative.first->title(),new CFUFFD(relative.second,m_path,relative.first->title(),bChild));
}
void CFUFFD::ProtGetFirstChild(pair<CArc_c *,CFDNode *>& ch)
{
  m_itChild.reset(m_pNode);
  ProtGetNextChild(ch);
}
void CFUFFD::ProtGetNextChild(pair<CArc_c *,CFDNode *>& ch)
{
	ch.first = 0;
	ch.second = 0;
	if (!m_itChild.is_past_the_end())
	{
		ch.first = m_itChild->first;
		ch.second = m_itChild->second;
		m_itChild.advance();
	}
}
void CFUFFD::ProtGetFirstParent(pair<CArc_c *,CFDNode *>& par)
{
  m_itParent.reset(m_pNode);
  ProtGetNextParent(par);
}
void CFUFFD::ProtGetNextParent(pair<CArc_c *,CFDNode *>& par)
{
  par.first = 0;
  par.second = 0;
  if (!m_itParent.is_past_the_end()){
    par = *m_itParent;
    m_itParent.advance();
  }
}
