#include "stdcfuf.h"

// #define NO_INDEX
// #define NO_BKCLASS
// #define F_TRACE
// #define NO_WAIT

#include "b_debug_tracer.hxx"
#include "u_btrv.hxx"
#include "u_fd.hxx"
#include "c_patnode.hxx"
#include "b_bkclass.hxx"
#include "t_tracer_l2.hxx"

/////////////////////////////////////////////////////////////////////////////////
// [NOTE]
// Before the proceed method succeeds to find the matching branch the stacks look:
//  clock stack: ...........[previous successful unif id]
// branch stack: ...........[cur-branch and related info on the stack about to become previous]
// i.e. the two stacks are of the same size.
// If the matching branch is found (proceed has succeeded), the new id is selected and the stacks look:
//  clock stack: ...........[previous successful unif id][*cur-unif-id* is on top]
// branch stack: ...........[previous successful  branch][cur-branch and related info is in variables, NOT on the stack]
// i.e. the lengths of the two stacks differ by one with the id stack being the longest.
// (*) At failure the current id is popped from the stack, thus equalizing the two stacks.
// The right brother of the cur-branch is tried.If exists it is made the current branch, a new id
// is selected and pushed back on the stack, thus we return to the above scheme.
// Note that the branch stack is still unchanged.
// If a right brother does not exist (meaning the alt has axhausted all its branches and must be
// retreated from), the id is popped and the top of the branch stack is popped and made current:
//  clock stack: ...........[previous successful unif id]
// branch stack: ...........[previous successful branch info is in variables, NOT on the stack]
//                                     |
//                                     V
//  clock stack: ..........[prev][*cur-unif-id* is on top]
// branch stack: ..........[prev][cur-branch and related info is in variables, NOT on the stack]
// Now the new current branch must be retried (since choosing it causes failure in the next alt, where we have just been).
// This means repeating the step (*).
// If when retreating the branch stack becomes the size it had when we entered this constituent, i.e.
// all the alts failed, (we have no previous alt to go) then report failure of the whole constituent.

// A once frozen alt can leave the freezer only if it is undone through retreat() function.
// If its wait condition is met the alt remains in the freezer, but in a melted state.

#ifndef NO_WAIT
#include "u_freezer.hxx"
#define WAIT_ONLY(smth) smth
#else
#define WAIT_ONLY(smth)
#endif

/////////////////////////////////////////////////////////////////////////////////
// CBranchTraversor methods
//
CBranchTraversor::CBranchTraversor(CFD *fd,CGrNode *top_branch):
  m_fd(fd),m_top_alt(top_branch->parent()),m_cur_inode(0)
{
  F_TRACE0();

#ifndef NO_WAIT
  if (m_top_alt){
    m_fd->go_alt_loc(m_top_alt);
    m_done_stack.push(BranchDone(0,0)); // stack should not be empty, if the first branch fails
    set_branch(0);
  }
#else
  assert(!m_top_alt);
#endif
  DBG_TRACE2(Tracer_l2::init(new BranchInfo(m_cur_branch)));
  set_branch(top_branch);
}
ustatus_t CBranchTraversor::set_branch(CGrNode *branch)
{
  F_TRACE1(branch);
  
  m_cur_branch = branch;
  WAIT_ONLY(if (m_cur_branch))
    {
      g_pTheEnv->m_clock.Proceed();
      DBG_TRACE2(Tracer_l2::accept_b());
    }
  BKClass::reset_fail_bk();
  return OK;
}
/*
  The clock stack is one unit larger than the branch stack.
  All the current information is in the variables (m_cur_branch, etc...)
*/
ustatus_t CBranchTraversor::proceed()
{
  F_TRACE0();

  CGrNode *first_alt = m_cur_branch->first_child();     // enter the first alt inside the current branch
  if (first_alt) return accept(first_alt);              // if the one exists at all

  // The current branch is a leaf (contains no alts)
  // So the current alt is finished and we have to move to the next one,
  // meaning the right brother of the current alt.We move up and retry if
  // none exists, until either an alt is found or the top is reached.
  CGrNode *branch = m_cur_branch;               // this is for tracer needs only
  for (CGrNode *alt=m_cur_branch->parent();alt != m_top_alt;alt = alt->grandparent()){
    assert(alt);
    DBG_TRACE2({ Tracer_l2::success_b(branch);branch=branch->grandparent(); });
    CGrNode *next_alt = alt->right_sibling();   // get the next alt of the branch
    if (next_alt) return accept(next_alt);      // enter it if it exists
  }
  return FINISHED;                              // reached the top, constituent is done
}
/*
 A new alt is being entered, meaning that the current branch information can be pushed onto stack -
 from now on it refers to the previous alt.The clock and branch stacks are thus made equal size.
 The specs of this new alt are acquired and made current in anticipation of success.
 All is left is to peek a branch in this alt, make it current too and push the clock forward.
 (Note that peeking a branch might fail prior trying the branch because of indexing).
*/ 
ustatus_t CBranchTraversor::accept(CGrNode *_alt)
{
  F_TRACE1(_alt);
  
  m_done_stack.push(BranchDone(m_cur_branch,m_cur_inode));  // save the current branch information
  CFDNode *dst = m_fd->go_alt_loc(_alt);        // go to the alt which is about to become the current one
  if (node2pat_alt(_alt))                       // if is this a pattern alt really
    return accept_pattern(node2pat_alt(_alt));  // handle the pattern, else - this is a usual alt
  else{
    CAltNode *alt = node2alt(_alt);
#ifndef NO_WAIT
    if (m_fd->is_wait_satisfied(alt,dst)){      // proceed routinely if the wait condition is satisfied
#endif
      const TokenList *index = alt->params()->index(); // get the alt's index path and if it's not empty then
      m_cur_inode = index->empty() ? NULL : dst->FindDescendant(index->begin(),index->end()); // get the index node
      return go_indexed(alt->first_child());    // pick the first branch matching the index
#ifndef NO_WAIT
    }
    Freezer::toBeFrozen = alt;                  // save the alt to be later frozen
    m_fd->ungo_alt_loc(alt);                    // reposition the FD at the constituent base
    set_branch(0);                              // treat the alt as though it has successfully unified with the grammar
    return FROZEN;                              // but do not return OK
#endif
  }
}
#ifndef NO_WAIT
/*
  Called by the CTM in response to the FROZEN status returned by proceed().
  The alt which wait condition was not satisfied is inserted into the freezer.
  All the subsequent child alts are skipped.
*/
ustatus_t CBranchTraversor::freeze()
{
  CGrNode *prev_alt = Freezer::toBeFrozen;
  Freezer::toBeFrozen = 0;
  while (prev_alt != m_top_alt){
    CGrNode *next_alt = prev_alt->right_sibling();
    if (next_alt) return accept(next_alt);
    prev_alt = prev_alt->grandparent();
  }
  return FINISHED;
}
#endif
/*
 The two stacks are equalized when getting here:
  clock stack: ...........[previous successful unif id]
 branch stack: ...........[previous successful  branch] [cur-branch and related info is in variables, NOT on the stack]
 when called from outside, branch is never #f
*/
ustatus_t CBranchTraversor::go_indexed(CGrNode *branch)
{
  F_TRACE1(branch);
  
#ifndef NO_INDEX
  if (m_cur_inode){
    hCType m_cur_itype(m_cur_inode->data().GetType());
    while (branch && !m_fd->index_matches(node2branch(branch),m_cur_itype)){ // browse the branches while the index mismatches
      m_cur_branch = branch;
      branch = branch->right_sibling();
    }
    if (!branch){
      DBG_TRACE2(Tracer_l2::fail_b());
      m_fd->ungo_alt_loc(m_cur_branch->parent());  // return to the constituent base after the unsuccessful index probing
      return fail_cur_alt();
    }
  }
#endif
  return set_branch(branch);              // a match is found - enter the branch
}
/*
 The two stacks are equalized when getting here:
  clock stack: ...........[previous successful clock tick]
 branch stack: ...........[previous successful  branch] [cur-branch and related info is in variables, NOT on the stack]
 The new alt is actually a pattern to be unified with fd.The outcome of pattern unification is
 a stream of possible patterns which are tried one at a time until an appropriate one is met.
 Pattern alt has one branch which iterates over the possible patterns, becoming respectively
 each one of them.The side effect of this is that CPatBranchNode::right_sibling() is actually a
 mutator - modifies and returns the same pattern branch.
*/
ustatus_t CBranchTraversor::accept_pattern(CPatAltNode *pat_alt)
{
  F_TRACE1(pat_alt);
  
  ret::utype_t t = m_fd->AddPattern(pat_alt->pattern()); // add and unify the pattern with fd
  if (t.fail){
    DBG_TRACE2(Tracer_l2::report_failure(t.fail));
    delete t.fail;
    return fail_cur_alt();              // fail the pattern alt
  }
  hCPatternType ptype = cast_h2h<CPatternType>(t.val);
  CPatBranchNode *pat_branch=node2pat_branch(pat_alt->first_child()); // get the pattern branch
  assert(pat_branch && bool(ptype));
  pat_branch->set_pattern(ptype);     // Initialize the pattern branch
  return set_branch(pat_branch);      // set the pattern branch as the current and enter it
}
ustatus_t CBranchTraversor::retreat()
{
  F_TRACE0();

  WAIT_ONLY(if (m_cur_branch))
    g_pTheEnv->m_clock.Retreat();
  if (m_done_stack.empty()) return FAILED;
  return try_brother();
}
/*
  The two stacks are equalized when gets here.
  The information about the current alt and branch are in variables (NOT on the stack).
  The right brother must be inspected and if ok made current.
*/
ustatus_t CBranchTraversor::try_brother()
{
  F_TRACE0();

#ifndef NO_WAIT
  if (m_cur_branch == 0){                       // this corresponds to a frozen alt
    Freezer::UnfreezeLast();
    return fail_cur_alt();                      // fail the unfreezed alt
  }
#endif
#ifndef NO_BKCLASS
  if (uint64 fail_bk = BKClass::fail_bk()){     // if the failed edges fall into some bkclass category
    CPatBranchNode *pat_branch=node2pat_branch(m_cur_branch); // continue if the current branch is normal 
    if (!pat_branch || !pat_branch->is_mergeable()){          // or a pattern, but not a mergeable one
      uint64 alt_bk = pat_branch ? 0 : node2alt(m_cur_branch->parent())->params()->bkclasses(); 
      if ((alt_bk & fail_bk) == 0){             // if the current alt bkclasses does not match any of the fail bkclasses
        DBG_TRACE2(Tracer_l2::bkprune_b());     // report that the branch was BK pruned
        if (pat_branch) pat_branch->failed();
        return fail_cur_alt();                  // fail the current alt
      }
    }
  }
#endif
  DBG_TRACE2(Tracer_l2::fail_b());
  CGrNode *next_branch=m_cur_branch->right_sibling();   // try the next branch (this one is a failure)
  if (!next_branch) return fail_cur_alt();      // if there is none - fail the whole alt
  m_fd->go_alt_loc(next_branch->parent());
  if (node2pat_branch(next_branch))             // else if the branch is a pattern - try it
    return set_branch(next_branch);
  return go_indexed(next_branch);               // else - browse the branches left for the matching index
}
/*
  The two stacks are equalized when gets here.
  Retreats from the current alt to the previous one (meaning the topmost on the stack).
  When read-only pruning is enabled, the read-only region is skipped if
  the nature of the failure is persistent (not because of something that the fd lacks,
  but because of what it has - given vs normal failures).
  persistent-fail describes the nature of the failure and MUST be set by the caller.
*/
ustatus_t CBranchTraversor::fail_cur_alt()
{
  F_TRACE0();

  prev2cur();       // retrieve the previous information and make it the current one
  if (m_done_stack.empty()) return FAILED;
  return try_brother();
}
/////////////////////////////////////////////////////////////////////////////////
// Called only when the current alt has to be retreated from.
// The stacks are equalized before and after the call:
//  clock stack: ...........[previous successful unif id]  <-- popped and returned
// branch stack: ...........[previous successful  branch]  <-- popped and made current
// The top on the branch stack always corresponds to the previous alt.
// Popping it and making current is in essence retreating to this alt.
void CBranchTraversor::prev2cur()
{
  F_TRACE0();
  
  BranchDone prev = m_done_stack.top();
  m_done_stack.pop();
  m_cur_branch = prev.branch;
  m_cur_inode = prev.inode;
  WAIT_ONLY(if (m_cur_branch))
    g_pTheEnv->m_clock.Retreat();
}
/*
  Called by the constituent traversor when the constituent this branch
  traversor belongs to is retried again.
  Refreshes the tracer.
*/
void CBranchTraversor::wake_up()
{
  F_TRACE0();

  DBG_TRACE2(Tracer_l2::init(new BranchInfo(m_cur_branch)));
}
void CBranchTraversor::ungo_loc() const
{
  m_fd->ungo_alt_loc(m_cur_branch->parent());
}
