#include "stdcfuf.h"

// #define F_TRACE
// #define CMD_TRACE_ON

#include "b_debug_tracer.hxx"
#include "b_pattern.hxx"
#include "u_fd.hxx"
#include "c_fditerator.hxx"
#include "c_pattern.hxx"
#include "c_fdwalk.hxx"
#include "u_fdmanip.hxx"

////////////////////////////////////////////////////////////////////////////////
// The actual creator of a subfd copy.To speed up things it does not use the
// usual fd creation methods, since they are too heavy for the purpose.
class CDoCopy:public CUnparseFD{
public:
  CDoCopy(CFD *dst_fd,CFDNode *src_root);
  Fail *StartInternal(CFDNode *root);
  Fail *StartEndLeaf(CFDNode *root){ return StartInternal(root); }
  Fail *EnterSubgraph(token_t,CFDNode *,EdgeList *);
  Fail *LeaveSubgraph();
  Fail *AddLeaf(token_t,CFDNode *,EdgeList *);
  Fail *AddPath(token_t,CFDNode *,EdgeList *);
private:
  CFD *m_dst_fd;
};

////////////////////////////////////////////////////////////////////////////////
// Pastes the source onto the destination subfd.Here I do use the usual fd creation
// procedure, since this is actually unification of the source and destination subfds.
class CDoPaste:public CUnparseFD{
public:
  CDoPaste(CFD *fd):m_fd(fd){}
  Fail *StartInternal(CFDNode *root);
  Fail *StartEndLeaf(CFDNode *root){ return StartInternal(root); }
  Fail *EnterSubgraph(token_t,CFDNode *,EdgeList *);
  Fail *LeaveSubgraph();
  Fail *AddLeaf(token_t,CFDNode *,EdgeList *);
  Fail *AddPath(token_t,CFDNode *,EdgeList *);
private:
  CFD *m_fd;
};

////////////////////////////////////////////////////////////////////////////////
// FDManip methods
//
// Copies all the nodes below <node>.Of all the special types only patterns get copied.
// They are collected as soon as met and after the copy is done their node references are 
// resolved.
CFD *FDManip::copy_subfd(CFDNode *src_root)
{
  CFDWalk walker(src_root);             // will traverse all of the graph (including specials) in no specific order
  CFD *dst_fd = new CFD;                // create the host for the copied nodes
  CDoCopy copier(dst_fd,src_root);      // create the copier
  Fail *f = walker.walk(&copier);       // copy the graph below src_root recursively into the dst_fd
  assert(!f);
  return dst_fd;
}
void FDManip::delete_subfd(CFDNode *node)
{
  vector<CFDNode *> nodes;
  nodes.reserve(100);
  // First collect all the nodes to be deleted.
  for (CFDGraphIterator it(node);!it.is_past_the_end();it.advance())
    nodes.push_back(it);
  // Next cancel the collected nodes.They are set to have the type NIL
  // (unlike the default NONE for unification in progress).
  while (nodes.size()){
    nodes.back()->cancel_node(g_pTheEnv->nil());
    nodes.pop_back();
  }
}

struct PreparePaste{
  PreparePaste(CFD *fd,CFDNode *node):m_fdpos(fd)
  {
    m_fd = fd;
    m_fd->set_base(node);
  }
  ~PreparePaste(){ m_fd->set_base(m_fd->root()); }
private:
  CMarkFDPos m_fdpos;   // mark the current fd position
  CFD *m_fd;
};

Fail *FDManip::paste_subfd(CFDNode *dst_node,const EdgeList& dst_elist,CFDNode *src_node)
{
  CFD *dst_fd = dynamic_cast<CFD *>(dst_node->fd());    // get the destination fd
  PreparePaste prepare(dst_fd,dst_node);                // ensure the consistent state even if an exception is thrown
  
  Fail *f = 0;
  EdgeList::const_iterator it = dst_elist.begin();      // Descent down the edge list
  for (uint i=dst_elist.size();i && !f;--i,++it)
    f = dst_fd->EnterSubgraph(*it);
  if (f) return f;
  
  CFDWalk walker(src_node);             // traverse all of the graph (including specials) in no specific order
  CDoPaste paster(dst_fd);              // create the paster
  return walker.walk(&paster);          // unify the fd below the src_node recursively with the dst_fd
}

////////////////////////////////////////////////////////////////////////////////
// CDoPaste methods
//
Fail *CDoPaste::StartInternal(CFDNode *root)
{
  m_fd->top_node()->data().SetType(root->data().GetType());
  return 0;
}
Fail *CDoPaste::EnterSubgraph(token_t edge,CFDNode *node,EdgeList *)
{
  F_TRACE1(edge);
  
  if (Fail *f = m_fd->EnterSubgraph(edge)) return f;
  hCType src_type = node->data().GetType();
  if (src_type->any()) m_fd->top_node()->data().SetType(src_type);
  return 0;
}
Fail *CDoPaste::LeaveSubgraph()
{
  F_TRACE0();
  
  m_fd->LeaveSubgraph();
  return 0;
}
Fail *CDoPaste::AddLeaf(token_t edge,CFDNode *orig,EdgeList *)
{
  F_TRACE1(edge);

  CGroupData& gr_data = orig->data();
  hCType type = gr_data.GetType();     // get the node type
  if (type->special()){
    CSpecial *special = dynamic_cast<CSpecial *>(type.pointee());
    return special->add_self(m_fd);
  }
  return m_fd->AddLeaf(edge,type);      // else just add the leaf
}
Fail *CDoPaste::AddPath(token_t edge,CFDNode *,EdgeList *path)
{
  F_TRACE2(edge,path);

  return m_fd->AddPath(edge,new CPath(CPath::ABS,*path));
}

////////////////////////////////////////////////////////////////////////////////
// CDoCopy methods
//
CDoCopy::CDoCopy(CFD *dst_fd,CFDNode *src_root):m_dst_fd(dst_fd)
{
  m_dst_fd->Initialize();         // properly Initialize the destination fd
  m_dst_fd->Realize();
  src_root->data().SetHook(m_dst_fd->root()); // attach the root of dst_fd to the src_root
}
Fail *CDoCopy::StartInternal(CFDNode *src_root)
{
  m_dst_fd->root()->data().SetType(src_root->data().GetType());
  return 0;
}
Fail *CDoCopy::EnterSubgraph(token_t edge,CFDNode *src_node,EdgeList *)
{
  CGroupData& src_gdata = src_node->data();
  CFDNode *dst_node = m_dst_fd->top_node()->AddChild(edge,src_gdata.GetType());
  src_gdata.SetHook(dst_node);        // "attach" the new node to the source node
  m_dst_fd->push_node(dst_node);        // push the new node on the node stack of the fd being built
  // !!! THIS IS VERY IMPORTANT TO ENSURE THAT CFUF:COPY-FD DOES NOT CHANGE THE RELATIVE ORDER OF CONSTITUENTS
  dst_node->set_tstamp(src_node->tstamp()); // set the time stamp of the new node to be that of the source node (for constituent selection)
  return 0;
}
Fail *CDoCopy::LeaveSubgraph()
{
  m_dst_fd->pop_node();
  return 0;
}
Fail *CDoCopy::AddLeaf(token_t edge,CFDNode *src_node,EdgeList *)
{
  CGroupData& src_gdata = src_node->data();
  hCType type = src_gdata.GetType();
  CFDNode *dst_node = m_dst_fd->top_node()->AddChild(edge,type);
  if (type->special()){
    CSpecial *special = dynamic_cast<CSpecial *>(type.pointee());
    dst_node->data().SetType(special->clone(dst_node));
  }
  src_gdata.SetHook(dst_node);
  // !!! THIS IS VERY IMPORTANT TO ENSURE THAT CFUF:COPY-FD DOES NOT CHANGE THE RELATIVE ORDER OF CONSTITUENTS
  dst_node->set_tstamp(src_node->tstamp()); // set the time stamp of the new node to be that of the source node
  return 0;
}
Fail *CDoCopy::AddPath(token_t edge,CFDNode *src_node,EdgeList *)
{
  CGroupData& src_gdata = src_node->data();
  CFDNode *dst_node1 = src_gdata.GetHook();
  CFDNode *dst_node2 = m_dst_fd->top_node()->AddChild(edge,src_gdata.GetType());
  // !!! THIS IS VERY IMPORTANT TO ENSURE THAT CFUF:COPY-FD DOES NOT CHANGE THE RELATIVE ORDER OF CONSTITUENTS
  dst_node2->set_tstamp(src_node->tstamp()); // set the time stamp of the new node to be that of the source node (for constituent selection)
  dst_node2->connect(dst_node1);
  return 0;
}
