#include "stdcfuf.h"
#include "u_freezer.hxx"
#include "u_fd.hxx"
#include "c_altnode.hxx"
#include "t_tracer_l2.hxx"

Freezer::FrozenAlts Freezer::frozenAlts;
Freezer::MeltedAlts Freezer::meltedAlts;
CAltNode *Freezer::toBeFrozen = 0;
////////////////////////////////////////////////////////////////////////////////
// Freezer methods and other related functions
Freezer::Cell::Cell(CAltNode *_alt,CFDNode *node,EdgeList *elist)
{
  alt = _alt;
  c_node = node;
  c_elist = new EdgeList(*elist);
  state = ALT_FROZEN;
}
Freezer::Cell::~Cell()
{
  if (state == ALT_FROZEN)      // if the cell is still frozen (may happen when exceeded the maximum branch count)
    delete c_elist;             // dispose of the path to the constituent base
}
void Freezer::Init()
{
  while (meltedAlts.size()) meltedAlts.pop();
  while (frozenAlts.size()){
    delete frozenAlts.back();
    frozenAlts.pop_back();
  }
  toBeFrozen = 0;
}
void Freezer::Freeze(CFDNode *cnode,EdgeList *elist)
{
  assert(toBeFrozen);
  DBG_TRACE2(Tracer_l2::freeze_b(toBeFrozen));
  frozenAlts.push_back(new Cell(toBeFrozen,cnode,elist));
}
Freezer::Cell *Freezer::GetReady(CFD *fd,bool bForceMelt)
{
  if (frozenAlts.empty()) return 0;

  DBG_TRACE2(Tracer_l2::scanfreezer_b(bForceMelt));
  
  Cell *found = 0;
  fd->EnterSubgraph(-1);                               // position the FD at the graph root
  
  FrozenAlts::iterator it = frozenAlts.begin();
  for (uint i=frozenAlts.size();i && !found;--i,++it){
    if ((*it)->state == ALT_MELTED) continue;           // skip already melted alts
    assert((*it)->state == ALT_FROZEN);                 // make sure the cell is frozen (not melting)
    
    fd->go_constituent_loc((*it)->c_elist);             // descent to the constituent of the frozen alt
    CFDNode *dst = fd->go_alt_loc((*it)->alt);          // position the FD at the frozen alt's location
    if (bForceMelt || fd->is_wait_satisfied((*it)->alt,dst)){   // if this frozen alt's wait is satisfied
      found = *it;                                      // a ready for melting frozen cell is found
      found->state = ALT_MELTING;                       // indicate the melting of the alt
      meltedAlts.push(found);                           // and push the cell on the stack of melted cells
      found->c_node->data().Mark();                     // mark the constituent node
      DBG_TRACE2(Tracer_l2::melt_b(found->c_elist,found->alt));
    }
    fd->ungo_alt_loc((*it)->alt);                       // restore the FD position at the constituent base
    fd->ungo_constituent_loc((*it)->c_elist);           // return to the FD root
  }
  fd->LeaveSubgraph();                                 // return to the original constituent base
  return found;
}
void Freezer::UnmeltLast()
{
  assert(meltedAlts.size() && meltedAlts.top()->state == ALT_MELTING);
  Cell *cell = meltedAlts.top();                        // get the respective melted cell
  meltedAlts.pop();                                     // remove it from the stack of melted
  cell->c_node->data().Unmark();                        // unmark the constituent node
  DBG_TRACE2(Tracer_l2::unmelt_b(cell->alt));
  cell->state = ALT_FROZEN;                             // freeze the alt again
}
void Freezer::UnfreezeLast()
{
  assert(!frozenAlts.empty());
  Cell *cell = frozenAlts.back();                       // get respective frozen cell from the freezer
  frozenAlts.pop_back();                                // and remove it from the freezer
  DBG_TRACE2(Tracer_l2::unfreeze_b(cell->alt));
  delete cell;                                          // delete the frozen cell
}
void Freezer::CommitLastMelted()
{
  assert(meltedAlts.size() && meltedAlts.top()->state == ALT_MELTING);
  meltedAlts.top()->state = ALT_MELTED;
}
void Freezer::UncommitLastMelted()
{
  assert(meltedAlts.size() && meltedAlts.top()->state == ALT_MELTED);
  meltedAlts.top()->state = ALT_MELTING;
}
