#include <stdio.h>
#include <string.h>
#include "lisp.h"

typedef LispVal (*LCBFunc0)();
extern "C" LCBFunc0 lisp_call_address(int index);

struct LispValue
{
  static bool IsList(LispVal lsp_val)   { return GetType(lsp_val) == ListType; }
  static bool IsNil(LispVal lsp_val)    { return GetType(lsp_val) == NilType; }
  static bool IsFixnum(LispVal lsp_val) { return GetType(lsp_val) == FixnumType || GetType(lsp_val) == OddFixnumType; }
  static bool IsBignum(LispVal lsp_val) { return GetType(lsp_val) == BignumType; }
  static bool IsChar(LispVal lsp_val)   { return GetType(lsp_val) == CharType; }
  static bool IsSymbol(LispVal lsp_val) { return GetType(lsp_val) == SymbolType; }
  static bool IsString(LispVal lsp_val) { return GetType(lsp_val) == V_charType; }
  static bool IsVector(LispVal lsp_val) { return GetType(lsp_val) == V_svecType; }
  static bool IsFloat(LispVal lsp_val)  { return GetType(lsp_val) == S_FloatType; }
  static bool IsDouble(LispVal lsp_val) { return GetType(lsp_val) == D_FloatType; }
};
struct LispList
{
  static LispVal GetCar(LispVal lsp_list){ return Car(lsp_list); }
  static LispVal GetCdr(LispVal lsp_list){ return Cdr(lsp_list); }
};
struct LispFixnum
{
  static int GetValue(LispVal lsp_fixnum){ return FixnumToInt(int(lsp_fixnum)); }
};
struct LispBignum
{
  unsigned char code;
  unsigned char unused;
  unsigned short nwords;
  union
  {
    unsigned int integer;
    unsigned short words[1];
  };
  static LispBignum *GetValue(LispVal lsp_bignum){ return (LispBignum *)(lsp_bignum-OtherTypeOfs); }
};
struct LispChar
{
  static char GetValue(LispVal lsp_char){ return ScharToChar(int(lsp_char)); }
};
struct LispString
{
  static char *GetValue(LispVal lsp_string){ return (char *)Vecdata(lsp_string); }
  static int GetSize(LispVal lsp_string)   { return VecSize(lsp_string); }
};
struct LispSymbol
{
  static char *GetValue(LispVal lsp_symbol){ return LispString::GetValue(SymbolName(lsp_symbol)); }
  static bool IsKeyword(LispVal lsp_symbol){
    LispVal lsp_packageName = (LispVal)*((int *)(SymbolPackage(lsp_symbol)-OtherTypeOfs)+3);
    return strcmp(LispString::GetValue(lsp_packageName),"KEYWORD") == 0;
  }
};
struct LispVector
{
  unsigned int code:8;
  unsigned int size:24;
  LispVal data[1];
  static LispVector *GetValue(LispVal lsp_vector){ return (LispVector *)(lsp_vector+sv_size_adj); }
};
struct LispFloat
{
  static double GetValue(LispVal lsp_float){ return S_FtoFloat(lsp_float); }
};
struct LispDouble
{
  static double GetValue(LispVal lsp_double){ return D_FtoFloat(lsp_double); }
};

class CLispType
{
  typedef void (*PrintInfoFunc)(LispVal lsp_val);
  static PrintInfoFunc s_arrInfoFuncs[256];
public:
  CLispType();
  void PrintLispTypeInfo(LispVal lsp_val);
private:
  static void DoFixnum(LispVal lsp_fixnum)
  {
    printf("Passed a fixnum %d\n",LispFixnum::GetValue(lsp_fixnum));
  }
  static void DoList(LispVal lsp_list)
  {
    printf("Passed a list:\n");
    while (GetType(lsp_list) == ListType){
      (*s_arrInfoFuncs[GetType(Car(lsp_list))])(Car(lsp_list));
      lsp_list = Cdr(lsp_list);
    }
    (*s_arrInfoFuncs[GetType(lsp_list)])(lsp_list);
  }
  static void DoNil(LispVal lsp_nil)
  {
    printf("Passed nil\n");
  }
  static void DoChar(LispVal lsp_char)
  {
    printf("Passed a character '%c'\n",ScharToChar(int(lsp_char)));
  }
  static void DoSymbol(LispVal lsp_symbol)
  {
    printf("Passed a symbol '%s%s\n",IsKeyword(lsp_symbol) ? ":" : "",lisp2string(SymbolName(lsp_symbol)));
  }
  static void DoStruct(LispVal lsp_struct)
  {
    printf("Passed a struct\n");
  }
  static void DoFunction(LispVal lsp_function)
  {
    printf("Passed a function\n");
  }
  static void DoFloat(LispVal lsp_float)
  {
    printf("Passed a float %f\n",(float)S_FtoFloat(lsp_float));
  }
  static void DoDouble(LispVal lsp_double)
  {
    printf("Passed a double %f\n",(double)D_FtoFloat(lsp_double));
  }
  static void DoBignum(LispVal lsp_bignum)
  {
    LispBignum *ptr = LispBignum::GetValue(lsp_bignum);
    printf("Passed a bignum %u%s (0x%x)\n",ptr->integer,ptr->nwords > 2 ? "..." : "",ptr->integer);
  }
  static void DoString(LispVal lsp_string)
  {
    printf("Passed a string \"%s\" (length %d)\n",Vecdata(lsp_string),VecSize(lsp_string));
  }
  static void DoVector(LispVal lsp_vector)
  {
    LispVector *ptr = LispVector::GetValue(lsp_vector);
    printf("Passed a vector of length %d\n",ptr->size);
    for (int i = 0;i < ptr->size; ++i)
      (*s_arrInfoFuncs[GetType(ptr->data[i])])(ptr->data[i]);
  }
  static void DoUnknown(LispVal lsp_unknown)
  {
    printf("Passed an unknown (%d)\n",GetType(lsp_unknown));
  }
private:
  static char *lisp2string(LispVal lsp_string)
  {
    assert(GetType(lsp_string) == V_charType);
    return (char *)Vecdata(lsp_string);
  }
  static bool IsKeyword(LispVal lsp_symbol)
  {
    LispVal lsp_packageName = (LispVal)*((int *)(SymbolPackage(lsp_symbol)-OtherTypeOfs)+3);
    return strcmp(lisp2string(lsp_packageName),"KEYWORD") == 0;
  }
};

CLispType::PrintInfoFunc CLispType::s_arrInfoFuncs[256];

CLispType::CLispType()
{
  memset(s_arrInfoFuncs,0,sizeof(s_arrInfoFuncs));
  s_arrInfoFuncs[FixnumType] = s_arrInfoFuncs[OddFixnumType] = DoFixnum;
  s_arrInfoFuncs[ListType] = DoList;
  s_arrInfoFuncs[NilType] = DoNil;
  s_arrInfoFuncs[CharType] = DoChar;
  s_arrInfoFuncs[SymbolType] = DoSymbol;
  s_arrInfoFuncs[FunctionType] = DoFunction;
  s_arrInfoFuncs[S_FloatType] = DoFloat;
  s_arrInfoFuncs[D_FloatType] = DoDouble;
  s_arrInfoFuncs[BignumType] = DoBignum;
  s_arrInfoFuncs[StructType] = DoStruct;
  s_arrInfoFuncs[V_charType] = DoString;
  s_arrInfoFuncs[V_svecType] = DoVector;
}
void CLispType::PrintLispTypeInfo(LispVal lsp_val)
{
  unsigned int type = GetType(lsp_val);
  assert(type < 256);
  if (!s_arrInfoFuncs[type]) s_arrInfoFuncs[type] = DoUnknown;
  (*s_arrInfoFuncs[type])(lsp_val);
  fflush(stdout);
}
