#include "runfuf.hxx"
#include "cfuf.hxx"
#include "lispval.hxx"
#include <stdio.h>
#include <iomanip.h>
#include "lcb.hxx"

typedef void (*PlugInFunc)();
#define PLUG(Module) extern void Plug##Module()
#define BEGIN_MODULE_MAP PlugInFunc g_arrModules[]={
#define MODULE(Module) Plug##Module,
#define END_MODULE_MAP 0};
#include "modules.inl"

extern "C" {
  void CFUF_Final();
  void CFUF_Initialize();
};

void CFUF_Final()
{
  CFUF::Shutdown();
}
void CFUF_Initialize()
{
  CFUF::Initialize(new CLispInterp,true);

  // load cfuf modules
  for (PlugInFunc *fptr = g_arrModules;*fptr;++fptr)
    (*fptr)();
}

LispVal LispValue::FALSE = nilval;
LispVal LispValue::NIL = nilval;
LispVal LispValue::TRUE = (LispVal)IntToFixnum(1);

/////////////////////////////////////////////////////////////////////////////////
// CLispInterp methods
//
bool CLispInterp::EvalControl(const char *const exp)
{
  return !LispValue::IsNil(LCB::EvalStr(exp));
}
CInputStream *CLispInterp::EvalExtern(const char *const exp)
{
  return new CLispInputStream(LCB::EvalStr(exp));
}

/////////////////////////////////////////////////////////////////////////////////
// LispLexerFail methods
//
string LispLexerFail::GetDesc() const
{
  LispVal lsp_str = LCB::MakeInvExprMsg(m_exp);
  assert(LispValue::IsString(lsp_str));
  return LispString::GetValue(lsp_str);
}

/////////////////////////////////////////////////////////////////////////////////
// CLispInputStream methods
//
void CLispInputStream::Reset(LispVal input)
{
  m_ds.erase(m_ds.begin(),m_ds.end());
  m_ret.erase(m_ret.begin(),m_ret.end());
  Init(input);
}
void CLispInputStream::Init(LispVal input)
{
  m_input = input;
  m_buf = &m_dummy;
  m_pos = -1;
  m_str = "";
  m_ds.push(m_input,true,0);
  m_ret.push(0);
  m_ungot = false;
  m_dummy = '\0';
}
int CLispInputStream::Get()
{
  m_ungot = false;
  if (m_buf[++m_pos]){
    return m_buf[m_pos];
  }
  Dismember();
  m_pos = 0;
  if (m_ret.empty()){
    m_ungot = true;
    return EOF;
  }
  m_buf = m_str.c_str();
  return *m_buf;
}
void CLispInputStream::Unget()
{
  if (m_ungot) return;
  --m_pos;
  m_ungot = true;
}

// When making a recursive call there is no application to the enter_code field
// of the stack Frame.It is set to -2 then.
#define REC_CALL(arg_fd,arg_start,ret_code) \
m_ds.push(fd,start,-2,i); \
m_ret.push(ret_code); \
fd = arg_fd;start = arg_start; \
goto __dismember_start; \
__dismember_ret_##ret_code: \
m_ds.pop(fd,start,i)

#define RETURN(enter_code) \
m_ds.push(fd,start,enter_code,i); \
return; \
__dismember_enter_##enter_code:

#define END() \
m_ds.push(fd,start,-1,i); \
return

void CLispInputStream::Dismember() throw (LispLexerFail *)
{
  static char buf[100];
    
  LispVal fd;
  bool start;
  ushort i;
  
  switch (m_ds.pop(fd,start,i)){
  case -1:goto __dismember_end;
  case 1:goto __dismember_enter_1;
  case 2:goto __dismember_enter_2;
  case 3:goto __dismember_enter_3;
  case 4:goto __dismember_enter_4;
  case -2:assert(false);
  }
  
__dismember_start:
  if (LispValue::IsList(fd)){
    if (start){
      m_str = '(';
      RETURN(1);
    }
    REC_CALL(LispList::GetCar(fd),true,1);
    if (!LispValue::IsNil(LispList::GetCdr(fd))){
      m_str = ' ';
      RETURN(2);
      REC_CALL(LispList::GetCdr(fd),false,2);
    }
    if (start){
      m_str = ')';
      END();
    }
  }else if (LispValue::IsFixnum(fd)){
    m_str = MakeStringFromInt(LispFixnum::GetValue(fd));
    END();
  }else if (LispValue::IsChar(fd)){
    m_str = LispChar::GetValue(fd);
    END();
  }else if (LispValue::IsNil(fd)){
    m_str = Predefined::nil;
    END();
  }else if (LispValue::IsSymbol(fd)){
    m_str = LispSymbol::IsKeyword(fd) ? ":" : "";
    m_str += LispSymbol::GetValue(fd);
    END();
  }else if (LispValue::IsString(fd)){
    ((m_str = '"') += LispString::GetValue(fd)) += '"';
    END();
  }else if (LispValue::IsVector(fd)){
    m_str = "#(";
    RETURN(3);
    LispVector *pVec = LispVector::GetValue(fd);
    if (pVec->size){
      for (i=0;i<pVec->size-1;++i){
        REC_CALL(pVec->data[i],true,3);
        m_str = ' ';
        RETURN(4);
        pVec = LispVector::GetValue(fd);
      }
      REC_CALL(pVec->data[i],true,4);
    }
    m_str = ')';
    END();
  }else if (LispValue::IsFloat(fd)){
    sprintf(buf,"%f",LispFloat::GetValue(fd));
    m_str = buf;
    END();
  }else if (LispValue::IsDouble(fd)){
    sprintf(buf,"%f",LispDouble::GetValue(fd));
    m_str = buf;
    END();
  }else throw new LispLexerFail(fd);
  
__dismember_end:
  switch (m_ret.pop()){
  case 1:goto __dismember_ret_1;
  case 2:goto __dismember_ret_2;
  case 3:goto __dismember_ret_3;
  case 4:goto __dismember_ret_4;
  }
  return;
}

/////////////////////////////////////////////////////////////////////
// CLispInputStream::DataStack methods
//
void CLispInputStream::DataStack::push(LispVal fd,bool start,byte enter_code,ushort i)
{
  push_back(Frame(fd,start,enter_code,i));
}
CLispInputStream::byte CLispInputStream::DataStack::pop(LispVal& fd,bool& start,ushort& i)
{
  Frame f(back());
  pop_back();
  fd = f.fd;
  start = f.start;
  i = f.i;
  return f.enter_code;
}

/////////////////////////////////////////////////////////////////////
// CLispInputStream::RetAddrStack methods
//
CLispInputStream::uchar CLispInputStream::RetAddrStack::pop()
{
  uchar tmp = back();
  pop_back();
  return tmp;
}
